@@ -1,9 +1,9 @@
 //
-// "$Id: gl.h,v 1.6.2.4 2001/01/22 15:13:38 easysw Exp $"
+// "$Id: gl.h,v 1.6.2.4.2.5 2004/04/11 04:38:55 easysw Exp $"
 //
 // OpenGL header file for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2004 by Bill Spitzak and others.
 //
 // You must include this instead of GL/gl.h to get the Microsoft
 // APIENTRY stuff included (from <windows.h>) prior to the OpenGL
@@ -30,21 +30,26 @@
 // Please report all bugs and problems to "fltk-bugs@fltk.org".
 //
 
-#ifndef gl_draw_H
-#define gl_draw_H
+#ifndef FL_gl_H
+#  define FL_gl_H
 
-#include "Enumerations.H" // for color names
-#ifdef _WIN32
-# include <windows.h>
-#endif
-#ifndef APIENTRY
-# if defined(__CYGWIN__)
-#  define APIENTRY __attribute__ ((__stdcall__))
-# else
-#  define APIENTRY
-# endif
-#endif
-#include <GL/gl.h>
+#  include "Enumerations.H" // for color names
+#  ifdef WIN32
+#    include <windows.h>
+#  endif
+#  ifndef APIENTRY
+#    if defined(__CYGWIN__)
+#      define APIENTRY __attribute__ ((__stdcall__))
+#    else
+#      define APIENTRY
+#    endif
+#  endif
+
+#  ifdef __APPLE__
+#    include <OpenGL/gl.h>
+#  else
+#    include <GL/gl.h>
+#  endif
 
 FL_EXPORT void gl_start();
 FL_EXPORT void gl_finish();
@@ -73,8 +78,8 @@ FL_EXPORT void gl_measure(const char*, int& x, int& y);
 
 FL_EXPORT void gl_draw_image(const uchar *, int x,int y,int w,int h, int d=3, int ld=0);
 
-#endif
+#endif // !FL_gl_H
 
 //
-// End of "$Id: gl.h,v 1.6.2.4 2001/01/22 15:13:38 easysw Exp $".
+// End of "$Id: gl.h,v 1.6.2.4.2.5 2004/04/11 04:38:55 easysw Exp $".
 //
