@@ -1,5 +1,5 @@
 //
-// "$Id: Fl_Menu_Type.cxx,v 1.16.2.4 1999/08/09 13:53:54 mike Exp $"
+// "$Id: Fl_Menu_Type.cxx,v 1.16.2.12.2.10 2004/11/21 14:53:48 easysw Exp $"
 //
 // Menu item code for the Fast Light Tool Kit (FLTK).
 //
@@ -9,7 +9,7 @@
 // This file also contains code to make Fl_Menu_Button, Fl_Menu_Bar,
 // etc widgets.
 //
-// Copyright 1998-1999 by Bill Spitzak and others.
+// Copyright 1998-2004 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -26,15 +26,16 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@easysw.com".
+// Please report all bugs and problems to "fltk-bugs@fltk.org".
 //
 
 #include <FL/Fl.H>
 #include "Fl_Widget_Type.h"
+#include "alignment_panel.h"
 #include <FL/fl_message.H>
 #include <FL/Fl_Menu_.H>
 #include <FL/Fl_Button.H>
-#include <string.h>
+#include "../src/flstring.h"
 #include <stdio.h>
 #include <stdlib.h>
 
@@ -46,6 +47,11 @@ Fl_Menu_Item menu_item_type_menu[] = {
 
 extern int reading_file;
 extern int force_parent;
+extern int i18n_type;
+extern const char* i18n_include;
+extern const char* i18n_function;
+extern const char* i18n_file;
+extern const char* i18n_set;
 
 static char submenuflag;
 
@@ -86,8 +92,6 @@ Fl_Submenu_Type Fl_Submenu_type;
 ////////////////////////////////////////////////////////////////
 // Writing the C code:
 
-#include <ctype.h>
-
 // test functions in Fl_Widget_Type.C:
 int is_name(const char *c);
 const char *array_name(Fl_Widget_Type *o);
@@ -132,7 +136,7 @@ void Fl_Menu_Item_Type::write_static() {
       while (*d && !is_id(*d)) d++;
     }
     const char* cn = callback_name();
-    const char* k = class_name();
+    const char* k = class_name(1);
     if (k) {
       write_c("\ninline void %s::%s_i(Fl_Menu_*", k, cn);
     } else {
@@ -143,7 +147,14 @@ void Fl_Menu_Item_Type::write_static() {
     write_c(", %s", ut);
     if (use_v) write_c(" v");
     write_c(") {\n  %s", callback());
-    if (*(d-1) != ';') write_c(";");
+    if (*(d-1) != ';') {
+      const char *p = strrchr(callback(), '\n');
+      if (p) p ++;
+      else p = callback();
+      // Only add trailing semicolon if the last line is not a preprocessor
+      // statement...
+      if (*p != '#' && *p) write_c(";");
+    }
     write_c("\n}\n");
     if (k) {
       write_c("void %s::%s(Fl_Menu_* o, %s v) {\n", k, cn, ut);
@@ -162,23 +173,21 @@ void Fl_Menu_Item_Type::write_static() {
   if (next && next->is_menu_item()) return;
   // okay, when we hit last item in the menu we have to write the
   // entire array out:
-  int level;
-  const char* k = class_name();
+  const char* k = class_name(1);
   if (k) {
-    write_c("\nFl_Menu_Item %s::%s[] = {\n", k, menu_name(level));
-  } else
-    write_c("\nFl_Menu_Item %s[] = {\n", menu_name(level));
+    int i; write_c("\nFl_Menu_Item %s::%s[] = {\n", k, menu_name(i));
+  } else {
+    int i; write_c("\nFl_Menu_Item %s[] = {\n", menu_name(i));
+  }
   Fl_Type* t = prev; while (t && t->is_menu_item()) t = t->prev;
-  level = t->level+1;
   for (Fl_Type* q = t->next; q && q->is_menu_item(); q = q->next) {
     ((Fl_Menu_Item_Type*)q)->write_item();
-    if (q->is_parent()) level++;
-    int l1 =
-      (q->next && q->next->is_menu_item()) ? q->next->level : t->next->level;
-    while (level > l1) {write_c(" {0},\n"); level--;}
-    level = l1;
+    int thislevel = q->level; if (q->is_parent()) thislevel++;
+    int nextlevel =
+      (q->next && q->next->is_menu_item()) ? q->next->level : t->level+1;
+    while (thislevel > nextlevel) {write_c(" {0,0,0,0,0,0,0,0,0},\n"); thislevel--;}
   }
-  write_c(" {0}\n};\n");
+  write_c(" {0,0,0,0,0,0,0,0,0}\n};\n");
 
   if (k) {
     // Write menu item variables...
@@ -209,14 +218,31 @@ int Fl_Menu_Item_Type::flags() {
 void Fl_Menu_Item_Type::write_item() {
   write_c(" {");
   if (image) write_c("0");
-  else if (label()) write_cstring(label());
+  else if (label()) {
+    switch (i18n_type) {
+    case 0 : /* None */
+        write_cstring(label());
+        break;
+    case 1 : /* GNU gettext */
+        write_c("%s(", i18n_function);
+        write_cstring(label());
+	write_c(")");
+        break;
+    case 2 : /* POSIX catgets */
+        write_c("catgets(%s,%s,%d,", i18n_file[0] ? i18n_file : "_catalog",
+	        i18n_set, msgnum());
+        write_cstring(label());
+	write_c(")");
+        break;
+    }
+  }
   else write_c("\"\"");
   if (((Fl_Button*)o)->shortcut())
     write_c(", 0x%x, ", ((Fl_Button*)o)->shortcut());
   else
     write_c(", 0, ");
   if (callback()) {
-    const char* k = is_name(callback()) ? 0 : class_name();
+    const char* k = is_name(callback()) ? 0 : class_name(1);
     if (k) {
       write_c(" (Fl_Callback*)%s::%s,", k, callback_name());
     } else {
@@ -234,26 +260,26 @@ void Fl_Menu_Item_Type::write_item() {
 }
 
 void Fl_Menu_Item_Type::write_code1() {
-  int i; const char* name = menu_name(i);
+  int i; const char* mname = menu_name(i);
   if (!prev->is_menu_item()) {
     // for first menu item, declare the array
-    if (class_name())
-      write_h("  static Fl_Menu_Item %s[];\n", name);
+    if (class_name(1))
+      write_h("  static Fl_Menu_Item %s[];\n", mname);
     else
-      write_h("extern Fl_Menu_Item %s[];\n", name);
+      write_h("extern Fl_Menu_Item %s[];\n", mname);
   }
 
   const char *c = array_name(this);
   if (c) {
-    if (class_name()) {
+    if (class_name(1)) {
       write_public(public_);
       write_h("  static Fl_Menu_Item *%s;\n", c);
     } else
-      write_h("#define %s (%s+%d)\n", c, name, i);
+      write_h("#define %s (%s+%d)\n", c, mname, i);
   }
 
   if (callback()) {
-    if (!is_name(callback()) && class_name()) {
+    if (!is_name(callback()) && class_name(1)) {
       const char* cn = callback_name();
       const char* ut = user_data_type() ? user_data_type() : "void*";
       write_public(0);
@@ -264,7 +290,7 @@ void Fl_Menu_Item_Type::write_code1() {
 
   int init = 0;
   if (image) {
-    write_c(" {Fl_Menu_Item* o = &%s[%d];\n", name, i);
+    write_c(" {Fl_Menu_Item* o = &%s[%d];\n", mname, i);
     init = 1;
     image->write_code();
   }
@@ -272,7 +298,7 @@ void Fl_Menu_Item_Type::write_code1() {
     if (extra_code(n) && !isdeclare(extra_code(n))) {
       if (!init) {
 	init = 1;
-	write_c("%s{ Fl_Menu_Item* o = &%s[%d];\n", indent(), name, i);
+	write_c("%s{ Fl_Menu_Item* o = &%s[%d];\n", indent(), mname, i);
       }
       write_c("%s  %s\n", indent(), extra_code(n));
     }
@@ -313,11 +339,14 @@ void Fl_Menu_Type::build_menu() {
     int lvl = level+1;
     for (q = next; q && q->level > level; q = q->next) {
       Fl_Menu_Item_Type* i = (Fl_Menu_Item_Type*)q;
-      m->label(i->o->label());
+      if (i->o->image()) i->o->image()->label(m);
+      else {
+        m->label(i->o->label() ? i->o->label() : "(nolabel)");
+        m->labeltype(i->o->labeltype());
+      }
       m->shortcut(((Fl_Button*)(i->o))->shortcut());
       m->callback(0,(void*)i);
       m->flags = i->flags();
-      m->labeltype(i->o->labeltype());
       m->labelfont(i->o->labelfont());
       m->labelsize(i->o->labelsize());
       m->labelcolor(i->o->labelcolor());
@@ -391,8 +420,8 @@ Fl_Menu_Bar_Type Fl_Menu_Bar_type;
 #include <FL/fl_draw.H>
 
 void Shortcut_Button::draw() {
-  if (value()) draw_box(FL_THIN_DOWN_BOX, (Fl_Color)9);
-  else draw_box(FL_THIN_UP_BOX, FL_WHITE);
+  if (value()) draw_box(FL_DOWN_BOX, (Fl_Color)9);
+  else draw_box(FL_UP_BOX, FL_WHITE);
   fl_font(FL_HELVETICA,14); fl_color(FL_BLACK);
   fl_draw(fl_shortcut_label(svalue),x()+6,y(),w(),h(),FL_ALIGN_LEFT);
 }
@@ -408,7 +437,7 @@ int Shortcut_Button::handle(int e) {
       v = Fl::event_state()&(FL_META|FL_ALT|FL_CTRL|FL_SHIFT) | Fl::event_key();
       if (v == FL_BackSpace && svalue) v = 0;
     }
-    if (v != svalue) {svalue = v; set_changed(); redraw();}
+    if (v != svalue) {svalue = v; set_changed(); redraw(); do_callback(); }
     return 1;
   } else if (e == FL_UNFOCUS) {
     int c = changed(); value(0); if (c) set_changed();
@@ -439,5 +468,5 @@ void shortcut_in_cb(Shortcut_Button* i, void* v) {
 }
 
 //
-// End of "$Id: Fl_Menu_Type.cxx,v 1.16.2.4 1999/08/09 13:53:54 mike Exp $".
+// End of "$Id: Fl_Menu_Type.cxx,v 1.16.2.12.2.10 2004/11/21 14:53:48 easysw Exp $".
 //
