@@ -1,5 +1,5 @@
 //
-// "$Id: Fl_Type.cxx,v 1.6.2.6 2001/01/22 15:13:38 easysw Exp $"
+// "$Id: Fl_Type.cxx,v 1.6.2.6.2.19 2004/05/13 21:02:39 easysw Exp $"
 //
 // Widget type code for the Fast Light Tool Kit (FLTK).
 //
@@ -15,7 +15,7 @@
 // not in the linked list and are not written to files or
 // copied or otherwise examined.
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2004 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -38,13 +38,121 @@
 #include <FL/Fl.H>
 #include <FL/Fl_Browser_.H>
 #include <FL/fl_draw.H>
-#include <ctype.h>
 #include <stdlib.h>
-#include <string.h>
+#include "../src/flstring.h"
 #include <stdio.h>
 
 #include "Fl_Type.h"
 
+#include <FL/Fl_Pixmap.H>
+#include "pixmaps/lock.xpm"
+//#include "pixmaps/unlock.xpm"
+
+static Fl_Pixmap	lock_pixmap(lock_xpm);
+//static Fl_Pixmap	unlock_pixmap(unlock_xpm);
+
+#include "pixmaps/flWindow.xpm"
+#include "pixmaps/flButton.xpm"
+#include "pixmaps/flCheckButton.xpm"
+#include "pixmaps/flRoundButton.xpm"
+#include "pixmaps/flBox.xpm"
+#include "pixmaps/flGroup.xpm"
+#include "pixmaps/flFunction.xpm"
+#include "pixmaps/flCode.xpm"
+#include "pixmaps/flCodeBlock.xpm"
+#include "pixmaps/flDeclaration.xpm"
+#include "pixmaps/flDeclarationBlock.xpm"
+#include "pixmaps/flClass.xpm"
+#include "pixmaps/flTabs.xpm"
+#include "pixmaps/flInput.xpm"
+#include "pixmaps/flChoice.xpm"
+#include "pixmaps/flMenuitem.xpm"
+#include "pixmaps/flMenubar.xpm"
+#include "pixmaps/flSubmenu.xpm"
+#include "pixmaps/flScroll.xpm"
+#include "pixmaps/flTile.xpm"
+#include "pixmaps/flWizard.xpm"
+#include "pixmaps/flPack.xpm"
+#include "pixmaps/flReturnButton.xpm"
+#include "pixmaps/flLightButton.xpm"
+#include "pixmaps/flRepeatButton.xpm"
+#include "pixmaps/flMenuButton.xpm"
+#include "pixmaps/flOutput.xpm"
+#include "pixmaps/flTextDisplay.xpm"
+#include "pixmaps/flTextEdit.xpm"
+#include "pixmaps/flFileInput.xpm"
+#include "pixmaps/flBrowser.xpm"
+#include "pixmaps/flCheckBrowser.xpm"
+#include "pixmaps/flFileBrowser.xpm"
+#include "pixmaps/flClock.xpm"
+#include "pixmaps/flHelp.xpm"
+#include "pixmaps/flProgress.xpm"
+#include "pixmaps/flSlider.xpm"
+#include "pixmaps/flScrollBar.xpm"
+#include "pixmaps/flValueSlider.xpm"
+#include "pixmaps/flAdjuster.xpm"
+#include "pixmaps/flCounter.xpm"
+#include "pixmaps/flDial.xpm"
+#include "pixmaps/flRoller.xpm"
+#include "pixmaps/flValueInput.xpm"
+#include "pixmaps/flValueOutput.xpm"
+
+static Fl_Pixmap	window_pixmap(flWindow_xpm);
+static Fl_Pixmap	button_pixmap(flButton_xpm);
+static Fl_Pixmap	checkbutton_pixmap(flCheckButton_xpm);
+static Fl_Pixmap	roundbutton_pixmap(flRoundButton_xpm);
+static Fl_Pixmap	box_pixmap(flBox_xpm);
+static Fl_Pixmap	group_pixmap(flGroup_xpm);
+static Fl_Pixmap	function_pixmap(flFunction_xpm);
+static Fl_Pixmap	code_pixmap(flCode_xpm);
+static Fl_Pixmap	codeblock_pixmap(flCodeBlock_xpm);
+static Fl_Pixmap	declaration_pixmap(flDeclaration_xpm);
+static Fl_Pixmap	declarationblock_pixmap(flDeclarationBlock_xpm);
+static Fl_Pixmap	class_pixmap(flClass_xpm);
+static Fl_Pixmap	tabs_pixmap(flTabs_xpm);
+static Fl_Pixmap	input_pixmap(flInput_xpm);
+static Fl_Pixmap	choice_pixmap(flChoice_xpm);
+static Fl_Pixmap	menuitem_pixmap(flMenuitem_xpm);
+static Fl_Pixmap	menubar_pixmap(flMenubar_xpm);
+static Fl_Pixmap	submenu_pixmap(flSubmenu_xpm);
+static Fl_Pixmap	scroll_pixmap(flScroll_xpm);
+static Fl_Pixmap	tile_pixmap(flTile_xpm);
+static Fl_Pixmap	wizard_pixmap(flWizard_xpm);
+static Fl_Pixmap	pack_pixmap(flPack_xpm);
+static Fl_Pixmap	returnbutton_pixmap(flReturnButton_xpm);
+static Fl_Pixmap	lightbutton_pixmap(flLightButton_xpm);
+static Fl_Pixmap	repeatbutton_pixmap(flRepeatButton_xpm);
+static Fl_Pixmap	menubutton_pixmap(flMenuButton_xpm);
+static Fl_Pixmap	output_pixmap(flOutput_xpm);
+static Fl_Pixmap	textdisplay_pixmap(flTextDisplay_xpm);
+static Fl_Pixmap	textedit_pixmap(flTextEdit_xpm);
+static Fl_Pixmap	fileinput_pixmap(flFileInput_xpm);
+static Fl_Pixmap	browser_pixmap(flBrowser_xpm);
+static Fl_Pixmap	checkbrowser_pixmap(flCheckBrowser_xpm);
+static Fl_Pixmap	filebrowser_pixmap(flFileBrowser_xpm);
+static Fl_Pixmap	clock_pixmap(flClock_xpm);
+static Fl_Pixmap	help_pixmap(flHelp_xpm);
+static Fl_Pixmap	progress_pixmap(flProgress_xpm);
+static Fl_Pixmap	slider_pixmap(flSlider_xpm);
+static Fl_Pixmap	scrollbar_pixmap(flScrollBar_xpm);
+static Fl_Pixmap	valueslider_pixmap(flValueSlider_xpm);
+static Fl_Pixmap	adjuster_pixmap(flAdjuster_xpm);
+static Fl_Pixmap	counter_pixmap(flCounter_xpm);
+static Fl_Pixmap	dial_pixmap(flDial_xpm);
+static Fl_Pixmap	roller_pixmap(flRoller_xpm);
+static Fl_Pixmap	valueinput_pixmap(flValueInput_xpm);
+static Fl_Pixmap	valueoutput_pixmap(flValueOutput_xpm);
+
+Fl_Pixmap *pixmap[] = { 0, &window_pixmap, &button_pixmap, &checkbutton_pixmap, &roundbutton_pixmap, /* 0..4 */
+ &box_pixmap, &group_pixmap, &function_pixmap, &code_pixmap, &codeblock_pixmap, &declaration_pixmap, /* 5..10 */ 
+ &declarationblock_pixmap, &class_pixmap, &tabs_pixmap, &input_pixmap, &choice_pixmap,               /* 11..15 */
+ &menuitem_pixmap, &menubar_pixmap, &submenu_pixmap, &scroll_pixmap, &tile_pixmap, &wizard_pixmap,   /* 16..21 */
+ &pack_pixmap, &returnbutton_pixmap, &lightbutton_pixmap, &repeatbutton_pixmap, &menubutton_pixmap,  /* 22..26 */
+ &output_pixmap, &textdisplay_pixmap, &textedit_pixmap, &fileinput_pixmap, &browser_pixmap,          /* 27..32 */
+ &checkbrowser_pixmap, &filebrowser_pixmap, &clock_pixmap, &help_pixmap, &progress_pixmap,	     /* 33..36 */
+ &slider_pixmap, &scrollbar_pixmap, &valueslider_pixmap, &adjuster_pixmap, &counter_pixmap,          /* 37..41 */
+ &dial_pixmap, &roller_pixmap, &valueinput_pixmap, &valueoutput_pixmap, /* 42..45*/ };
+
 ////////////////////////////////////////////////////////////////
 
 class Widget_Browser : public Fl_Browser_ {
@@ -94,8 +202,8 @@ static void Widget_Browser_callback(Fl_Widget *o,void *) {
   ((Widget_Browser *)o)->callback();
 }
 
-Widget_Browser::Widget_Browser(int x,int y,int w,int h,const char*l)
-: Fl_Browser_(x,y,w,h,l) {
+Widget_Browser::Widget_Browser(int X,int Y,int W,int H,const char*l)
+: Fl_Browser_(X,Y,W,H,l) {
   type(FL_MULTI_BROWSER);
   Fl_Widget::callback(Widget_Browser_callback);
   when(FL_WHEN_RELEASE);
@@ -127,37 +235,41 @@ const char* Fl_Type::title() {
 
 extern const char* subclassname(Fl_Type*);
 
-void Widget_Browser::item_draw(void *v, int x, int y, int, int) const {
+void Widget_Browser::item_draw(void *v, int X, int Y, int, int) const {
   Fl_Type *l = (Fl_Type *)v;
-  x += 3 + l->level * 10;
-  if (l->new_selected) fl_color(contrast(FL_BLACK,FL_SELECTION_COLOR));
+  X += 3 + 18 + l->level * 12;
+  if (l->new_selected) fl_color(fl_contrast(FL_BLACK,FL_SELECTION_COLOR));
   else fl_color(FL_BLACK);
+  Fl_Pixmap *pm = pixmap[l->pixmapID()];
+  if (pm) pm->draw(X-18, Y);
+  if (l->is_public() == 0) lock_pixmap.draw(X - 17, Y);
+  else if (l->is_public() > 0) ; //unlock_pixmap.draw(X - 17, Y);
   if (l->is_parent()) {
     if (!l->next || l->next->level <= l->level) {
       if (l->open_!=(l==pushedtitle)) {
-	fl_loop(x,y+7,x+5,y+12,x+10,y+7);
+	fl_loop(X,Y+7,X+5,Y+12,X+10,Y+7);
       } else {
-	fl_loop(x+2,y+2,x+7,y+7,x+2,y+12);
+	fl_loop(X+2,Y+2,X+7,Y+7,X+2,Y+12);
       }
     } else {
       if (l->open_!=(l==pushedtitle)) {
-	fl_polygon(x,y+7,x+5,y+12,x+10,y+7);
+	fl_polygon(X,Y+7,X+5,Y+12,X+10,Y+7);
       } else {
-	fl_polygon(x+2,y+2,x+7,y+7,x+2,y+12);
+	fl_polygon(X+2,Y+2,X+7,Y+7,X+2,Y+12);
       }
     }
-    x += 10;
+    X += 10;
   }
   if (l->is_widget() || l->is_class()) {
     const char* c = subclassname(l);
     if (!strncmp(c,"Fl_",3)) c += 3;
     fl_font(textfont(), textsize());
-    fl_draw(c, x, y+13);
-    x += int(fl_width(c)+fl_width('n'));
+    fl_draw(c, X, Y+13);
+    X += int(fl_width(c)+fl_width('n'));
     c = l->name();
     if (c) {
       fl_font(textfont()|FL_BOLD, textsize());
-      fl_draw(c, x, y+13);
+      fl_draw(c, X, Y+13);
     } else if ((c=l->label())) {
       char buf[50]; char* p = buf;
       *p++ = '"';
@@ -168,7 +280,7 @@ void Widget_Browser::item_draw(void *v, int x, int y, int, int) const {
       if (*c) {strcpy(p,"..."); p+=3;}
       *p++ = '"';
       *p = 0;
-      fl_draw(buf, x, y+13);
+      fl_draw(buf, X, Y+13);
     }
   } else {
     const char* c = l->title();
@@ -180,7 +292,7 @@ void Widget_Browser::item_draw(void *v, int x, int y, int, int) const {
     if (*c) {strcpy(p,"..."); p+=3;}
     *p = 0;
     fl_font(textfont() | (l->is_code_block() && (l->level==0 || l->parent->is_class())?0:FL_BOLD), textsize());
-    fl_draw(buf, x, y+13);
+    fl_draw(buf, X, Y+13);
   }
 }
 
@@ -189,18 +301,18 @@ int Widget_Browser::item_width(void *v) const {
 
   if (!l->visible) return 0;
 
-  int w = 3 + l->level*10;
-  if (l->is_parent()) w += 10;
+  int W = 3 + 16 + 18 + l->level*10;
+  if (l->is_parent()) W += 10;
 
   if (l->is_widget() || l->is_class()) {
     const char* c = l->type_name();
     if (!strncmp(c,"Fl_",3)) c += 3;
     fl_font(textfont(), textsize());
-    w += int(fl_width(c) + fl_width('n'));
+    W += int(fl_width(c) + fl_width('n'));
     c = l->name();
     if (c) {
       fl_font(textfont()|FL_BOLD, textsize());
-      w += int(fl_width(c));
+      W += int(fl_width(c));
     } else if ((c=l->label())) {
       char buf[50]; char* p = buf;
       *p++ = '"';
@@ -211,7 +323,7 @@ int Widget_Browser::item_width(void *v) const {
       if (*c) {strcpy(p,"..."); p+=3;}
       *p++ = '"';
       *p = 0;
-      w += int(fl_width(buf));
+      W += int(fl_width(buf));
     }
   } else {
     const char* c = l->title();
@@ -223,10 +335,10 @@ int Widget_Browser::item_width(void *v) const {
     if (*c) {strcpy(p,"..."); p+=3;}
     *p = 0;
     fl_font(textfont() | (l->is_code_block() && (l->level==0 || l->parent->is_class())?0:FL_BOLD), textsize());
-    w += int(fl_width(buf));
+    W += int(fl_width(buf));
   }
 
-  return w;
+  return W;
 }
 
 void redraw_browser() {
@@ -246,7 +358,7 @@ int Widget_Browser::handle(int e) {
     if (!Fl::event_inside(X,Y,W,H)) break;
     l = (Fl_Type*)find_item(Fl::event_y());
     if (l) {
-      X += 10*l->level;
+      X += 12*l->level + 18 - hposition();
       if (l->is_parent() && Fl::event_x()>X && Fl::event_x()<X+13) {
 	title = pushedtitle = l;
 	redraw_line(l);
@@ -258,7 +370,7 @@ int Widget_Browser::handle(int e) {
     if (!title) break;
     l = (Fl_Type*)find_item(Fl::event_y());
     if (l) {
-      X += 10*l->level;
+      X += 12*l->level + 18 - hposition();
       if (l->is_parent() && Fl::event_x()>X && Fl::event_x()<X+13) ;
       else l = 0;
     }
@@ -396,7 +508,8 @@ Fl_Type::msgnum() {
   Fl_Type	*p;
 
   for (count = 0, p = this; p;) {
-    if (p->is_widget() && p->label() && !((Fl_Widget_Type *)p)->image) count ++;
+    if (p->label()) count ++;
+    if (p != this && p->is_widget() && ((Fl_Widget_Type *)p)->tooltip()) count ++;
 
     if (p->prev) p = p->prev;
     else p = p->parent;
@@ -444,8 +557,7 @@ int storestring(const char *n, const char * & p, int nostrip) {
     p = 0;
   } else {
     char *q = (char *)malloc(length+1);
-    strncpy(q,n,length);
-    q[length] = 0;
+    strlcpy(q,n,length+1);
     p = q;
   }
   modflag = 1;
@@ -504,6 +616,11 @@ int Fl_Type::is_window() const {return 0;}
 int Fl_Type::is_code_block() const {return 0;}
 int Fl_Type::is_decl_block() const {return 0;}
 int Fl_Type::is_class() const {return 0;}
+int Fl_Type::is_public() const {return 1;}
+
+int Fl_Code_Type::is_public()const { return -1; }
+int Fl_CodeBlock_Type::is_public()const { return -1; }
+
 
 ////////////////////////////////////////////////////////////////
 
@@ -609,6 +726,13 @@ void later_cb(Fl_Widget*,void*) {
 void Fl_Type::write() {
     write_indent(level);
     write_word(type_name());
+    
+    if (is_class()) {
+      const char * p = 	((Fl_Class_Type*)this)->prefix();
+      if (p &&	strlen(p))
+	write_word(p);
+    }
+
     write_word(name());
     write_open(level);
     write_properties();
@@ -633,10 +757,10 @@ void Fl_Type::write_properties() {
     write_indent(level+1);
     write_word("user_data");
     write_word(user_data());
-    if (user_data_type()) {
-      write_word("user_data_type");
-      write_word(user_data_type());
-    }
+  }
+  if (user_data_type()) {
+    write_word("user_data_type");
+    write_word(user_data_type());
   }
   if (callback()) {
     write_indent(level+1);
@@ -667,5 +791,5 @@ void Fl_Type::read_property(const char *c) {
 int Fl_Type::read_fdesign(const char*, const char*) {return 0;}
 
 //
-// End of "$Id: Fl_Type.cxx,v 1.6.2.6 2001/01/22 15:13:38 easysw Exp $".
+// End of "$Id: Fl_Type.cxx,v 1.6.2.6.2.19 2004/05/13 21:02:39 easysw Exp $".
 //
