@@ -1,5 +1,5 @@
 //
-// "$Id: Fl_Type.h,v 1.5.2.11 2001/01/22 15:13:38 easysw Exp $"
+// "$Id: Fl_Type.h,v 1.5.2.11.2.13 2004/04/11 04:38:55 easysw Exp $"
 //
 // Widget type header file for the Fast Light Tool Kit (FLTK).
 //
@@ -11,7 +11,7 @@
 // instance of this object.  It could also have a "copy()" function,
 // but it was easier to implement this by using the file read/write
 // that is needed to save the setup anyways.
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2004 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -125,6 +125,9 @@ public:
   virtual int is_code_block() const;
   virtual int is_decl_block() const;
   virtual int is_class() const;
+  virtual int is_public() const;
+
+  virtual int pixmapID() { return 0; }
 
   const char* class_name(const int need_nest) const;
 };
@@ -144,6 +147,8 @@ public:
   }
   int is_parent() const {return 1;}
   int is_code_block() const {return 1;}
+  virtual int is_public() const;
+  int pixmapID() { return 7; }
   void write_properties();
   void read_property(const char *);
 };
@@ -156,6 +161,8 @@ public:
   void open();
   virtual const char *type_name() {return "code";}
   int is_code_block() const {return 0;}
+  int pixmapID() { return 8; }
+  virtual int is_public() const;
 };
 
 class Fl_CodeBlock_Type : public Fl_Type {
@@ -168,6 +175,8 @@ public:
   virtual const char *type_name() {return "codeblock";}
   int is_code_block() const {return 1;}
   int is_parent() const {return 1;}
+  virtual int is_public() const;
+  int pixmapID() { return 9; }
   void write_properties();
   void read_property(const char *);
 };
@@ -182,10 +191,13 @@ public:
   virtual const char *type_name() {return "decl";}
   void write_properties();
   void read_property(const char *);
+  virtual int is_public() const;
+  int pixmapID() { return 10; }
 };
 
 class Fl_DeclBlock_Type : public Fl_Type {
   const char* after;
+  char public_;
 public:
   Fl_Type *make();
   void write_code1();
@@ -196,6 +208,8 @@ public:
   void read_property(const char *);
   int is_parent() const {return 1;}
   int is_decl_block() const {return 1;}
+  virtual int is_public() const;
+  int pixmapID() { return 11; }
 };
 
 class Fl_Class_Type : public Fl_Type {
@@ -214,8 +228,16 @@ public:
   int is_parent() const {return 1;}
   int is_decl_block() const {return 1;}
   int is_class() const {return 1;}
+  virtual int is_public() const;
+  int pixmapID() { return 12; }
   void write_properties();
   void read_property(const char *);
+
+  // class prefix attribute access
+  void prefix(const char* p);
+  const char*  prefix() const {return class_prefix;}
+private:
+  const char* class_prefix;
 };
 
 #define NUM_EXTRA_CODE 4
@@ -227,6 +249,9 @@ class Fl_Widget_Type : public Fl_Type {
 
   const char *extra_code_[NUM_EXTRA_CODE];
   const char *subclass_;
+  const char *tooltip_;
+  const char *image_name_;
+  const char *inactive_name_;
   uchar hotspot_;
 
 protected:
@@ -237,6 +262,7 @@ protected:
   void write_extra_code();
   void write_block_close();
   void write_code2();
+  void write_color(const char*, Fl_Color);
 
 public:
 
@@ -246,6 +272,8 @@ public:
 
   Fluid_Image *image;
   void setimage(Fluid_Image *);
+  Fluid_Image *inactive;
+  void setinactive(Fluid_Image *);
 
   Fl_Widget_Type();
   Fl_Type *make();
@@ -255,6 +283,12 @@ public:
   void extra_code(int n,const char *);
   const char *subclass() const {return subclass_;}
   void subclass(const char *);
+  const char *tooltip() const {return tooltip_;}
+  void tooltip(const char *);
+  const char *image_name() const {return image_name_;}
+  void image_name(const char *);
+  const char *inactive_name() const {return inactive_name_;}
+  void inactive_name(const char *);
   uchar hotspot() const {return hotspot_;}
   void hotspot(uchar v) {hotspot_ = v;}
   uchar resizable() const;
@@ -264,6 +298,7 @@ public:
   virtual Fl_Menu_Item *subtypes();
 
   virtual int is_widget() const;
+  virtual int is_public() const;
 
   virtual void write_properties();
   virtual void read_property(const char *);
@@ -275,24 +310,34 @@ public:
 
 #include <FL/Fl_Tabs.H>
 #include <FL/Fl_Pack.H>
+#include <FL/Fl_Wizard.H>
 
 class igroup : public Fl_Group {
 public:
   void resize(int,int,int,int);
-  igroup(int x,int y,int w,int h) : Fl_Group(x,y,w,h) {Fl_Group::current(0);}
+  void full_resize(int X, int Y, int W, int H) { Fl_Group::resize(X, Y, W, H); }
+  igroup(int X,int Y,int W,int H) : Fl_Group(X,Y,W,H) {Fl_Group::current(0);}
 };
 
 class itabs : public Fl_Tabs {
 public:
   void resize(int,int,int,int);
-  itabs(int x,int y,int w,int h) : Fl_Tabs(x,y,w,h) {}
+  void full_resize(int X, int Y, int W, int H) { Fl_Group::resize(X, Y, W, H); }
+  itabs(int X,int Y,int W,int H) : Fl_Tabs(X,Y,W,H) {}
+};
+
+class iwizard : public Fl_Wizard {
+public:
+  void resize(int,int,int,int);
+  void full_resize(int X, int Y, int W, int H) { Fl_Group::resize(X, Y, W, H); }
+  iwizard(int X,int Y,int W,int H) : Fl_Wizard(X,Y,W,H) {}
 };
 
 class Fl_Group_Type : public Fl_Widget_Type {
 public:
   virtual const char *type_name() {return "Fl_Group";}
-  Fl_Widget *widget(int x,int y,int w,int h) {
-    igroup *g = new igroup(x,y,w,h); Fl_Group::current(0); return g;}
+  Fl_Widget *widget(int X,int Y,int W,int H) {
+    igroup *g = new igroup(X,Y,W,H); Fl_Group::current(0); return g;}
   Fl_Widget_Type *_make() {return new Fl_Group_Type();}
   Fl_Type *make();
   void write_code1();
@@ -302,6 +347,7 @@ public:
   void remove_child(Fl_Type*);
   int is_parent() const {return 1;}
   int is_group() const {return 1;}
+  int pixmapID() { return 6; }
 };
 
 extern const char pack_type_name[];
@@ -312,6 +358,7 @@ class Fl_Pack_Type : public Fl_Group_Type {
 public:
   virtual const char *type_name() {return pack_type_name;}
   Fl_Widget_Type *_make() {return new Fl_Pack_Type();}
+  int pixmapID() { return 22; }
 };
 
 extern const char tabs_type_name[];
@@ -319,12 +366,13 @@ extern const char tabs_type_name[];
 class Fl_Tabs_Type : public Fl_Group_Type {
 public:
   virtual const char *type_name() {return tabs_type_name;}
-  Fl_Widget *widget(int x,int y,int w,int h) {
-    itabs *g = new itabs(x,y,w,h); Fl_Group::current(0); return g;}
+  Fl_Widget *widget(int X,int Y,int W,int H) {
+    itabs *g = new itabs(X,Y,W,H); Fl_Group::current(0); return g;}
   Fl_Widget_Type *_make() {return new Fl_Tabs_Type();}
   Fl_Type* click_test(int,int);
   void add_child(Fl_Type*, Fl_Type*);
   void remove_child(Fl_Type*);
+  int pixmapID() { return 13; }
 };
 
 extern const char scroll_type_name[];
@@ -335,6 +383,7 @@ class Fl_Scroll_Type : public Fl_Group_Type {
 public:
   virtual const char *type_name() {return scroll_type_name;}
   Fl_Widget_Type *_make() {return new Fl_Scroll_Type();}
+  int pixmapID() { return 19; }
 };
 
 extern const char tile_type_name[];
@@ -343,6 +392,18 @@ class Fl_Tile_Type : public Fl_Group_Type {
 public:
   virtual const char *type_name() {return tile_type_name;}
   Fl_Widget_Type *_make() {return new Fl_Tile_Type();}
+  int pixmapID() { return 20; }
+};
+
+extern const char wizard_type_name[];
+
+class Fl_Wizard_Type : public Fl_Group_Type {
+public:
+  virtual const char *type_name() {return wizard_type_name;}
+  Fl_Widget *widget(int X,int Y,int W,int H) {
+    iwizard *g = new iwizard(X,Y,W,H); Fl_Group::current(0); return g;}
+  Fl_Widget_Type *_make() {return new Fl_Wizard_Type();}
+  int pixmapID() { return 21; }
 };
 
 extern Fl_Menu_Item window_type_menu[];
@@ -369,6 +430,7 @@ class Fl_Window_Type : public Fl_Widget_Type {
   Fl_Widget *widget(int,int,int,int) {return 0;}
   int recalc;		// set by fix_overlay()
   void moveallchildren();
+  int pixmapID() { return 1; }
 
 public:
 
@@ -411,6 +473,7 @@ public:
   void write_item();
   void write_code1();
   void write_code2();
+  int pixmapID() { return 16; }
 };
 
 class Fl_Submenu_Type : public Fl_Menu_Item_Type {
@@ -425,6 +488,7 @@ public:
   void add_child(Fl_Type*a, Fl_Type*b) {parent->add_child(a,b);}
   void move_child(Fl_Type*a, Fl_Type*b) {parent->move_child(a,b);}
   void remove_child(Fl_Type*a) {parent->remove_child(a);}
+  int pixmapID() { return 18; }
 };
 
 
@@ -464,9 +528,10 @@ class Fl_Menu_Button_Type : public Fl_Menu_Type {
   Fl_Menu_Item *subtypes() {return button_type_menu;}
 public:
   virtual const char *type_name() {return "Fl_Menu_Button";}
-  Fl_Widget *widget(int x,int y,int w,int h) {
-    return new Fl_Menu_Button(x,y,w,h,"menu");}
+  Fl_Widget *widget(int X,int Y,int W,int H) {
+    return new Fl_Menu_Button(X,Y,W,H,"menu");}
   Fl_Widget_Type *_make() {return new Fl_Menu_Button_Type();}
+  int pixmapID() { return 26; }
 };
 
 extern Fl_Menu_Item dummymenu[];
@@ -475,24 +540,26 @@ extern Fl_Menu_Item dummymenu[];
 class Fl_Choice_Type : public Fl_Menu_Type {
 public:
   virtual const char *type_name() {return "Fl_Choice";}
-  Fl_Widget *widget(int x,int y,int w,int h) {
-    Fl_Choice *myo = new Fl_Choice(x,y,w,h,"choice:");
+  Fl_Widget *widget(int X,int Y,int W,int H) {
+    Fl_Choice *myo = new Fl_Choice(X,Y,W,H,"choice:");
     myo->menu(dummymenu);
     return myo;
   }
   Fl_Widget_Type *_make() {return new Fl_Choice_Type();}
+  int pixmapID() { return 15; }
 };
 
 #include <FL/Fl_Menu_Bar.H>
 class Fl_Menu_Bar_Type : public Fl_Menu_Type {
 public:
   virtual const char *type_name() {return "Fl_Menu_Bar";}
-  Fl_Widget *widget(int x,int y,int w,int h) {
-    return new Fl_Menu_Bar(x,y,w,h);}
+  Fl_Widget *widget(int X,int Y,int W,int H) {
+    return new Fl_Menu_Bar(X,Y,W,H);}
   Fl_Widget_Type *_make() {return new Fl_Menu_Bar_Type();}
+  int pixmapID() { return 17; }
 };
 // object list operations:
-Fl_Widget *make_widget_browser(int x,int y,int w,int h);
+Fl_Widget *make_widget_browser(int X,int Y,int W,int H);
 extern int modflag;
 void delete_all(int selected_only=0);
 void selection_changed(Fl_Type* new_current);
@@ -532,5 +599,5 @@ int storestring(const char *n, const char * & p, int nostrip=0);
 extern int include_H_from_C;
 
 //
-// End of "$Id: Fl_Type.h,v 1.5.2.11 2001/01/22 15:13:38 easysw Exp $".
+// End of "$Id: Fl_Type.h,v 1.5.2.11.2.13 2004/04/11 04:38:55 easysw Exp $".
 //
