@@ -1,5 +1,5 @@
 //
-// "$Id: Fl_Window_Type.cxx,v 1.13.2.10 2001/04/13 19:07:40 easysw Exp $"
+// "$Id: Fl_Window_Type.cxx,v 1.13.2.10.2.10 2004/11/20 15:42:24 easysw Exp $"
 //
 // Window type code for the Fast Light Tool Kit (FLTK).
 //
@@ -7,7 +7,7 @@
 // for interacting with the overlay, which allows the user to
 // select, move, and resize the children widgets.
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2004 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -38,9 +38,9 @@
 #include "alignment_panel.h"
 #include <stdio.h>
 
-int gridx = 5;
-int gridy = 5;
-int snap = 3;
+extern int gridx;
+extern int gridy;
+extern int snap;
 
 int include_H_from_C = 1;
 extern int i18n_type;
@@ -50,15 +50,39 @@ extern const char* i18n_file;
 extern const char* i18n_set;
 extern int modflag;
 
-void alignment_cb(Fl_Input *i, long v) {
+extern Fl_Preferences	fluid_prefs;
+
+void grid_cb(Fl_Input *i, long v) {
   int n = atoi(i->value());
   if (n < 0) n = 0;
   switch (v) {
-  case 1: gridx = n; break;
-  case 2: gridy = n; break;
-  case 3: snap  = n; break;
+    case 1:
+      gridx = n;
+      fluid_prefs.set("gridx", n);
+      break;
+    case 2:
+      gridy = n;
+      fluid_prefs.set("gridy", n);
+      break;
+    case 3:
+      snap = n;
+      fluid_prefs.set("snap", n);
+      break;
+  }
+
+  // Next go through all of the windows in the project and set the
+  // stepping for resizes...
+  Fl_Type *p;
+  Fl_Window_Type *w;
+
+  for (p = Fl_Type::first; p; p = p->next) {
+    if (p->is_window()) {
+      w = (Fl_Window_Type *)p;
+      ((Fl_Window *)(w->o))->size_range(gridx * snap, gridy * snap,
+                                        Fl::w(), Fl::h(),
+                                        gridx * snap, gridy * snap, 0);
+    }
   }
-  modflag = 1;
 }
 
 void i18n_type_cb(Fl_Choice *c, void *) {
@@ -112,15 +136,11 @@ void i18n_text_cb(Fl_Input *i, void *) {
 extern const char* header_file_name;
 extern const char* code_file_name;
 
-void show_alignment_cb(Fl_Widget *, void *) {
-  if(alignment_window==0) make_alignment_window();
+void show_project_cb(Fl_Widget *, void *) {
+  if(project_window==0) make_project_window();
   include_H_from_C_button->value(include_H_from_C);
   header_file_input->value(header_file_name);
   code_file_input->value(code_file_name);
-  char buf[128];
-  sprintf(buf,"%d",gridx); horizontal_input->value(buf);
-  sprintf(buf,"%d",gridy); vertical_input->value(buf);
-  sprintf(buf,"%d",snap); snap_input->value(buf);
   i18n_type_chooser->value(i18n_type);
   i18n_function_input->value(i18n_function);
   i18n_file_input->value(i18n_file);
@@ -146,7 +166,22 @@ void show_alignment_cb(Fl_Widget *, void *) {
       i18n_function_input->hide();
       break;
   }
-  alignment_window->show();
+  project_window->hotspot(project_window);
+  project_window->show();
+}
+
+void show_grid_cb(Fl_Widget *, void *) {
+  char buf[128];
+  sprintf(buf,"%d",gridx); horizontal_input->value(buf);
+  sprintf(buf,"%d",gridy); vertical_input->value(buf);
+  sprintf(buf,"%d",snap); snap_input->value(buf);
+  grid_window->hotspot(grid_window);
+  grid_window->show();
+}
+
+void show_settings_cb(Fl_Widget *, void *) {
+  settings_window->hotspot(settings_window);
+  settings_window->show();
 }
 
 void header_input_cb(Fl_Input* i, void*) {
@@ -178,7 +213,7 @@ class Overlay_Window : public Fl_Overlay_Window {
 public:
   Fl_Window_Type *window;
   int handle(int);
-  Overlay_Window(int w,int h) : Fl_Overlay_Window(w,h) {Fl_Group::current(0);}
+  Overlay_Window(int W,int H) : Fl_Overlay_Window(W,H) {Fl_Group::current(0);}
   void resize(int,int,int,int);
 };
 void Overlay_Window::draw() {
@@ -187,11 +222,11 @@ void Overlay_Window::draw() {
   if ((damage()&FL_DAMAGE_ALL) &&
       (!box() || (box()>=4&&!(box()&2)) || box()>=_FL_ROUNDED_BOX)) {
     // if so, draw checkerboard so user can see what areas are clear:
-    for (int y = 0; y < h(); y += CHECKSIZE) 
-      for (int x = 0; x < w(); x += CHECKSIZE) {
-	fl_color(((y/(2*CHECKSIZE))&1) != ((x/(2*CHECKSIZE))&1) ?
+    for (int Y = 0; Y < h(); Y += CHECKSIZE) 
+      for (int X = 0; X < w(); X += CHECKSIZE) {
+	fl_color(((Y/(2*CHECKSIZE))&1) != ((X/(2*CHECKSIZE))&1) ?
 		 FL_WHITE : FL_BLACK);
-	fl_rectf(x,y,CHECKSIZE,CHECKSIZE);
+	fl_rectf(X,Y,CHECKSIZE,CHECKSIZE);
       }
   }
   Fl_Overlay_Window::draw();
@@ -216,6 +251,11 @@ Fl_Type *Fl_Window_Type::make() {
     this->o = new Fl_Window(100,100);
     Fl_Group::current(0);
   }
+  // Set the size ranges for this window; in order to avoid opening the
+  // X display we use an arbitrary maximum size...
+  ((Fl_Window *)(this->o))->size_range(gridx * snap, gridy * snap,
+                                       3072, 2048,
+                                       gridx * snap, gridy * snap, 0);
   myo->factory = this;
   myo->drag = 0;
   myo->numselected = 0;
@@ -264,6 +304,9 @@ void Fl_Window_Type::open() {
     w->show();
     w->resizable(p);
   }
+
+  w->size_range(gridx * snap, gridy * snap, Fl::w(), Fl::h(),
+                gridx * snap, gridy * snap, 0);
 }
 
 // control panel items:
@@ -484,6 +527,16 @@ extern void fix_group_size(Fl_Type *t);
 extern Fl_Menu_Item Main_Menu[];
 extern Fl_Menu_Item New_Menu[];
 
+// Update the XYWH values in the widget panel...
+static void update_xywh() {
+  if (current_widget && current_widget->is_widget()) {
+    widget_x_input->value(((Fl_Widget_Type *)current_widget)->o->x());
+    widget_y_input->value(((Fl_Widget_Type *)current_widget)->o->y());
+    widget_w_input->value(((Fl_Widget_Type *)current_widget)->o->w());
+    widget_h_input->value(((Fl_Widget_Type *)current_widget)->o->h());
+  }
+}
+
 // move the selected children according to current dx,dy,drag state:
 void Fl_Window_Type::moveallchildren()
 {
@@ -499,9 +552,9 @@ void Fl_Window_Type::moveallchildren()
       for (p = myo->next; p && p->level>myo->level; p = p->next)
 	if (p->is_widget() && !p->is_menu_item()) {
 	  Fl_Widget_Type* myo2 = (Fl_Widget_Type*)p;
-	  int x,y,r,t;
-	  newposition(myo2,x,y,r,t);
-	  myo2->o->resize(x,y,r-x,t-y);
+	  int X,Y,R,T;
+	  newposition(myo2,X,Y,R,T);
+	  myo2->o->resize(X,Y,R-X,T-Y);
 	}
       i = p;
     } else {
@@ -515,6 +568,8 @@ void Fl_Window_Type::moveallchildren()
   ((Overlay_Window *)(this->o))->redraw_overlay();
   modflag = 1;
   dx = dy = 0;
+
+  update_xywh();
 }
 
 int Fl_Window_Type::handle(int event) {
@@ -685,7 +740,7 @@ int Fl_Window_Type::handle(int event) {
 ////////////////////////////////////////////////////////////////
 
 #include <stdio.h>
-#include <string.h>
+#include "../src/flstring.h"
 
 void Fl_Window_Type::write_code1() {
   Fl_Widget_Type::write_code1();
@@ -732,25 +787,25 @@ void Fl_Window_Type::read_property(const char *c) {
   }
 }
 
-int Fl_Window_Type::read_fdesign(const char* name, const char* value) {
+int Fl_Window_Type::read_fdesign(const char* propname, const char* value) {
   int x;
   o->box(FL_NO_BOX); // because fdesign always puts an Fl_Box next
-  if (!strcmp(name,"Width")) {
+  if (!strcmp(propname,"Width")) {
     if (sscanf(value,"%d",&x) == 1) o->size(x,o->h());
-  } else if (!strcmp(name,"Height")) {
+  } else if (!strcmp(propname,"Height")) {
     if (sscanf(value,"%d",&x) == 1) o->size(o->w(),x);
-  } else if (!strcmp(name,"NumberofWidgets")) {
+  } else if (!strcmp(propname,"NumberofWidgets")) {
     return 1; // we can figure out count from file
-  } else if (!strcmp(name,"border")) {
+  } else if (!strcmp(propname,"border")) {
     if (sscanf(value,"%d",&x) == 1) ((Fl_Window*)o)->border(x);
-  } else if (!strcmp(name,"title")) {
+  } else if (!strcmp(propname,"title")) {
     label(value);
   } else {
-    return Fl_Widget_Type::read_fdesign(name,value);
+    return Fl_Widget_Type::read_fdesign(propname,value);
   }
   return 1;
 }
 
 //
-// End of "$Id: Fl_Window_Type.cxx,v 1.13.2.10 2001/04/13 19:07:40 easysw Exp $".
+// End of "$Id: Fl_Window_Type.cxx,v 1.13.2.10.2.10 2004/11/20 15:42:24 easysw Exp $".
 //
