@@ -1,5 +1,5 @@
 //
-// "$Id: file.cxx,v 1.7.2.6 2001/01/22 15:13:39 easysw Exp $"
+// "$Id: file.cxx,v 1.7.2.6.2.10 2004/04/11 04:38:56 easysw Exp $"
 //
 // Fluid file routines for the Fast Light Tool Kit (FLTK).
 //
@@ -8,7 +8,7 @@
 // They are somewhat similar to tcl, using matching { and }
 // to quote strings.
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2004 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -28,10 +28,9 @@
 // Please report all bugs and problems to "fltk-bugs@fltk.org".
 //
 
-#include <ctype.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <string.h>
+#include "../src/flstring.h"
 #include <stdarg.h>
 #include "alignment_panel.h"
 
@@ -157,7 +156,7 @@ void read_error(const char *format, ...) {
   va_start(args, format);
   if (!fin) {
     char buffer[1024];
-    vsprintf(buffer, format, args);
+    vsnprintf(buffer, sizeof(buffer), format, args);
     fl_message(buffer);
   } else {
     fprintf(stderr, "%s:%d: ", fname, lineno);
@@ -304,17 +303,11 @@ const char *read_word(int wantbrace) {
 #include "Fl_Widget_Type.h"
 
 // global int variables:
-extern int gridx, gridy, snap;
 extern int i18n_type;
 extern const char* i18n_include;
 extern const char* i18n_function;
 extern const char* i18n_file;
 extern const char* i18n_set;
-static struct {const char* name; int* value;} inttable[] = {
-  {"gridx", &gridx},
-  {"gridy", &gridy},
-  {"snap", &snap}
-};
 
 
 extern int header_file_set;
@@ -344,8 +337,6 @@ int write_file(const char *filename, int selected_only) {
   if (!selected_only) {
     write_string("\nheader_name"); write_word(header_file_name);
     write_string("\ncode_name"); write_word(code_file_name);
-    for (unsigned int i=0; i<sizeof(inttable)/sizeof(*inttable); i++)
-      write_string("\n%s %d",inttable[i].name, *inttable[i].value);
   }
   for (Fl_Type *p = Fl_Type::first; p;) {
     if (!selected_only || p->selected) {
@@ -372,7 +363,6 @@ extern Fl_Type *Fl_Type_make(const char *tn);
 static void read_children(Fl_Type *p, int paste) {
   Fl_Type::current = p;
   for (;;) {
-    unsigned int i;
     const char *c = read_word();
   REUSE_C:
     if (!c) {
@@ -454,12 +444,10 @@ static void read_children(Fl_Type *p, int paste) {
       goto CONTINUE;
     }
 
-    for (i=0; i<sizeof(inttable)/sizeof(*inttable); i++) {
-      if (!strcmp(c,inttable[i].name)) {
-	c = read_word();
-	*inttable[i].value = atoi(c);
-	goto CONTINUE;
-      }
+    if (!strcmp(c, "snap") || !strcmp(c, "gridx") || !strcmp(c, "gridy")) {
+      // grid settings are now global
+      read_word();
+      goto CONTINUE;
     }
 
     {Fl_Type *t = Fl_Type_make(c);
@@ -470,6 +458,12 @@ static void read_children(Fl_Type *p, int paste) {
     t->name(read_word());
 
     c = read_word(1);
+    if (strcmp(c,"{") && t->is_class()) {   // <prefix> <name>
+      ((Fl_Class_Type*)t)->prefix(t->name());
+      t->name(c);
+      c = read_word(1);
+    }
+
     if (strcmp(c,"{")) {
       read_error("Missing property list for %s\n",t->title());
       goto REUSE_C;
@@ -477,9 +471,9 @@ static void read_children(Fl_Type *p, int paste) {
 
     t->open_ = 0;
     for (;;) {
-      const char *c = read_word();
-      if (!c || !strcmp(c,"}")) break;
-      t->read_property(c);
+      const char *cc = read_word();
+      if (!cc || !strcmp(cc,"}")) break;
+      t->read_property(cc);
     }
 
     if (!t->is_parent()) continue;
@@ -497,12 +491,16 @@ static void read_children(Fl_Type *p, int paste) {
 extern void deselect();
 
 int read_file(const char *filename, int merge) {
+  Fl_Type *o;
   read_version = 0.0;
   if (!open_read(filename)) return 0;
   if (merge) deselect(); else    delete_all();
   read_children(Fl_Type::current, merge);
   Fl_Type::current = 0;
-  for (Fl_Type *o = Fl_Type::first; o; o = o->next)
+  // Force menu items to be rebuilt...
+  for (o = Fl_Type::first; o; o = o->next)
+    if (o->is_menu_button()) o->add_child(0,0);
+  for (o = Fl_Type::first; o; o = o->next)
     if (o->selected) {Fl_Type::current = o; break;}
   return close_read();
 }
@@ -639,5 +637,5 @@ void read_fdesign() {
 }
 
 //
-// End of "$Id: file.cxx,v 1.7.2.6 2001/01/22 15:13:39 easysw Exp $".
+// End of "$Id: file.cxx,v 1.7.2.6.2.10 2004/04/11 04:38:56 easysw Exp $".
 //
