@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_Browser.cxx,v 1.9.2.12 2001/01/22 15:13:39 easysw Exp $"
+// "$Id: Fl_Browser.cxx,v 1.9.2.12.2.13 2004/11/20 03:19:58 easysw Exp $"
 //
 // Browser widget for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2004 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -26,7 +26,7 @@
 #include <FL/Fl.H>
 #include <FL/Fl_Browser.H>
 #include <FL/fl_draw.H>
-#include <string.h>
+#include "flstring.h"
 #include <stdlib.h>
 #include <math.h>
 
@@ -116,13 +116,13 @@ FL_BLINE* Fl_Browser::_remove(int line) {
 
   cacheline = line-1;
   cache = ttt->prev;
+  lines--;
+  full_height_ -= item_height(ttt);
   if (ttt->prev) ttt->prev->next = ttt->next;
   else first = ttt->next;
   if (ttt->next) ttt->next->prev = ttt->prev;
   else last = ttt->prev;
 
-  lines--;
-  full_height_ -= item_height(ttt);
   return(ttt);
 }
 
@@ -161,13 +161,13 @@ void Fl_Browser::insert(int line, FL_BLINE* t) {
   redraw_line(t);
 }
 
-void Fl_Browser::insert(int line, const char* newtext, void* data) {
+void Fl_Browser::insert(int line, const char* newtext, void* d) {
   int l = strlen(newtext);
   FL_BLINE* t = (FL_BLINE*)malloc(sizeof(FL_BLINE)+l);
-  t->length = l;
+  t->length = (short)l;
   t->flags = 0;
   strcpy(t->txt, newtext);
-  t->data = data;
+  t->data = d;
   insert(line, t);
 }
 
@@ -185,7 +185,7 @@ void Fl_Browser::text(int line, const char* newtext) {
     replacing(t, n);
     cache = n;
     n->data = t->data;
-    n->length = l;
+    n->length = (short)l;
     n->flags = t->flags;
     n->prev = t->prev;
     if (n->prev) n->prev->next = n; else first = n;
@@ -198,9 +198,9 @@ void Fl_Browser::text(int line, const char* newtext) {
   redraw_line(t);
 }
 
-void Fl_Browser::data(int line, void* data) {
+void Fl_Browser::data(int line, void* d) {
   if (line < 1 || line > lines) return;
-  find_line(line)->data = data;
+  find_line(line)->data = d;
 }
 
 int Fl_Browser::item_height(void* lv) const {
@@ -212,39 +212,41 @@ int Fl_Browser::item_height(void* lv) const {
   if (!l->txt[0]) {
     // For blank lines set the height to exactly 1 line!
     fl_font(textfont(), textsize());
-    int h = fl_height();
-    if (h > hmax) hmax = h;
+    int hh = fl_height();
+    if (hh > hmax) hmax = hh;
   }
   else {
+    const int* i = column_widths();
     // do each column separately as they may all set different fonts:
-    for (char* str = l->txt; *str; str++) {
+    for (char* str = l->txt; str && *str; str++) {
       Fl_Font font = textfont(); // default font
-      int size = textsize(); // default size
+      int tsize = textsize(); // default size
       while (*str==format_char()) {
 	str++;
 	switch (*str++) {
-	case 'l': case 'L': size = 24; break;
-	case 'm': case 'M': size = 18; break;
-	case 's': size = 11; break;
+	case 'l': case 'L': tsize = 24; break;
+	case 'm': case 'M': tsize = 18; break;
+	case 's': tsize = 11; break;
 	case 'b': font = (Fl_Font)(font|FL_BOLD); break;
 	case 'i': font = (Fl_Font)(font|FL_ITALIC); break;
 	case 'f': case 't': font = FL_COURIER; break;
 	case 'B':
 	case 'C': strtol(str, &str, 10); break;// skip a color number
 	case 'F': font = (Fl_Font)strtol(str,&str,10); break;
-	case 'S': size = strtol(str,&str,10); break;
+	case 'S': tsize = strtol(str,&str,10); break;
 	case 0: case '@': str--;
 	case '.': goto END_FORMAT;
 	}
       }
       END_FORMAT:
       char* ptr = str;
-      for(;*str && (*str!=column_char()); str++) ;
-      if (ptr < str) {
-	fl_font(font, size); int h = fl_height();
-	if (h > hmax) hmax = h;
+      if (ptr && *i++) str = strchr(str, column_char());
+      else str = NULL;
+      if((!str && *ptr) || (str && ptr < str)) {
+	fl_font(font, tsize); int hh = fl_height();
+	if (hh > hmax) hmax = hh;
       }
-      if (!*str) str --;
+      if (!str || !*str) break;
     }
   }
 
@@ -254,37 +256,37 @@ int Fl_Browser::item_height(void* lv) const {
 int Fl_Browser::item_width(void* v) const {
   char* str = ((FL_BLINE*)v)->txt;
   const int* i = column_widths();
-  int w = 0;
+  int ww = 0;
 
   while (*i) { // add up all tab-seperated fields
-    w += *i++;
     char* e;
-    for (e = str; *e && *e != column_char(); e++);
-    if (!*e) return 0; // last one occupied by text
+    e = strchr(str, column_char());
+    if (!e) break; // last one occupied by text
     str = e+1;
+    ww += *i++;
   }
 
   // OK, we gotta parse the string and find the string width...
-  int size = textsize();
+  int tsize = textsize();
   Fl_Font font = textfont();
   int done = 0;
 
-  // MRS - might this cause problems on some platforms - order of operations?
-
-  while (*str == format_char_ && *++str && *str != format_char_) {
+  while (*str == format_char_ && str[1] && str[1] != format_char_) {
+    str ++;
     switch (*str++) {
-    case 'l': case 'L': size = 24; break;
-    case 'm': case 'M': size = 18; break;
-    case 's': size = 11; break;
+    case 'l': case 'L': tsize = 24; break;
+    case 'm': case 'M': tsize = 18; break;
+    case 's': tsize = 11; break;
     case 'b': font = (Fl_Font)(font|FL_BOLD); break;
     case 'i': font = (Fl_Font)(font|FL_ITALIC); break;
     case 'f': case 't': font = FL_COURIER; break;
     case 'B':
     case 'C': strtol(str, &str, 10); break;// skip a color number
     case 'F': font = (Fl_Font)strtol(str, &str, 10); break;
-    case 'S': size = strtol(str, &str, 10); break;
+    case 'S': tsize = strtol(str, &str, 10); break;
     case '.':
       done = 1;
+      break;
     case '@':
       str--;
       done = 1;
@@ -294,8 +296,11 @@ int Fl_Browser::item_width(void* v) const {
       break;
   }
 
-  fl_font(font, size);
-  return w + int(fl_width(str)) + 6;
+  if (*str == format_char_ && str[1])
+    str ++;
+
+  fl_font(font, tsize);
+  return ww + int(fl_width(str)) + 6;
 }
 
 int Fl_Browser::full_height() const {
@@ -306,36 +311,36 @@ int Fl_Browser::incr_height() const {
   return textsize()+2;
 }
 
-void Fl_Browser::item_draw(void* v, int x, int y, int w, int h) const {
+void Fl_Browser::item_draw(void* v, int X, int Y, int W, int H) const {
   char* str = ((FL_BLINE*)v)->txt;
   const int* i = column_widths();
 
-  while (w > 6) {	// do each tab-seperated field
-    int w1 = w;	// width for this field
+  while (W > 6) {	// do each tab-seperated field
+    int w1 = W;	// width for this field
     char* e = 0; // pointer to end of field or null if none
     if (*i) { // find end of field and temporarily replace with 0
-      for (e = str; *e && *e != column_char(); e++);
-      if (*e) {*e = 0; w1 = *i++;} else e = 0;
+      e = strchr(str, column_char());
+      if (e) {*e = 0; w1 = *i++;}
     }
-    int size = textsize();
+    int tsize = textsize();
     Fl_Font font = textfont();
     Fl_Color lcol = textcolor();
-    Fl_Align align = FL_ALIGN_LEFT;
+    Fl_Align talign = FL_ALIGN_LEFT;
     // check for all the @-lines recognized by XForms:
     while (*str == format_char() && *++str && *str != format_char()) {
       switch (*str++) {
-      case 'l': case 'L': size = 24; break;
-      case 'm': case 'M': size = 18; break;
-      case 's': size = 11; break;
+      case 'l': case 'L': tsize = 24; break;
+      case 'm': case 'M': tsize = 18; break;
+      case 's': tsize = 11; break;
       case 'b': font = (Fl_Font)(font|FL_BOLD); break;
       case 'i': font = (Fl_Font)(font|FL_ITALIC); break;
       case 'f': case 't': font = FL_COURIER; break;
-      case 'c': align = FL_ALIGN_CENTER; break;
-      case 'r': align = FL_ALIGN_RIGHT; break;
+      case 'c': talign = FL_ALIGN_CENTER; break;
+      case 'r': talign = FL_ALIGN_RIGHT; break;
       case 'B': 
 	if (!(((FL_BLINE*)v)->flags & SELECTED)) {
 	  fl_color((Fl_Color)strtol(str, &str, 10));
-	  fl_rectf(x, y, w1, h);
+	  fl_rectf(X, Y, w1, H);
 	} else strtol(str, &str, 10);
         break;
       case 'C':
@@ -348,18 +353,18 @@ void Fl_Browser::item_draw(void* v, int x, int y, int w, int h) const {
 	lcol = FL_INACTIVE_COLOR;
 	break;
       case 'S':
-	size = strtol(str, &str, 10);
+	tsize = strtol(str, &str, 10);
 	break;
       case '-':
 	fl_color(FL_DARK3);
-	fl_line(x+3, y+h/2, x+w1-3, y+h/2);
+	fl_line(X+3, Y+H/2, X+w1-3, Y+H/2);
 	fl_color(FL_LIGHT3);
-	fl_line(x+3, y+h/2+1, x+w1-3, y+h/2+1);
+	fl_line(X+3, Y+H/2+1, X+w1-3, Y+H/2+1);
 	break;
       case 'u':
       case '_':
 	fl_color(lcol);
-	fl_line(x+3, y+h-1, x+w1-3, y+h-1);
+	fl_line(X+3, Y+H-1, X+w1-3, Y+H-1);
 	break;
       case '.':
 	goto BREAK;
@@ -368,24 +373,24 @@ void Fl_Browser::item_draw(void* v, int x, int y, int w, int h) const {
       }
     }
   BREAK:
-    fl_font(font, size);
+    fl_font(font, tsize);
     if (((FL_BLINE*)v)->flags & SELECTED)
-      lcol = contrast(lcol, selection_color());
-    if (!active_r()) lcol = inactive(lcol);
+      lcol = fl_contrast(lcol, selection_color());
+    if (!active_r()) lcol = fl_inactive(lcol);
     fl_color(lcol);
-    fl_draw(str, x+3, y, w1-6, h, e ? Fl_Align(align|FL_ALIGN_CLIP) : align);
+    fl_draw(str, X+3, Y, w1-6, H, e ? Fl_Align(talign|FL_ALIGN_CLIP) : talign, 0, 0);
     if (!e) break; // no more fields...
     *e = column_char(); // put the seperator back
-    x += w1;
-    w -= w1;
+    X += w1;
+    W -= w1;
     str = e+1;
   }
 }
 
 static const int no_columns[1] = {0};
 
-Fl_Browser::Fl_Browser(int x, int y, int w, int h, const char*l)
-  : Fl_Browser_(x, y, w, h, l) {
+Fl_Browser::Fl_Browser(int X, int Y, int W, int H, const char*l)
+  : Fl_Browser_(X, Y, W, H, l) {
   column_widths_ = no_columns;
   lines = 0;
   full_height_ = 0;
@@ -425,9 +430,9 @@ int Fl_Browser::topline() const {
 
 void Fl_Browser::clear() {
   for (FL_BLINE* l = first; l;) {
-    FL_BLINE* h = l->next;
+    FL_BLINE* n = l->next;
     free(l);
-    l = h;
+    l = n;
   }
   full_height_ = 0;
   first = 0;
@@ -435,8 +440,8 @@ void Fl_Browser::clear() {
   new_list();
 }
 
-void Fl_Browser::add(const char* newtext, void* data) {
-  insert(lines+1, newtext, data);
+void Fl_Browser::add(const char* newtext, void* d) {
+  insert(lines+1, newtext, d);
   //Fl_Browser_::display(last);
 }
 
@@ -450,9 +455,9 @@ void* Fl_Browser::data(int line) const {
   return find_line(line)->data;
 }
 
-int Fl_Browser::select(int line, int value) {
+int Fl_Browser::select(int line, int v) {
   if (line < 1 || line > lines) return 0;
-  return Fl_Browser_::select(find_line(line), value);
+  return Fl_Browser_::select(find_line(line), v);
 }
 
 int Fl_Browser::selected(int line) const {
@@ -478,9 +483,9 @@ void Fl_Browser::hide(int line) {
   }
 }
 
-void Fl_Browser::display(int line, int value) {
+void Fl_Browser::display(int line, int v) {
   if (line < 1 || line > lines) return;
-  if (value) show(line); else hide(line);
+  if (v) show(line); else hide(line);
 }
 
 int Fl_Browser::visible(int line) const {
@@ -492,6 +497,54 @@ int Fl_Browser::value() const {
   return lineno(selection());
 }
 
+// SWAP TWO LINES
+void Fl_Browser::swap(FL_BLINE *a, FL_BLINE *b) {
+
+  if ( a == b || !a || !b) return;          // nothing to do
+  FL_BLINE *aprev  = a->prev;
+  FL_BLINE *anext  = a->next;
+  FL_BLINE *bprev  = b->prev;
+  FL_BLINE *bnext  = b->next;
+  if ( b->prev == a ) { 		// A ADJACENT TO B
+     if ( aprev ) aprev->next = b; else first = b;
+     b->next = a;
+     a->next = bnext;
+     b->prev = aprev;
+     a->prev = b;
+     if ( bnext ) bnext->prev = a; else last = a;
+  } else if ( a->prev == b ) {		// B ADJACENT TO A
+     if ( bprev ) bprev->next = a; else first = a;
+     a->next = b;
+     b->next = anext;
+     a->prev = bprev;
+     b->prev = a;
+     if ( anext ) anext->prev = b; else last = b;
+  } else {				// A AND B NOT ADJACENT
+     // handle prev's
+     b->prev = aprev;
+     if ( anext ) anext->prev = b; else last = b;
+     a->prev = bprev;
+     if ( bnext ) bnext->prev = a; else last = a;
+     // handle next's
+     if ( aprev ) aprev->next = b; else first = b;
+     b->next = anext;
+     if ( bprev ) bprev->next = a; else first = a;
+     a->next = bnext;
+  }
+  // Disable cache -- we played around with positions
+  cacheline = 0;
+  // Redraw modified lines
+  redraw_line(a);
+  redraw_line(b);
+}
+
+void Fl_Browser::swap(int ai, int bi) {
+  if (ai < 1 || ai > lines || bi < 1 || bi > lines) return;
+  FL_BLINE* a = find_line(ai);
+  FL_BLINE* b = find_line(bi);
+  swap(a,b);
+}
+
 //
-// End of "$Id: Fl_Browser.cxx,v 1.9.2.12 2001/01/22 15:13:39 easysw Exp $".
+// End of "$Id: Fl_Browser.cxx,v 1.9.2.12.2.13 2004/11/20 03:19:58 easysw Exp $".
 //
