@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_Button.cxx,v 1.4.2.6 2001/01/22 15:13:39 easysw Exp $"
+// "$Id: Fl_Button.cxx,v 1.4.2.6.2.23 2004/07/27 16:02:20 easysw Exp $"
 //
 // Button widget for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2004 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -26,6 +26,7 @@
 #include <FL/Fl.H>
 #include <FL/Fl_Button.H>
 #include <FL/Fl_Group.H>
+#include <FL/Fl_Window.H>
 
 // There are a lot of subclasses, named Fl_*_Button.  Some of
 // them are implemented by setting the type() value and testing it
@@ -35,8 +36,13 @@ int Fl_Button::value(int v) {
   v = v ? 1 : 0;
   oldval = v;
   clear_changed();
-  if (value_ != v) {value_ = v; redraw(); return 1;}
-  else return 0;
+  if (value_ != v) {
+    value_ = v;
+    redraw();
+    return 1;
+  } else {
+    return 0;
+  }
 }
 
 void Fl_Button::setonly() { // set this radio button on, turn others off
@@ -53,9 +59,11 @@ void Fl_Button::draw() {
   if (type() == FL_HIDDEN_BUTTON) return;
   Fl_Color col = value() ? selection_color() : color();
 //if (col == FL_GRAY && Fl::belowmouse()==this) col = FL_LIGHT1;
-  draw_box(value() ? (down_box()?down_box():down(box())) : box(), col);
+  draw_box(value() ? (down_box()?down_box():fl_down(box())) : box(), col);
   draw_label();
+  if (Fl::focus() == this) draw_focus();
 }
+
 int Fl_Button::handle(int event) {
   int newval;
   switch (event) {
@@ -64,6 +72,7 @@ int Fl_Button::handle(int event) {
 //  if ((value_?selection_color():color())==FL_GRAY) redraw();
     return 1;
   case FL_PUSH:
+    if (Fl::visible_focus() && handle(FL_FOCUS)) Fl::focus(this);
   case FL_DRAG:
     if (Fl::event_inside(this)) {
       if (type() == FL_RADIO_BUTTON) newval = 1;
@@ -72,6 +81,7 @@ int Fl_Button::handle(int event) {
       newval = oldval;
     if (newval != value_) {
       value_ = newval;
+      set_changed();
       redraw();
       if (when() & FL_WHEN_CHANGED) do_callback();
     }
@@ -81,35 +91,65 @@ int Fl_Button::handle(int event) {
       if (when() & FL_WHEN_NOT_CHANGED) do_callback();
       return 1;
     }
-    if (type() == FL_RADIO_BUTTON)
-      setonly();
-    else if (type() == FL_TOGGLE_BUTTON)
-      oldval = value_;
+    set_changed();
+    if (type() == FL_RADIO_BUTTON) setonly();
+    else if (type() == FL_TOGGLE_BUTTON) oldval = value_;
     else {
       value(oldval);
       if (when() & FL_WHEN_CHANGED) do_callback();
     }
-    if (when() & FL_WHEN_RELEASE) do_callback(); else set_changed();
+    if (when() & FL_WHEN_RELEASE) do_callback();
     return 1;
   case FL_SHORTCUT:
     if (!(shortcut() ?
 	  Fl::test_shortcut(shortcut()) : test_shortcut())) return 0;
+    
+    if (Fl::visible_focus() && handle(FL_FOCUS)) Fl::focus(this);
+
     if (type() == FL_RADIO_BUTTON && !value_) {
       setonly();
+      set_changed();
       if (when() & FL_WHEN_CHANGED) do_callback();
     } else if (type() == FL_TOGGLE_BUTTON) {
       value(!value());
+      set_changed();
       if (when() & FL_WHEN_CHANGED) do_callback();
-    }
-    if (when() & FL_WHEN_RELEASE) do_callback(); else set_changed();
+    } else if (when() & FL_WHEN_RELEASE) do_callback();
     return 1;
+  case FL_FOCUS :
+  case FL_UNFOCUS :
+    if (Fl::visible_focus()) {
+      if (box() == FL_NO_BOX) {
+	// Widgets with the FL_NO_BOX boxtype need a parent to
+	// redraw, since it is responsible for redrawing the
+	// background...
+	int X = x() > 0 ? x() - 1 : 0;
+	int Y = y() > 0 ? y() - 1 : 0;
+	if (window()) window()->damage(FL_DAMAGE_ALL, X, Y, w() + 2, h() + 2);
+      } else redraw();
+      return 1;
+    } else return 0;
+  case FL_KEYBOARD :
+    if (Fl::focus() == this && Fl::event_key() == ' ' &&
+        !(Fl::event_state() & (FL_SHIFT | FL_CTRL | FL_ALT | FL_META))) {
+      set_changed();
+      if (type() == FL_RADIO_BUTTON && !value_) {
+	setonly();
+	if (when() & FL_WHEN_CHANGED) do_callback();
+      } else if (type() == FL_TOGGLE_BUTTON) {
+	value(!value());
+	if (when() & FL_WHEN_CHANGED) do_callback();
+      }
+      if (when() & FL_WHEN_RELEASE) do_callback();
+      return 1;
+    }
   default:
     return 0;
   }
 }
 
-Fl_Button::Fl_Button(int x,int y,int w,int h, const char *l)
-: Fl_Widget(x,y,w,h,l) {
+Fl_Button::Fl_Button(int X, int Y, int W, int H, const char *l)
+: Fl_Widget(X,Y,W,H,l) {
   box(FL_UP_BOX);
   down_box(FL_NO_BOX);
   value_ = oldval = 0;
@@ -118,5 +158,5 @@ Fl_Button::Fl_Button(int x,int y,int w,int h, const char *l)
 }
 
 //
-// End of "$Id: Fl_Button.cxx,v 1.4.2.6 2001/01/22 15:13:39 easysw Exp $".
+// End of "$Id: Fl_Button.cxx,v 1.4.2.6.2.23 2004/07/27 16:02:20 easysw Exp $".
 //
