@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_Check_Button.cxx,v 1.4.2.3 2001/01/22 15:13:39 easysw Exp $"
+// "$Id: Fl_Check_Button.cxx,v 1.4.2.3.2.6 2004/04/11 04:38:57 easysw Exp $"
 //
 // Check button widget for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2004 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -30,9 +30,9 @@
 // diamond is smaller than the widget size and can be surchecked by
 // another box type, for compatability with Forms.
 
-Fl_Check_Button::Fl_Check_Button(int x, int y, int w, int h, const char *l)
-: Fl_Light_Button(x, y, w, h, l) {
+Fl_Check_Button::Fl_Check_Button(int X, int Y, int W, int H, const char *l)
+: Fl_Light_Button(X, Y, W, H, l) {
   box(FL_NO_BOX);
-  down_box(FL_DIAMOND_DOWN_BOX);
-  selection_color(FL_RED);
+  down_box(FL_DOWN_BOX);
+  selection_color(FL_BLACK);
 }
