@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_Input_.cxx,v 1.21.2.11 2001/01/22 15:13:39 easysw Exp $"
+// "$Id: Fl_Input_.cxx,v 1.21.2.11.2.32 2004/09/21 13:35:39 easysw Exp $"
 //
 // Common input widget routines for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2004 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -30,23 +30,19 @@
 
 #include <FL/Fl.H>
 #include <FL/Fl_Input_.H>
+#include <FL/Fl_Window.H>
 #include <FL/fl_draw.H>
+#include <FL/fl_ask.H>
 #include <math.h>
-#include <string.h>
+#include "flstring.h"
 #include <stdlib.h>
 #include <ctype.h>
 
 #define MAXBUF 1024
 
-////////////////////////////////////////////////////////////////
+extern void fl_draw(const char*, int, float, float);
 
-// Wordwrap kludge.  This works ok but is commented out because
-// we want to move multi-line functionality to it's own subclass.
-// To try the wordwrap define this:
-//#define WORDWRAP
-#ifdef WORDWRAP
-static int wordwrap; // width for wordwrapping, should be are to Fl_Input_.
-#endif
+////////////////////////////////////////////////////////////////
 
 // Copy string p..e to the buffer, replacing characters with ^X and \nnn
 // as necessary.  Truncate if necessary so the resulting string and
@@ -54,20 +50,20 @@ static int wordwrap; // width for wordwrapping, should be are to Fl_Input_.
 const char* Fl_Input_::expand(const char* p, char* buf) const {
   char* o = buf;
   char* e = buf+(MAXBUF-4);
-#ifdef WORDWRAP
   const char* lastspace = p;
   char* lastspace_out = o;
   int width_to_lastspace = 0;
   int word_count = 0;
-#endif
-  if (type()==FL_SECRET_INPUT) {
+  int word_wrap;
+
+  if (input_type()==FL_SECRET_INPUT) {
     while (o<e && p < value_+size_) {*o++ = '*'; p++;}
   } else while (o<e) {
-#ifdef WORDWRAP
-    if (wordwrap && (p >= value_+size_ || isspace(*p))) {
+    if (wrap() && (p >= value_+size_ || isspace(*p))) {
+      word_wrap = w() - Fl::box_dw(box()) - 2;
       width_to_lastspace += (int)fl_width(lastspace_out, o-lastspace_out);
       if (p > lastspace+1) {
-	if (word_count && width_to_lastspace > wordwrap) {
+	if (word_count && width_to_lastspace > word_wrap) {
 	  p = lastspace; o = lastspace_out; break;
 	}
 	word_count++;
@@ -75,22 +71,17 @@ const char* Fl_Input_::expand(const char* p, char* buf) const {
       lastspace = p;
       lastspace_out = o;
     }
-#endif
+
     if (p >= value_+size_) break;
     int c = *p++ & 255;
     if (c < ' ' || c == 127) {
-      if (c=='\n' && type()==FL_MULTILINE_INPUT) {p--; break;}
-      if (c == '\t' && type()==FL_MULTILINE_INPUT) {
+      if (c=='\n' && input_type()==FL_MULTILINE_INPUT) {p--; break;}
+      if (c == '\t' && input_type()==FL_MULTILINE_INPUT) {
 	for (c = (o-buf)%8; c<8 && o<e; c++) *o++ = ' ';
       } else {
 	*o++ = '^';
 	*o++ = c ^ 0x40;
       }
-    } else if (c >= 128 && c < 0xA0) {
-      *o++ = '\\';
-      *o++ = (c>>6)+'0';
-      *o++ = ((c>>3)&7)+'0';
-      *o++ = (c&7)+'0';
     } else if (c == 0xA0) { // nbsp
       *o++ = ' ';
     } else {
@@ -109,11 +100,11 @@ double Fl_Input_::expandpos(
   int* returnn		// return offset into buf here
 ) const {
   int n = 0;
-  if (type()==FL_SECRET_INPUT) n = e-p;
+  if (input_type()==FL_SECRET_INPUT) n = e-p;
   else while (p<e) {
     int c = *p++ & 255;
     if (c < ' ' || c == 127) {
-      if (c == '\t' && type()==FL_MULTILINE_INPUT) n += 8-(n%8);
+      if (c == '\t' && input_type()==FL_MULTILINE_INPUT) n += 8-(n%8);
       else n += 2;
     } else if (c >= 128 && c < 0xA0) {
       n += 4;
@@ -140,6 +131,7 @@ void Fl_Input_::minimal_update(int p) {
   } else {
     mu_p = p;
   }
+
   damage(FL_DAMAGE_EXPOSE);
   erase_cursor_only = 0;
 }
@@ -159,12 +151,12 @@ void Fl_Input_::setfont() const {
 }
 
 void Fl_Input_::drawtext(int X, int Y, int W, int H) {
-
   int do_mu = !(damage()&FL_DAMAGE_ALL);
+
   if (Fl::focus()!=this && !size()) {
     if (do_mu) { // we have to erase it if cursor was there
-      fl_color(color());
-      fl_rectf(X, Y, W, H);
+      draw_box(box(), X-Fl::box_dx(box()), Y-Fl::box_dy(box()),
+               W+Fl::box_dw(box()), H+Fl::box_dh(box()), color());
     }
     return;
   }
@@ -179,10 +171,6 @@ void Fl_Input_::drawtext(int X, int Y, int W, int H) {
   }
 
   setfont();
-#ifdef WORDWRAP
-  if (type()==FL_MULTILINE_INPUT) wordwrap = W; else wordwrap = 0;
-#endif
-
   const char *p, *e;
   char buf[MAXBUF];
 
@@ -220,7 +208,7 @@ void Fl_Input_::drawtext(int X, int Y, int W, int H) {
   }
 
   // adjust the scrolling:
-  if (type()==FL_MULTILINE_INPUT) {
+  if (input_type()==FL_MULTILINE_INPUT) {
     int newy = yscroll_;
     if (cury < newy) newy = cury;
     if (cury > newy+H-height) newy = cury-H+height;
@@ -231,11 +219,12 @@ void Fl_Input_::drawtext(int X, int Y, int W, int H) {
   }
 
   fl_clip(X, Y, W, H);
-  Fl_Color color = active_r() ? textcolor() : inactive(textcolor());
+  Fl_Color tc = active_r() ? textcolor() : fl_inactive(textcolor());
 
   p = value();
   // visit each line and draw it:
   int desc = height-fl_descent();
+  float xpos = X - xscroll_ + 1;
   int ypos = -yscroll_;
   for (; ypos < H;) {
 
@@ -246,64 +235,90 @@ void Fl_Input_::drawtext(int X, int Y, int W, int H) {
 
     if (do_mu) {	// for minimal update:
       const char* pp = value()+mu_p; // pointer to where minimal update starts
-      if (e >= pp && (!erase_cursor_only || p <= pp)) { // we must erase this
-	// calculate area to erase:
-	int x1 = -xscroll_;
-	if (p < pp) x1 += int(expandpos(p, pp, buf, 0));
-	// erase it:
-	fl_color(this->color());
-	fl_rectf(X+x1, Y+ypos, erase_cursor_only?2:W-x1, height);
-	// it now draws entire line over it
-	// this should not draw letters to left of erased area, but
-	// that is nyi.
+      if (e < pp) goto CONTINUE2; // this line is before the changes
+      if (readonly()) erase_cursor_only = 0; // this isn't the most efficient way
+      if (erase_cursor_only && p > pp) goto CONTINUE2; // this line is after
+      // calculate area to erase:
+      float r = X+W;
+      float xx;
+      if (p >= pp) {
+	xx = X;
+	if (erase_cursor_only) r = xpos+2;
+	else if (readonly()) xx -= 3;
+      } else {
+	xx = xpos+expandpos(p, pp, buf, 0);
+	if (erase_cursor_only) r = xx+2;
+	else if (readonly()) xx -= 3;
       }
+      // clip to and erase it:
+      fl_push_clip((int)xx, Y+ypos, (int)(r-xx+1), height);
+      draw_box(box(), X-Fl::box_dx(box()), Y-Fl::box_dy(box()),
+               W+Fl::box_dw(box()), H+Fl::box_dh(box()), color());
+      // it now draws entire line over it
+      // this should not draw letters to left of erased area, but
+      // that is nyi.
     }
 
     // Draw selection area if required:
     if (selstart < selend && selstart <= e-value() && selend > p-value()) {
       const char* pp = value()+selstart;
-      int x1 = -xscroll_;
+      float x1 = xpos;
       int offset1 = 0;
       if (pp > p) {
-	fl_color(color);
-	x1 += int(expandpos(p, pp, buf, &offset1));
-	fl_draw(buf, offset1, X-xscroll_, Y+ypos+desc);
+	fl_color(tc);
+	x1 += expandpos(p, pp, buf, &offset1);
+	fl_draw(buf, offset1, xpos, (float)(Y+ypos+desc));
       }
       pp = value()+selend;
-      int x2 = W;
+      float x2 = X+W;
       int offset2;
-      if (pp <= e) x2 = int(expandpos(p, pp, buf, &offset2))-xscroll_;
+      if (pp <= e) x2 = xpos+expandpos(p, pp, buf, &offset2);
       else offset2 = strlen(buf);
       fl_color(selection_color());
-      fl_rectf(X+int(x1+.5), Y+ypos, int(x2-x1), height);
-      fl_color(contrast(textcolor(), selection_color()));
-      fl_draw(buf+offset1, offset2-offset1, X+x1, Y+ypos+desc);
+      fl_rectf((int)(x1+0.5), Y+ypos, (int)(x2-x1+0.5), height);
+      fl_color(fl_contrast(textcolor(), selection_color()));
+      fl_draw(buf+offset1, offset2-offset1, x1, (float)(Y+ypos+desc));
       if (pp < e) {
-	fl_color(color);
-	fl_draw(buf+offset2, X+x2, Y+ypos+desc);
+	fl_color(tc);
+	fl_draw(buf+offset2, strlen(buf+offset2), x2, (float)(Y+ypos+desc));
       }
     } else {
-      // draw the cursor:
-      if (Fl::focus() == this && selstart == selend &&
-	  position() >= p-value() && position() <= e-value()) {
-	fl_color(cursor_color());
-	fl_rectf(X+curx-xscroll_, Y+ypos, 2, height);
-      }
-      fl_color(color);
-      fl_draw(buf, X-xscroll_, Y+ypos+desc);
+      // draw unselected text
+      fl_color(tc);
+      fl_draw(buf, strlen(buf), xpos, (float)(Y+ypos+desc));
     }
+
+    if (do_mu) fl_pop_clip();
+
+  CONTINUE2:
+    // draw the cursor:
+    if (Fl::focus() == this && selstart == selend &&
+	position() >= p-value() && position() <= e-value()) {
+      fl_color(cursor_color());
+      if (readonly()) {
+        fl_line((int)(xpos+curx-2.5f), Y+ypos+height-1,
+	        (int)(xpos+curx+0.5f), Y+ypos+height-4,
+	        (int)(xpos+curx+3.5f), Y+ypos+height-1);
+      } else {
+        fl_rectf((int)(xpos+curx+0.5), Y+ypos, 2, height);
+      }
+    }
+
   CONTINUE:
     ypos += height;
     if (e >= value_+size_) break;
-    p = e+1;
+    if (*e == '\n' || *e == ' ') e++;
+    p = e;
   }
 
   // for minimal update, erase all lines below last one if necessary:
-  if (type()==FL_MULTILINE_INPUT && do_mu && ypos<H
+  if (input_type()==FL_MULTILINE_INPUT && do_mu && ypos<H
       && (!erase_cursor_only || p <= value()+mu_p)) {
     if (ypos < 0) ypos = 0;
-    fl_color(this->color());
-    fl_rectf(X, Y+ypos, W, H-ypos);
+    fl_push_clip(X, Y+ypos, W, H-ypos);
+    draw_box(box(), X-Fl::box_dx(box()), Y-Fl::box_dy(box()),
+             W+Fl::box_dw(box()), H+Fl::box_dh(box()), color());
+    fl_pop_clip();
   }
 
   fl_pop_clip();
@@ -314,14 +329,14 @@ static int isword(char c) {
 }
 
 int Fl_Input_::word_end(int i) const {
-  if (type() == FL_SECRET_INPUT) return size();
+  if (input_type() == FL_SECRET_INPUT) return size();
   //while (i < size() && !isword(index(i))) i++;
   while (i < size() && isword(index(i))) i++;
   return i;
 }
 
 int Fl_Input_::word_start(int i) const {
-  if (type() == FL_SECRET_INPUT) return 0;
+  if (input_type() == FL_SECRET_INPUT) return 0;
 //   if (i >= size() || !isword(index(i)))
 //     while (i > 0 && !isword(index(i-1))) i--;
   while (i > 0 && isword(index(i-1))) i--;
@@ -329,52 +344,43 @@ int Fl_Input_::word_start(int i) const {
 }
 
 int Fl_Input_::line_end(int i) const {
-  if (type() != FL_MULTILINE_INPUT) return size();
-#ifdef WORDWRAP
-  // go to the start of the paragraph:
-  int j = i;
-  while (j > 0 && index(j-1) != '\n') j--;
-  // now measure lines until we get past i, end of that line is real eol:
-  wordwrap = w()-Fl::box_dw(box())-6;
-  setfont();
-  for (const char* p=value()+j; ;) {
-    char buf[MAXBUF];
-    p = expand(p, buf);
-    if (p-value() >= i) return p-value();
-    p++;
+  if (input_type() != FL_MULTILINE_INPUT) return size();
+
+  if (wrap()) {
+    // go to the start of the paragraph:
+    int j = i;
+    while (j > 0 && index(j-1) != '\n') j--;
+    // now measure lines until we get past i, end of that line is real eol:
+    setfont();
+    for (const char* p=value()+j; ;) {
+      char buf[MAXBUF];
+      p = expand(p, buf);
+      if (p-value() >= i) return p-value();
+      p++;
+    }
+  } else {
+    while (i < size() && index(i) != '\n') i++;
+    return i;
   }
-#else
-  while (i < size() && index(i) != '\n') i++;
-  return i;
-#endif
 }
 
 int Fl_Input_::line_start(int i) const {
-  if (type() != FL_MULTILINE_INPUT) return 0;
+  if (input_type() != FL_MULTILINE_INPUT) return 0;
   int j = i;
   while (j > 0 && index(j-1) != '\n') j--;
-#ifdef WORDWRAP
-  // now measure lines until we get past i, start of that line is real eol:
-  wordwrap = w()-Fl::box_dw(box())-6;
-  setfont();
-  for (const char* p=value()+j; ;) {
-    char buf[MAXBUF];
-    const char* e = expand(p, buf);
-    if (e-value() >= i) return p-value();
-    p = e+1;
-  }
-#else
-  return j;
-#endif
+  if (wrap()) {
+    // now measure lines until we get past i, start of that line is real eol:
+    setfont();
+    for (const char* p=value()+j; ;) {
+      char buf[MAXBUF];
+      const char* e = expand(p, buf);
+      if (e-value() >= i) return p-value();
+      p = e+1;
+    }
+  } else return j;
 }
 
-void Fl_Input_::handle_mouse(int X, int Y,
-#ifdef WORDWRAP
-			     int W,
-#else
-			     int /*W*/,
-#endif
-			     int /*H*/, int drag) {
+void Fl_Input_::handle_mouse(int X, int Y, int /*W*/, int /*H*/, int drag) {
   was_up_down = 0;
   if (!size()) return;
   setfont();
@@ -382,13 +388,10 @@ void Fl_Input_::handle_mouse(int X, int Y,
   const char *p, *e;
   char buf[MAXBUF];
 
-  int theline = (type()==FL_MULTILINE_INPUT) ?
+  int theline = (input_type()==FL_MULTILINE_INPUT) ?
     (Fl::event_y()-Y+yscroll_)/fl_height() : 0;
 
   int newpos = 0;
-#ifdef WORDWRAP
-  if (type()==FL_MULTILINE_INPUT) wordwrap = W; else wordwrap = 0;
-#endif
   for (p=value();; ) {
     e = expand(p, buf);
     theline--; if (theline < 0) break;
@@ -475,11 +478,6 @@ int Fl_Input_::up_down_position(int i, int keepmark) {
   // unlike before, i must be at the start of the line already!
 
   setfont();
-#ifdef WORDWRAP
-  if (type()==FL_MULTILINE_INPUT)
-    wordwrap = w()-Fl::box_dw(box())-6;
-  else wordwrap = 0;
-#endif
   char buf[MAXBUF];
   const char* p = value()+i;
   const char* e = expand(p, buf);
@@ -495,14 +493,13 @@ int Fl_Input_::up_down_position(int i, int keepmark) {
   return j;
 }
 
-int Fl_Input_::copy() {
-  if (mark() != position()) {
-    int b, e; if (position() < mark()) {
-      b = position(); e = mark();
-    } else {
-      e = position(); b = mark();
-    }
-    Fl::selection(*this, value()+b, (type()!=FL_SECRET_INPUT) ? e-b : 0);
+int Fl_Input_::copy(int clipboard) {
+  int b = position();
+  int e = mark();
+  if (b != e) {
+    if (b > e) {b = mark(); e = position();}
+    if (input_type() == FL_SECRET_INPUT) e = b;
+    Fl::copy(value()+b, e-b, clipboard);
     return 1;
   }
   return 0;
@@ -571,7 +568,7 @@ int Fl_Input_::replace(int b, int e, const char* text, int ilen) {
     size_ -= e-b;
     undowidget = this;
     undoat = b;
-    if (type() == FL_SECRET_INPUT) yankcut = 0; else yankcut = undocut;
+    if (input_type() == FL_SECRET_INPUT) yankcut = 0; else yankcut = undocut;
   }
 
   if (ilen) {
@@ -588,15 +585,13 @@ int Fl_Input_::replace(int b, int e, const char* text, int ilen) {
   undowidget = this;
   undoat = b+ilen;
 
-#ifdef WORDWRAP
   // Insertions into the word at the end of the line will cause it to
   // wrap to the next line, so we must indicate that the changes may start
   // right after the whitespace before the current word.  This will
   // result in sub-optimal update when such wrapping does not happen
   // but it is too hard to figure out for now...
-  if (type() == FL_MULTILINE_INPUT)
+  if (wrap())
     while (b > 0 && !isspace(index(b))) b--;
-#endif
 
   // make sure we redraw the old selection or cursor:
   if (mark_ < b) b = mark_;
@@ -606,7 +601,8 @@ int Fl_Input_::replace(int b, int e, const char* text, int ilen) {
 
   mark_ = position_ = undoat;
 
-  if (when()&FL_WHEN_CHANGED) do_callback(); else set_changed();
+  set_changed();
+  if (when()&FL_WHEN_CHANGED) do_callback();
   return 1;
 }
 
@@ -643,33 +639,35 @@ int Fl_Input_::undo() {
   position_ = b;
 
   minimal_update(b1);
-  if (when()&FL_WHEN_CHANGED) do_callback(); else set_changed();
+  set_changed();
+  if (when()&FL_WHEN_CHANGED) do_callback();
   return 1;
 }
 
-#if 0
-int Fl_Input_::yank() {
-  // fake yank by trying to get it out of undobuffer
-  if (!yankcut) return 0;
-  return change(position(), position(), undobuffer, yankcut);
-}
-#endif
-
 int Fl_Input_::copy_cuts() {
   // put the yank buffer into the X clipboard
-  if (!yankcut) return 0;
-  Fl::selection(*this, undobuffer, yankcut);
+  if (!yankcut || input_type()==FL_SECRET_INPUT) return 0;
+  Fl::copy(undobuffer, yankcut, 1);
   return 1;
 }
 
 void Fl_Input_::maybe_do_callback() {
   if (changed() || (when()&FL_WHEN_NOT_CHANGED)) {
-    clear_changed(); do_callback();}
+    do_callback();
+  }
 }
 
 int Fl_Input_::handletext(int event, int X, int Y, int W, int H) {
   switch (event) {
 
+  case FL_ENTER:
+    if (active_r() && window()) window()->cursor(FL_CURSOR_INSERT);
+    return 1;
+
+  case FL_LEAVE:
+    if (active_r() && window()) window()->cursor(FL_CURSOR_DEFAULT);
+    return 1;
+
   case FL_FOCUS:
     if (mark_ == position_) {
       minimal_update(size()+1);
@@ -688,6 +686,11 @@ int Fl_Input_::handletext(int event, int X, int Y, int W, int H) {
 
   case FL_PUSH:
     handle_mouse(X, Y, W, H, Fl::event_state(FL_SHIFT));
+
+    if (Fl::focus() != this) {
+      Fl::focus(this);
+      handle(FL_FOCUS);
+    }
     return 1;
 
   case FL_DRAG:
@@ -695,20 +698,57 @@ int Fl_Input_::handletext(int event, int X, int Y, int W, int H) {
     return 1;
 
   case FL_RELEASE:
-//  handle_mouse(X, Y, W, H, 1);
-    copy();
+    copy(0);
     return 1;
 
-//   case FL_SELECTIONCLEAR:
-//     minimal_update(mark_, position_);
-//     mark_ = position_;
-//     return 1;
-
   case FL_PASTE: {
+    // Don't allow pastes into readonly widgets...
+    if (readonly()) {
+      fl_beep(FL_BEEP_ERROR);
+      return 1;
+    }
+
+    // See if we have anything to paste...
+    if (!Fl::event_text() || !Fl::event_length()) return 1;
+
     // strip trailing control characters and spaces before pasting:
     const char* t = Fl::event_text();
     const char* e = t+Fl::event_length();
-    if (type() != FL_MULTILINE_INPUT) while (e > t && isspace(*(e-1))) e--;
+    if (input_type() != FL_MULTILINE_INPUT) while (e > t && isspace(*(e-1))) e--;
+    if (!t || e <= t) return 1; // Int/float stuff will crash without this test
+    if (input_type() == FL_INT_INPUT) {
+      while (isspace(*t) && t < e) t ++;
+      const char *p = t;
+      if (*p == '+' || *p == '-') p ++;
+      if (strncmp(p, "0x", 2) == 0) {
+        p += 2;
+        while (isxdigit(*p) && p < e) p ++;
+      } else {
+        while (isdigit(*p) && p < e) p ++;
+      }
+      if (p < e) {
+        fl_beep(FL_BEEP_ERROR);
+        return 1;
+      } else return replace(0, size(), t, e - t);
+    } else if (input_type() == FL_FLOAT_INPUT) {
+      while (isspace(*t) && t < e) t ++;
+      const char *p = t;
+      if (*p == '+' || *p == '-') p ++;
+      while (isdigit(*p) && p < e) p ++;
+      if (*p == '.') {
+        p ++;
+        while (isdigit(*p) && p < e) p ++;
+	if (*p == 'e' || *p == 'E') {
+	  p ++;
+	  if (*p == '+' || *p == '-') p ++;
+	  while (isdigit(*p) && p < e) p ++;
+	}
+      }
+      if (p < e) {
+        fl_beep(FL_BEEP_ERROR);
+        return 1;
+      } else return replace(0, size(), t, e - t);
+    }
     return replace(position(), mark(), t, e-t);}
 
   default:
@@ -718,15 +758,15 @@ int Fl_Input_::handletext(int event, int X, int Y, int W, int H) {
 
 /*------------------------------*/
 
-Fl_Input_::Fl_Input_(int x, int y, int w, int h, const char* l)
-: Fl_Widget(x, y, w, h, l) {
+Fl_Input_::Fl_Input_(int X, int Y, int W, int H, const char* l)
+: Fl_Widget(X, Y, W, H, l) {
   box(FL_DOWN_BOX);
-  color(FL_WHITE, FL_SELECTION_COLOR);
+  color(FL_BACKGROUND2_COLOR, FL_SELECTION_COLOR);
   align(FL_ALIGN_LEFT);
-  textsize_ = FL_NORMAL_SIZE;
+  textsize_ = (uchar)FL_NORMAL_SIZE;
   textfont_ = FL_HELVETICA;
-  textcolor_ = FL_BLACK;
-  cursor_color_ = FL_BLACK; // was FL_BLUE
+  textcolor_ = FL_FOREGROUND_COLOR;
+  cursor_color_ = FL_FOREGROUND_COLOR; // was FL_BLUE
   mark_ = position_ = size_ = 0;
   bufsize = 0;
   buffer  = 0;
@@ -753,6 +793,15 @@ void Fl_Input_::put_in_buffer(int len) {
     } else {
       bufsize = len+1;
     }
+    // Note: the following code is equivalent to:
+    //
+    //   if (moveit) value_ = value_ - buffer;
+    //   char* nbuffer = (char*)realloc(buffer, bufsize);
+    //   if (moveit) value_ = value_ + nbuffer;
+    //   buffer = nbuffer;
+    //
+    // We just optimized the pointer arithmetic for value_...
+    //
     char* nbuffer = (char*)realloc(buffer, bufsize);
     if (moveit) value_ += (nbuffer-buffer);
     buffer = nbuffer;
@@ -787,7 +836,7 @@ int Fl_Input_::static_value(const char* str, int len) {
     xscroll_ = yscroll_ = 0;
     minimal_update(0);
   }
-  position(0, size());
+  position(0, readonly() ? 0 : size());
   return 1;
 }
 
@@ -817,5 +866,5 @@ Fl_Input_::~Fl_Input_() {
 }
 
 //
-// End of "$Id: Fl_Input_.cxx,v 1.21.2.11 2001/01/22 15:13:39 easysw Exp $".
+// End of "$Id: Fl_Input_.cxx,v 1.21.2.11.2.32 2004/09/21 13:35:39 easysw Exp $".
 //
