@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_Menu_Bar.cxx,v 1.7 1999/01/19 19:05:39 mike Exp $"
+// "$Id: Fl_Menu_Bar.cxx,v 1.7.2.6.2.5 2004/06/07 19:23:12 matthiaswm Exp $"
 //
 // Menu bar widget for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-1999 by Bill Spitzak and others.
+// Copyright 1998-2004 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,20 +20,32 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@easysw.com".
+// Please report all bugs and problems to "fltk-bugs@fltk.org".
 //
 
 #include <FL/Fl.H>
 #include <FL/Fl_Menu_Bar.H>
+#include <FL/fl_draw.H>
 
 void Fl_Menu_Bar::draw() {
   draw_box();
   if (!menu() || !menu()->text) return;
   const Fl_Menu_Item* m;
   int X = x()+6;
-  for (m=menu(); m->text; m = m->next()) {
-    m->draw(X, y(), 0, h(), this);
-    X += m->measure(0,this) + 16;
+  for (m=menu()->first(); m->text; m = m->next()) {
+    int W = m->measure(0,this) + 16;
+    m->draw(X, y(), W, h(), this);
+    X += W;
+    if (m->flags & FL_MENU_DIVIDER) {
+      int y1 = y() + Fl::box_dy(box());
+      int y2 = y1 + h() - Fl::box_dh(box()) - 1;
+
+      // Draw a vertical divider between menus...
+      fl_color(FL_DARK3);
+      fl_yxline(X - 6, y1, y2);
+      fl_color(FL_LIGHT3);
+      fl_yxline(X - 5, y1, y2);
+    }
   }
 }
 
@@ -50,14 +62,15 @@ int Fl_Menu_Bar::handle(int event) {
     picked(v);
     return 1;
   case FL_SHORTCUT:
-    v = menu()->test_shortcut();
-    if (v) {picked(v); return 1;}
-    if (visible_r() && (v = menu()->find_shortcut())) goto J1;
-    return 0;
+    if (visible_r()) {
+      v = menu()->find_shortcut();
+      if (v && v->submenu()) goto J1;
+    }
+    return test_shortcut() != 0;
   }
   return 0;
 }
 
 //
-// End of "$Id: Fl_Menu_Bar.cxx,v 1.7 1999/01/19 19:05:39 mike Exp $".
+// End of "$Id: Fl_Menu_Bar.cxx,v 1.7.2.6.2.5 2004/06/07 19:23:12 matthiaswm Exp $".
 //
