@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_Menu_add.cxx,v 1.9.2.11 2001/02/12 15:34:37 easysw Exp $"
+// "$Id: Fl_Menu_add.cxx,v 1.9.2.13.2.8 2004/04/11 04:38:58 easysw Exp $"
 //
 // Menu utilities for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2004 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -34,7 +34,7 @@
 // string with a % sign in it!
 
 #include <FL/Fl_Menu_.H>
-#include <string.h>
+#include "flstring.h"
 #include <stdio.h>
 #include <stdlib.h>
 
@@ -99,7 +99,7 @@ static int compare(const char* a, const char* b) {
 // now add submenu titles directly by setting SUBMENU in the flags):
 int Fl_Menu_Item::add(
   const char *mytext,
-  int shortcut,
+  int sc,
   Fl_Callback *cb,	
   void *data,
   int myflags
@@ -110,18 +110,25 @@ int Fl_Menu_Item::add(
   char *q;
   char buf[1024];
 
-  int size = array==local_array ? local_array_size : array->size();
+  int msize = array==local_array ? local_array_size : array->size();
   int flags1 = 0;
-  char* item;
-  for (;;) {    /* do all the supermenus: */
+  const char* item;
 
-    /* fill in the buf with name, changing \x to x: */
+  // split at slashes to make submenus:
+  for (;;) {
+
+    // leading slash makes us assumme it is a filename:
+    if (*mytext == '/') {item = mytext; break;}
+
+    // leading underscore causes divider line:
+    if (*mytext == '_') {mytext++; flags1 = FL_MENU_DIVIDER;}
+
+    // copy to buf, changing \x to x:
     q = buf;
-    for (p=mytext; *p && *p != '/'; *q++ = *p++) if (*p=='\\') p++;
+    for (p=mytext; *p && *p != '/'; *q++ = *p++) if (*p=='\\' && p[1]) p++;
     *q = 0;
 
     item = buf;
-    if (*item == '_') {item++; flags1 = FL_MENU_DIVIDER;}
     if (*p != '/') break; /* not a menu title */
     mytext = p+1;	/* point at item title */
 
@@ -131,10 +138,10 @@ int Fl_Menu_Item::add(
 
     if (!m->text) { /* create a new menu */
       int n = m-array;
-      array = insert(array, size, n, item, FL_SUBMENU|flags1);
-      size++;
-      array = insert(array, size, n+1, 0, 0);
-      size++;
+      array = insert(array, msize, n, item, FL_SUBMENU|flags1);
+      msize++;
+      array = insert(array, msize, n+1, 0, 0);
+      msize++;
       m = array+n;
     }
     m++;	/* go into the submenu */
@@ -143,26 +150,26 @@ int Fl_Menu_Item::add(
 
   /* find a matching menu item: */
   for (; m->text; m = m->next())
-    if (!compare(m->text,item)) break;
+    if (!(m->flags&FL_SUBMENU) && !compare(m->text,item)) break;
 
   if (!m->text) {	/* add a new menu item */
     int n = m-array;
-    array = insert(array, size, n, item, myflags|flags1);
-    size++;
-    if (flags & FL_SUBMENU) { // add submenu delimiter
-      array = insert(array, size, n+1, 0, 0);
-      size++;
+    array = insert(array, msize, n, item, myflags|flags1);
+    msize++;
+    if (myflags & FL_SUBMENU) { // add submenu delimiter
+      array = insert(array, msize, n+1, 0, 0);
+      msize++;
     }
     m = array+n;
   }
 
   /* fill it in */
-  m->shortcut_ = shortcut;
+  m->shortcut_ = sc;
   m->callback_ = cb;
   m->user_data_ = data;
   m->flags = myflags|flags1;
 
-  if (array == local_array) local_array_size = size;
+  if (array == local_array) local_array_size = msize;
   return m-array;
 }
 
@@ -212,17 +219,17 @@ int Fl_Menu_::add(const char *t, int s, Fl_Callback *c,void *v,int f) {
 // adds many menu items, with '|' seperating the menu items, and tab
 // seperating the menu item names from an optional shortcut string.
 int Fl_Menu_::add(const char *str) {
-  char buf[128];
+  char buf[1024];
   int r = 0;
   while (*str) {
-    int shortcut = 0;
+    int sc = 0;
     char *c;
-    for (c = buf; *str && *str != '|'; str++) {
-      if (*str == '\t') {*c++ = 0; shortcut = fl_old_shortcut(str);}
+    for (c = buf; c < (buf + sizeof(buf) - 2) && *str && *str != '|'; str++) {
+      if (*str == '\t') {*c++ = 0; sc = fl_old_shortcut(str);}
       else *c++ = *str;
     }
     *c = 0;
-    r = add(buf, shortcut, 0, 0, 0);
+    r = add(buf, sc, 0, 0, 0);
     if (*str) str++;
   }
   return r;
@@ -250,9 +257,10 @@ void Fl_Menu_::remove(int i) {
     for (Fl_Menu_Item* m = item; m < next_item; m++)
       if (m->text) free((void*)(m->text));
   }
-  memmove(item, next_item, ((menu_+n+1)-next_item)*sizeof(Fl_Menu_Item));
+  // MRS: "n" is the menu size(), which includes the trailing NULL entry...
+  memmove(item, next_item, (menu_+n-next_item)*sizeof(Fl_Menu_Item));
 }
 
 //
-// End of "$Id: Fl_Menu_add.cxx,v 1.9.2.11 2001/02/12 15:34:37 easysw Exp $".
+// End of "$Id: Fl_Menu_add.cxx,v 1.9.2.13.2.8 2004/04/11 04:38:58 easysw Exp $".
 //
