@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_Overlay_Window.cxx,v 1.7.2.6 2001/05/05 23:39:01 spitzak Exp $"
+// "$Id: Fl_Overlay_Window.cxx,v 1.7.2.6.2.4 2004/04/11 04:38:58 easysw Exp $"
 //
 // Overlay window code for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2004 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -53,7 +53,7 @@ void Fl_Overlay_Window::flush() {
   }
 #endif
   int erase_overlay = (damage()&FL_DAMAGE_OVERLAY);
-  clear_damage(damage()&~FL_DAMAGE_OVERLAY);
+  clear_damage((uchar)(damage()&~FL_DAMAGE_OVERLAY));
   Fl_Double_Window::flush(erase_overlay);
   if (overlay_ == this) draw_overlay();
 }
@@ -74,7 +74,7 @@ int Fl_Overlay_Window::can_do_overlay() {return 0;}
 
 void Fl_Overlay_Window::redraw_overlay() {
   overlay_ = this;
-  clear_damage(damage()|FL_DAMAGE_OVERLAY);
+  clear_damage((uchar)(damage()|FL_DAMAGE_OVERLAY));
   Fl::damage(FL_DAMAGE_CHILD);
 }
 
@@ -149,5 +149,5 @@ void Fl_Overlay_Window::redraw_overlay() {
 #endif
 
 //
-// End of "$Id: Fl_Overlay_Window.cxx,v 1.7.2.6 2001/05/05 23:39:01 spitzak Exp $".
+// End of "$Id: Fl_Overlay_Window.cxx,v 1.7.2.6.2.4 2004/04/11 04:38:58 easysw Exp $".
 //
