@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_Return_Button.cxx,v 1.5.2.3 2001/01/22 15:13:40 easysw Exp $"
+// "$Id: Fl_Return_Button.cxx,v 1.5.2.3.2.6 2004/04/11 04:38:58 easysw Exp $"
 //
 // Return button widget for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2004 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -47,12 +47,13 @@ int fl_return_arrow(int x, int y, int w, int h) {
 
 void Fl_Return_Button::draw() {
   if (type() == FL_HIDDEN_BUTTON) return;
-  draw_box(value() ? (down_box()?down_box():down(box())) : box(),
+  draw_box(value() ? (down_box()?down_box():fl_down(box())) : box(),
 	   value() ? selection_color() : color());
   int W = h();
   if (w()/3 < W) W = w()/3;
   fl_return_arrow(x()+w()-W-4, y(), W, h());
   draw_label(x(), y(), w()-W+4, h());
+  if (Fl::focus() == this) draw_focus();
 }
 
 int Fl_Return_Button::handle(int event) {
@@ -65,5 +66,5 @@ int Fl_Return_Button::handle(int event) {
 }
 
 //
-// End of "$Id: Fl_Return_Button.cxx,v 1.5.2.3 2001/01/22 15:13:40 easysw Exp $".
+// End of "$Id: Fl_Return_Button.cxx,v 1.5.2.3.2.6 2004/04/11 04:38:58 easysw Exp $".
 //
