@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_Roller.cxx,v 1.6.2.4 2001/01/22 15:13:40 easysw Exp $"
+// "$Id: Fl_Roller.cxx,v 1.6.2.4.2.9 2004/04/11 04:38:58 easysw Exp $"
 //
 // Roller widget for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2004 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -35,6 +35,7 @@ int Fl_Roller::handle(int event) {
   int newpos = horizontal() ? Fl::event_x() : Fl::event_y();
   switch (event) {
   case FL_PUSH:
+    if (Fl::visible_focus()) Fl::focus(this);
     handle_push();
     ipos = newpos;
     return 1;
@@ -44,6 +45,37 @@ int Fl_Roller::handle(int event) {
   case FL_RELEASE:
     handle_release();
     return 1;
+  case FL_KEYBOARD :
+    switch (Fl::event_key()) {
+      case FL_Up:
+        if (horizontal()) return 0;
+	handle_drag(clamp(increment(value(),-1)));
+	return 1;
+      case FL_Down:
+        if (horizontal()) return 0;
+	handle_drag(clamp(increment(value(),1)));
+	return 1;
+      case FL_Left:
+        if (!horizontal()) return 0;
+	handle_drag(clamp(increment(value(),-1)));
+	return 1;
+      case FL_Right:
+        if (!horizontal()) return 0;
+	handle_drag(clamp(increment(value(),1)));
+	return 1;
+      default:
+        return 0;
+    }
+    // break not required because of switch...
+  case FL_FOCUS :
+  case FL_UNFOCUS :
+    if (Fl::visible_focus()) {
+      redraw();
+      return 1;
+    } else return 0;
+  case FL_ENTER :
+  case FL_LEAVE :
+    return 1;
   default:
     return 0;
   }
@@ -73,13 +105,13 @@ void Fl_Roller::draw() {
     if (active_r()) {
       // draw ridges:
       double junk;
-      for (double y = -ARC+modf(offset*sin(ARC)/(W/2)/delta,&junk)*delta;;
-	   y += delta) {
-	int y1 = int((sin(y)/sin(ARC)+1)*W/2);
-	if (y1 <= 0) continue; else if (y1 >= W-1) break;
-	fl_color(FL_DARK3); fl_yxline(X+y1,Y+1,Y+H-1);
-	if (y < 0) y1--; else y1++;
-	fl_color(FL_LIGHT1);fl_yxline(X+y1,Y+1,Y+H-1);
+      for (double yy = -ARC+modf(offset*sin(ARC)/(W/2)/delta,&junk)*delta;;
+	   yy += delta) {
+	int yy1 = int((sin(yy)/sin(ARC)+1)*W/2);
+	if (yy1 <= 0) continue; else if (yy1 >= W-1) break;
+	fl_color(FL_DARK3); fl_yxline(X+yy1,Y+1,Y+H-1);
+	if (yy < 0) yy1--; else yy1++;
+	fl_color(FL_LIGHT1);fl_yxline(X+yy1,Y+1,Y+H-1);
       }
       // draw edges:
       h1 = W/8+1; // distance from end the color inverts
@@ -107,13 +139,13 @@ void Fl_Roller::draw() {
     if (active_r()) {
       // draw ridges:
       double junk;
-      for (double y = -ARC+modf(offset*sin(ARC)/(H/2)/delta,&junk)*delta;
-	   ; y += delta) {
-	int y1 = int((sin(y)/sin(ARC)+1)*H/2);
-	if (y1 <= 0) continue; else if (y1 >= H-1) break;
-	fl_color(FL_DARK3); fl_xyline(X+1,Y+y1,X+W-1);
-	if (y < 0) y1--; else y1++;
-	fl_color(FL_LIGHT1);fl_xyline(X+1,Y+y1,X+W-1);
+      for (double yy = -ARC+modf(offset*sin(ARC)/(H/2)/delta,&junk)*delta;
+	   ; yy += delta) {
+	int yy1 = int((sin(yy)/sin(ARC)+1)*H/2);
+	if (yy1 <= 0) continue; else if (yy1 >= H-1) break;
+	fl_color(FL_DARK3); fl_xyline(X+1,Y+yy1,X+W-1);
+	if (yy < 0) yy1--; else yy1++;
+	fl_color(FL_LIGHT1);fl_xyline(X+1,Y+yy1,X+W-1);
       }
       // draw edges:
       h1 = H/8+1; // distance from end the color inverts
@@ -128,14 +160,16 @@ void Fl_Roller::draw() {
       fl_yxline(X+W,Y+h1,Y);
     }
   }
+
+  if (Fl::focus() == this) draw_focus(FL_THIN_UP_FRAME, x(), y(), w(), h());
 }
 
 Fl_Roller::Fl_Roller(int X,int Y,int W,int H,const char* L)
   : Fl_Valuator(X,Y,W,H,L) {
-  box(FL_UP_FRAME);
+  box(FL_UP_BOX);
   step(1,1000);
 }
 
 //
-// End of "$Id: Fl_Roller.cxx,v 1.6.2.4 2001/01/22 15:13:40 easysw Exp $".
+// End of "$Id: Fl_Roller.cxx,v 1.6.2.4.2.9 2004/04/11 04:38:58 easysw Exp $".
 //
