@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_Scrollbar.cxx,v 1.7.2.12 2001/01/22 15:13:40 easysw Exp $"
+// "$Id: Fl_Scrollbar.cxx,v 1.7.2.14.2.17 2004/07/27 16:02:21 easysw Exp $"
 //
 // Scroll bar widget for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2004 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -43,9 +43,11 @@ void Fl_Scrollbar::increment_cb() {
     break;
   case 5:
     i = -int((maximum()-minimum())*slider_size()/(1.0-slider_size())) + ls;
+    if (i > -ls) i = -ls;
     break;
   case 6:
     i =  int((maximum()-minimum())*slider_size()/(1.0-slider_size())) - ls;
+    if (i < ls) i = ls;
     break;
   }
   handle_drag(clamp(value() + i));
@@ -91,7 +93,8 @@ int Fl_Scrollbar::handle(int event) {
     if (val >= 1.0) sliderx = ww-S;
     else if (val <= 0.0) sliderx = 0;
     else sliderx = int(val*(ww-S)+.5);
-    if (relx < sliderx) area = 5;
+    if (Fl::event_button() == FL_MIDDLE_MOUSE) area = 8;
+    else if (relx < sliderx) area = 5;
     else if (relx >= sliderx+S) area = 6;
     else area = 8;
   }
@@ -122,7 +125,12 @@ int Fl_Scrollbar::handle(int event) {
   case FL_DRAG:
     if (pushed_) return 1;
     return Fl_Slider::handle(event, X,Y,W,H);
-  case FL_SHORTCUT: {
+  case FL_MOUSEWHEEL :
+    if (horizontal()) return 0;
+    handle_drag(clamp(value() + linesize_ * Fl::e_dy));
+    return 1;
+  case FL_SHORTCUT:
+  case FL_KEYBOARD: {
     int v = value();
     int ls = maximum()>=minimum() ? linesize_ : -linesize_;
     if (horizontal()) {
@@ -168,6 +176,7 @@ int Fl_Scrollbar::handle(int event) {
     if (v != value()) {
       Fl_Slider::value(v);
       value_damage();
+      set_changed();
       do_callback();
     }
     return 1;}
@@ -185,39 +194,39 @@ void Fl_Scrollbar::draw() {
     if (W < 3*H) {Fl_Slider::draw(X,Y,W,H); return;}
     Fl_Slider::draw(X+H,Y,W-2*H,H);
     if (damage()&FL_DAMAGE_ALL) {
-      draw_box((pushed_==1) ? down(slider()) : slider(),
+      draw_box((pushed_==1) ? fl_down(slider()) : slider(),
 	       X, Y, H, H, selection_color());
-      draw_box((pushed_==2) ? down(slider()) : slider(),
-		  X+W-H, Y, H, H, selection_color());
+      draw_box((pushed_==2) ? fl_down(slider()) : slider(),
+	       X+W-H, Y, H, H, selection_color());
       if (active_r())
         fl_color(labelcolor());
       else
-        fl_color(inactive(labelcolor()));
+        fl_color(fl_inactive(labelcolor()));
       int w1 = (H-4)/3; if (w1 < 1) w1 = 1;
       int x1 = X+(H-w1-1)/2;
-      int y1 = Y+(H-2*w1-1)/2;
-      fl_polygon(x1, y1+w1, x1+w1, y1+2*w1, x1+w1, y1);
+      int yy1 = Y+(H-2*w1-1)/2;
+      fl_polygon(x1, yy1+w1, x1+w1, yy1+2*w1, x1+w1, yy1);
       x1 += (W-H);
-      fl_polygon(x1, y1, x1, y1+2*w1, x1+w1, y1+w1);
+      fl_polygon(x1, yy1, x1, yy1+2*w1, x1+w1, yy1+w1);
     }
   } else { // vertical
     if (H < 3*W) {Fl_Slider::draw(X,Y,W,H); return;}
     Fl_Slider::draw(X,Y+W,W,H-2*W);
     if (damage()&FL_DAMAGE_ALL) {
-      draw_box((pushed_==1) ? down(slider()) : slider(),
+      draw_box((pushed_==1) ? fl_down(slider()) : slider(),
 	       X, Y, W, W, selection_color());
-      draw_box((pushed_==2) ? down(slider()) : slider(),
+      draw_box((pushed_==2) ? fl_down(slider()) : slider(),
 	       X, Y+H-W, W, W, selection_color());
       if (active_r())
         fl_color(labelcolor());
       else
-        fl_color(labelcolor() | 8);
+        fl_color(fl_inactive(labelcolor()));
       int w1 = (W-4)/3; if (w1 < 1) w1 = 1;
       int x1 = X+(W-2*w1-1)/2;
-      int y1 = Y+(W-w1-1)/2;
-      fl_polygon(x1, y1+w1, x1+2*w1, y1+w1, x1+w1, y1);
-      y1 += H-W;
-      fl_polygon(x1, y1, x1+w1, y1+w1, x1+2*w1, y1);
+      int yy1 = Y+(W-w1-1)/2;
+      fl_polygon(x1, yy1+w1, x1+2*w1, yy1+w1, x1+w1, yy1);
+      yy1 += H-W;
+      fl_polygon(x1, yy1, x1+w1, yy1+w1, x1+2*w1, yy1);
     }
   }
 }
@@ -234,5 +243,5 @@ Fl_Scrollbar::Fl_Scrollbar(int X, int Y, int W, int H, const char* L)
 }
 
 //
-// End of "$Id: Fl_Scrollbar.cxx,v 1.7.2.12 2001/01/22 15:13:40 easysw Exp $".
+// End of "$Id: Fl_Scrollbar.cxx,v 1.7.2.14.2.17 2004/07/27 16:02:21 easysw Exp $".
 //
