@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_Tile.cxx,v 1.5.2.2 1999/11/16 07:39:59 bill Exp $"
+// "$Id: Fl_Tile.cxx,v 1.5.2.5.2.7 2004/07/27 16:02:21 easysw Exp $"
 //
 // Tile widget for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-1999 by Bill Spitzak and others.
+// Copyright 1998-2004 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,7 +20,7 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@easysw.com".
+// Please report all bugs and problems to "fltk-bugs@fltk.org".
 //
 
 // Group of 2 or 4 "tiles" that can be resized by dragging border
@@ -82,21 +82,21 @@ void Fl_Tile::resize(int X,int Y,int W,int H) {
   p += 8;
   for (int i=children(); i--;) {
     Fl_Widget* o = *a++;
-    int X = o->x()+dx;
-    int R = X+o->w();
-    if (*p++ >= OR) X += dw; else if (X > NR) X = NR;
+    int xx = o->x()+dx;
+    int R = xx+o->w();
+    if (*p++ >= OR) xx += dw; else if (xx > NR) xx = NR;
     if (*p++ >= OR) R += dw; else if (R > NR) R = NR;
-    int Y = o->y()+dy;
-    int B = Y+o->h();
-    if (*p++ >= OB) Y += dh; else if (Y > NB) Y = NB;
+    int yy = o->y()+dy;
+    int B = yy+o->h();
+    if (*p++ >= OB) yy += dh; else if (yy > NB) yy = NB;
     if (*p++ >= OB) B += dh; else if (B > NB) B = NB;
-    o->resize(X,Y,R-X,B-Y); o->redraw();
+    o->resize(xx,yy,R-xx,B-yy); o->redraw();
   }
 }
 
 static void set_cursor(Fl_Tile*t, Fl_Cursor c) {
   static Fl_Cursor cursor;
-  if (cursor == c) return;
+  if (cursor == c || !t->window()) return;
   cursor = c;
 #ifdef __sgi
   t->window()->cursor(c,FL_RED,FL_WHITE);
@@ -167,7 +167,7 @@ int Fl_Tile::handle(int event) {
     break;
 
   case FL_DRAG:
-    // This is necessary if CONSOLIDATE_MOTION in Fl_x.C is turned off:
+    // This is necessary if CONSOLIDATE_MOTION in Fl_x.cxx is turned off:
     // if (damage()) return 1; // don't fall behind
   case FL_RELEASE: {
     if (!sdrag) return 0; // should not happen
@@ -187,6 +187,7 @@ int Fl_Tile::handle(int event) {
     } else
       newy = sy;
     position(sx,sy,newx,newy);
+    if (event == FL_DRAG) set_changed();
     do_callback();
     return 1;}
 
@@ -196,5 +197,5 @@ int Fl_Tile::handle(int event) {
 }
 
 //
-// End of "$Id: Fl_Tile.cxx,v 1.5.2.2 1999/11/16 07:39:59 bill Exp $".
+// End of "$Id: Fl_Tile.cxx,v 1.5.2.5.2.7 2004/07/27 16:02:21 easysw Exp $".
 //
