@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_Value_Input.cxx,v 1.6.2.1 1999/05/20 18:01:41 bill Exp $"
+// "$Id: Fl_Value_Input.cxx,v 1.6.2.5.2.10 2004/07/27 16:02:21 easysw Exp $"
 //
 // Value input widget for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-1999 by Bill Spitzak and others.
+// Copyright 1998-2004 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,10 +20,10 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@easysw.com".
+// Please report all bugs and problems to "fltk-bugs@fltk.org".
 //
 
-// Fltk widget for drag-adjusting a floating point value.
+// FLTK widget for drag-adjusting a floating point value.
 // Warning: this works by making a child Fl_Input object, even
 // though this object is *not* an Fl_Group.  May be a kludge?
 
@@ -31,19 +31,19 @@
 #include <FL/Fl_Value_Input.H>
 #include <FL/Fl_Group.H>
 #include <stdlib.h>
+#include <FL/math.h>
 
-static Fl_Value_Input* hack_o_rama;
 
 void Fl_Value_Input::input_cb(Fl_Widget*, void* v) {
   Fl_Value_Input& t = *(Fl_Value_Input*)v;
   double nv;
-  if (t.step()>=1.0) nv = strtol(t.input.value(), 0, 0);
-  else nv = strtod(t.input.value(), 0);
-  hack_o_rama = &t;
-  t.handle_push();
-  t.handle_drag(nv);
-  t.handle_release();
-  hack_o_rama = 0;
+  if ((t.step() - floor(t.step()))>0.0 || t.step() == 0.0) nv = strtod(t.input.value(), 0);
+  else nv = strtol(t.input.value(), 0, 0);
+  if (nv != t.value() || t.when() & FL_WHEN_NOT_CHANGED) {
+    t.set_value(nv);
+    t.set_changed();
+    if (t.when()) t.do_callback();
+  }
 }
 
 void Fl_Value_Input::draw() {
@@ -60,7 +60,6 @@ void Fl_Value_Input::resize(int X, int Y, int W, int H) {
 }
 
 void Fl_Value_Input::value_damage() {
-  if (hack_o_rama==this) return;
   char buf[128];
   format(buf);
   input.value(buf);
@@ -72,6 +71,7 @@ int Fl_Value_Input::handle(int event) {
   int delta;
   int mx = Fl::event_x();
   static int ix, drag;
+  input.when(when());
   switch (event) {
   case FL_PUSH:
     if (!step()) goto DEFAULT;
@@ -106,17 +106,17 @@ int Fl_Value_Input::handle(int event) {
     return input.take_focus();
   default:
   DEFAULT:
-    input.type(step()>=1.0 ? FL_INT_INPUT : FL_FLOAT_INPUT);
+    input.type(((step() - floor(step()))>0.0 || step() == 0.0) ? FL_FLOAT_INPUT : FL_INT_INPUT);
     return input.handle(event);
   }
 }
 
-Fl_Value_Input::Fl_Value_Input(int x, int y, int w, int h, const char* l)
-: Fl_Valuator(x, y, w, h, l), input(x, y, w, h, 0) {
+Fl_Value_Input::Fl_Value_Input(int X, int Y, int W, int H, const char* l)
+: Fl_Valuator(X, Y, W, H, l), input(X, Y, W, H, 0) {
   soft_ = 0;
   if (input.parent())  // defeat automatic-add
     ((Fl_Group*)input.parent())->remove(input);
-  input.parent(this); // kludge!
+  input.parent((Fl_Group *)this); // kludge!
   input.callback(input_cb, this);
   input.when(FL_WHEN_CHANGED);
   box(input.box());
@@ -127,5 +127,5 @@ Fl_Value_Input::Fl_Value_Input(int x, int y, int w, int h, const char* l)
 }
 
 //
-// End of "$Id: Fl_Value_Input.cxx,v 1.6.2.1 1999/05/20 18:01:41 bill Exp $".
+// End of "$Id: Fl_Value_Input.cxx,v 1.6.2.5.2.10 2004/07/27 16:02:21 easysw Exp $".
 //
