@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_Value_Output.cxx,v 1.6.2.3 2001/01/22 15:13:40 easysw Exp $"
+// "$Id: Fl_Value_Output.cxx,v 1.6.2.3.2.8 2004/04/11 04:38:58 easysw Exp $"
 //
 // Value output widget for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2004 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -45,9 +45,9 @@ void Fl_Value_Output::draw() {
   }
   char buf[128];
   format(buf);
-  fl_color(active_r() ? textcolor() : inactive(textcolor()));
+  fl_color(active_r() ? textcolor() : fl_inactive(textcolor()));
   fl_font(textfont(), textsize());
-  fl_draw(buf,X+3,Y,W-6,H,FL_ALIGN_LEFT);
+  fl_draw(buf,X,Y,W,H,FL_ALIGN_LEFT);
 }
 
 int Fl_Value_Output::handle(int event) {
@@ -78,21 +78,24 @@ int Fl_Value_Output::handle(int event) {
   case FL_RELEASE:
     handle_release();
     return 1;
+  case FL_ENTER :
+  case FL_LEAVE :
+    return 1;
   default:
     return 0;
   }
 }
 
-Fl_Value_Output::Fl_Value_Output(int x,int y,int w,int h,const char *l)
-: Fl_Valuator(x,y,w,h,l) {
+Fl_Value_Output::Fl_Value_Output(int X, int Y, int W, int H,const char *l)
+: Fl_Valuator(X,Y,W,H,l) {
   box(FL_NO_BOX);
   align(FL_ALIGN_LEFT);
   textfont_ = FL_HELVETICA;
-  textsize_ = FL_NORMAL_SIZE;
+  textsize_ = (uchar)FL_NORMAL_SIZE;
   textcolor_ = FL_BLACK;
   soft_ = 0;
 }
 
 //
-// End of "$Id: Fl_Value_Output.cxx,v 1.6.2.3 2001/01/22 15:13:40 easysw Exp $".
+// End of "$Id: Fl_Value_Output.cxx,v 1.6.2.3.2.8 2004/04/11 04:38:58 easysw Exp $".
 //
