@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_Widget.cxx,v 1.5 1999/01/07 19:17:29 mike Exp $"
+// "$Id: Fl_Widget.cxx,v 1.5.2.4.2.26 2004/11/23 19:51:03 easysw Exp $"
 //
 // Base widget class for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-1999 by Bill Spitzak and others.
+// Copyright 1998-2004 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,12 +20,17 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@easysw.com".
+// Please report all bugs and problems to "fltk-bugs@fltk.org".
 //
 
 #include <FL/Fl.H>
 #include <FL/Fl_Widget.H>
 #include <FL/Fl_Group.H>
+#include <FL/Fl_Tooltip.H>
+#include <FL/fl_draw.H>
+#include <stdlib.h>
+#include "flstring.h"
+
 
 ////////////////////////////////////////////////////////////////
 // for compatability with Forms, all widgets without callbacks are
@@ -65,27 +70,34 @@ Fl_Widget *Fl::readqueue() {
     
 ////////////////////////////////////////////////////////////////
 
-int Fl_Widget::handle(int) {return 0;}
+int Fl_Widget::handle(int) {
+  return 0;
+}
+
+int FL_NORMAL_SIZE = 14;
 
 Fl_Widget::Fl_Widget(int X, int Y, int W, int H, const char* L) {
 
   x_ = X; y_ = Y; w_ = W; h_ = H;
 
-  label_.value	= L;
-  label_.type	= FL_NORMAL_LABEL;
-  label_.font	= FL_HELVETICA;
-  label_.size	= FL_NORMAL_SIZE;
-  label_.color	= FL_BLACK;
-  callback_	= default_callback;
-  user_data_ 	= 0;
-  type_		= 0;
-  flags_	= 0;
-  damage_	= 0;
-  box_		= FL_NO_BOX;
-  color_	= FL_GRAY;
-  color2_	= FL_GRAY;
-  align_	= FL_ALIGN_CENTER;
-  when_		= FL_WHEN_RELEASE;
+  label_.value	 = L;
+  label_.image   = 0;
+  label_.deimage = 0;
+  label_.type	 = FL_NORMAL_LABEL;
+  label_.font	 = FL_HELVETICA;
+  label_.size	 = (uchar)FL_NORMAL_SIZE;
+  label_.color	 = FL_BLACK;
+  tooltip_       = 0;
+  callback_	 = default_callback;
+  user_data_ 	 = 0;
+  type_		 = 0;
+  flags_	 = VISIBLE_FOCUS;
+  damage_	 = 0;
+  box_		 = FL_NO_BOX;
+  color_	 = FL_GRAY;
+  color2_	 = FL_GRAY;
+  align_	 = FL_ALIGN_CENTER;
+  when_		 = FL_WHEN_RELEASE;
 
   parent_ = 0;
   if (Fl_Group::current()) Fl_Group::current()->add(this);
@@ -105,37 +117,75 @@ int Fl_Widget::damage_resize(int X, int Y, int W, int H) {
 
 int Fl_Widget::take_focus() {
   if (!takesevents()) return 0;
+  if (!visible_focus()) return 0;
   if (!handle(FL_FOCUS)) return 0; // see if it wants it
   if (contains(Fl::focus())) return 1; // it called Fl::focus for us
   Fl::focus(this);
   return 1;
 }
 
-extern void fl_throw_focus(Fl_Widget*); // in Fl_x.C
+extern void fl_throw_focus(Fl_Widget*); // in Fl_x.cxx
 
 // Destruction does not remove from any parent group!  And groups when
 // destroyed destroy all their children.  This is convienent and fast.
 // However, it is only legal to destroy a "root" such as an Fl_Window,
 // and automatic destructors may be called.
 Fl_Widget::~Fl_Widget() {
-  parent_ = 0; // kludge to prevent ~Fl_Group from destroying again
+  if (flags() & COPIED_LABEL) free((void *)(label_.value));
+  parent_ = 0; // Don't throw focus to a parent widget.
   fl_throw_focus(this);
 }
 
-// redraw this, plus redraw opaque object if there is an outside label
-static void redraw_label(Fl_Widget* w) {
-  w->redraw();
-  if (w->label() && (w->align()&15) && !(w->align() & FL_ALIGN_INSIDE)) {
-    for (Fl_Widget *p = w->parent(); p; p = p->parent())
-      if (p->box() || !p->parent()) {p->redraw(); break;}
+// draw a focus box for the widget...
+void
+Fl_Widget::draw_focus(Fl_Boxtype B, int X, int Y, int W, int H) const {
+  if (!Fl::visible_focus()) return;
+  switch (B) {
+    case FL_DOWN_BOX:
+    case FL_DOWN_FRAME:
+    case FL_THIN_DOWN_BOX:
+    case FL_THIN_DOWN_FRAME:
+      X ++;
+      Y ++;
+    default:
+      break;
   }
+
+  fl_color(fl_contrast(FL_BLACK, color()));
+
+#if defined(WIN32) || defined(__APPLE_QD__)
+  // Windows 95/98/ME do not implement the dotted line style, so draw
+  // every other pixel around the focus area...
+  //
+  // Also, QuickDraw (MacOS) does not support line styles specifically,
+  // and the hack we use in fl_line_style() will not draw horizontal lines
+  // on odd-numbered rows...
+  int i, xx, yy;
+
+  X += Fl::box_dx(B);
+  Y += Fl::box_dy(B);
+  W -= Fl::box_dw(B) + 2;
+  H -= Fl::box_dh(B) + 2;
+
+  for (xx = 0, i = 1; xx < W; xx ++, i ++) if (i & 1) fl_point(X + xx, Y);
+  for (yy = 0; yy < H; yy ++, i ++) if (i & 1) fl_point(X + W, Y + yy);
+  for (xx = W; xx > 0; xx --, i ++) if (i & 1) fl_point(X + xx, Y + H);
+  for (yy = H; yy > 0; yy --, i ++) if (i & 1) fl_point(X, Y + yy);
+#else
+  fl_line_style(FL_DOT);
+  fl_rect(X + Fl::box_dx(B), Y + Fl::box_dy(B),
+          W - Fl::box_dw(B) - 1, H - Fl::box_dh(B) - 1);
+  fl_line_style(FL_SOLID);
+#endif // WIN32
 }
 
+
 void Fl_Widget::activate() {
   if (!active()) {
     clear_flag(INACTIVE);
     if (active_r()) {
-      redraw_label(this);
+      redraw();
+      redraw_label();
       handle(FL_ACTIVATE);
       if (inside(Fl::focus())) Fl::focus()->take_focus();
     }
@@ -145,7 +195,8 @@ void Fl_Widget::activate() {
 void Fl_Widget::deactivate() {
   if (active_r()) {
     set_flag(INACTIVE);
-    redraw_label(this);
+    redraw();
+    redraw_label();
     handle(FL_DEACTIVATE);
     fl_throw_focus(this);
   } else {
@@ -163,7 +214,8 @@ void Fl_Widget::show() {
   if (!visible()) {
     clear_flag(INVISIBLE);
     if (visible_r()) {
-      redraw_label(this);
+      redraw();
+      redraw_label();
       handle(FL_SHOW);
       if (inside(Fl::focus())) Fl::focus()->take_focus();
     }
@@ -195,6 +247,27 @@ int Fl_Widget::contains(const Fl_Widget *o) const {
   return 0;
 }
 
+
+void
+Fl_Widget::label(const char *a) {
+  if (flags() & COPIED_LABEL) {
+    free((void *)(label_.value));
+    clear_flag(COPIED_LABEL);
+  }
+  label_.value=a;
+  redraw_label();
+}
+
+
+void
+Fl_Widget::copy_label(const char *a) {
+  if (flags() & COPIED_LABEL) free((void *)(label_.value));
+  set_flag(COPIED_LABEL);
+  label_.value=strdup(a);
+  redraw_label();
+}
+
+
 //
-// End of "$Id: Fl_Widget.cxx,v 1.5 1999/01/07 19:17:29 mike Exp $".
+// End of "$Id: Fl_Widget.cxx,v 1.5.2.4.2.26 2004/11/23 19:51:03 easysw Exp $".
 //
