@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_Window.cxx,v 1.6.2.3 2001/01/22 15:13:40 easysw Exp $"
+// "$Id: Fl_Window.cxx,v 1.6.2.3.2.12 2004/04/11 04:38:58 easysw Exp $"
 //
 // Window widget class for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2004 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -25,8 +25,8 @@
 
 // The Fl_Window is a window in the fltk library.
 // This is the system-independent portions.  The huge amount of 
-// crap you need to do to communicate with X is in Fl_x.C, the
-// equivalent (but totally different) crap for MSWindows is in Fl_win32.C
+// crap you need to do to communicate with X is in Fl_x.cxx, the
+// equivalent (but totally different) crap for MSWindows is in Fl_win32.cxx
 
 #include <FL/Fl.H>
 #include <FL/Fl_Window.H>
@@ -34,18 +34,29 @@
 void Fl_Window::_Fl_Window() {
   type(FL_WINDOW);
   box(FL_FLAT_BOX);
-  labeltype(FL_NO_LABEL);
+  if (Fl::scheme_bg_) {
+    labeltype(FL_NORMAL_LABEL);
+    align(FL_ALIGN_CENTER | FL_ALIGN_INSIDE | FL_ALIGN_CLIP);
+    image(Fl::scheme_bg_);
+  } else {
+    labeltype(FL_NO_LABEL);
+  }
   i = 0;
   xclass_ = 0;
   icon_ = 0;
   iconlabel_ = 0;
   resizable(0);
   size_range_set = 0;
+  minw = maxw = minh = maxh = 0;
   callback((Fl_Callback*)default_callback);
 }
 
 Fl_Window::Fl_Window(int X,int Y,int W, int H, const char *l)
 : Fl_Group(X, Y, W, H, l) {
+  cursor_default = FL_CURSOR_DEFAULT;
+  cursor_fg      = FL_BLACK;
+  cursor_bg      = FL_WHITE;
+
   _Fl_Window();
   set_flag(FL_FORCE_POSITION);
 }
@@ -53,6 +64,10 @@ Fl_Window::Fl_Window(int X,int Y,int W, int H, const char *l)
 Fl_Window::Fl_Window(int W, int H, const char *l)
 // fix common user error of a missing end() with current(0):
   : Fl_Group((Fl_Group::current(0),0), 0, W, H, l) {
+  cursor_default = FL_CURSOR_DEFAULT;
+  cursor_fg      = FL_BLACK;
+  cursor_bg      = FL_WHITE;
+
   _Fl_Window();
   clear_visible();
 }
@@ -76,9 +91,14 @@ int Fl_Window::y_root() const {
 }
 
 void Fl_Window::draw() {
+  const char *savelabel = label();
   int savex = x(); x(0);
   int savey = y(); y(0);
+  // Make sure we don't draw the window title in the window background...
+  Fl_Widget::label(0);
   Fl_Group::draw();
+  // Restore the label...
+  Fl_Widget::label(savelabel);
   y(savey);
   x(savex);
 }
@@ -97,10 +117,15 @@ void Fl::default_atclose(Fl_Window* window, void* v) {
 
 void (*Fl::atclose)(Fl_Window*, void*) = default_atclose;
 
-void Fl_Window::default_callback(Fl_Window* window, void* v) {
-  Fl::atclose(window, v);
+void Fl_Window::default_callback(Fl_Window* win, void* v) {
+  Fl::atclose(win, v);
 }
 
+Fl_Window *Fl_Window::current() {
+  return current_;
+}
+
+
 //
-// End of "$Id: Fl_Window.cxx,v 1.6.2.3 2001/01/22 15:13:40 easysw Exp $".
+// End of "$Id: Fl_Window.cxx,v 1.6.2.3.2.12 2004/04/11 04:38:58 easysw Exp $".
 //
