@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_Window_iconize.cxx,v 1.5.2.3 2001/01/22 15:13:40 easysw Exp $"
+// "$Id: Fl_Window_iconize.cxx,v 1.5.2.3.2.6 2004/04/11 04:38:58 easysw Exp $"
 //
 // Window minification code for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2004 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -25,7 +25,7 @@
 
 #include <FL/x.H>
 
-extern char fl_show_iconic; // in Fl_x.C
+extern char fl_show_iconic; // in Fl_x.cxx
 
 void Fl_Window::iconize() {
   if (!shown()) {
@@ -34,6 +34,8 @@ void Fl_Window::iconize() {
   } else {
 #ifdef WIN32
     ShowWindow(i->xid, SW_SHOWMINNOACTIVE);
+#elif defined(__APPLE__)
+    CollapseWindow( i->xid, true );
 #else
     XIconifyWindow(fl_display, i->xid, fl_screen);
 #endif
@@ -41,5 +43,5 @@ void Fl_Window::iconize() {
 }
 
 //
-// End of "$Id: Fl_Window_iconize.cxx,v 1.5.2.3 2001/01/22 15:13:40 easysw Exp $".
+// End of "$Id: Fl_Window_iconize.cxx,v 1.5.2.3.2.6 2004/04/11 04:38:58 easysw Exp $".
 //
