@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_arg.cxx,v 1.5.2.8 2001/03/20 18:02:52 spitzak Exp $"
+// "$Id: Fl_arg.cxx,v 1.5.2.8.2.17 2004/04/11 04:38:58 easysw Exp $"
 //
 // Optional argument initialization code for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2004 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -29,31 +29,32 @@
 #include <FL/Fl.H>
 #include <FL/x.H>
 #include <FL/Fl_Window.H>
+#include <FL/Fl_Tooltip.H>
 #include <FL/filename.H>
 #include <FL/fl_draw.H>
 #include <ctype.h>
-#include <string.h>
+#include "flstring.h"
 
-#ifdef WIN32
+#if defined(WIN32) || defined(__APPLE__)
 int XParseGeometry(const char*, int*, int*, unsigned int*, unsigned int*);
-#define NoValue		0x0000
-#define XValue  	0x0001
-#define YValue		0x0002
-#define WidthValue  	0x0004
-#define HeightValue  	0x0008
-#define AllValues 	0x000F
-#define XNegative 	0x0010
-#define YNegative 	0x0020
+#  define NoValue	0x0000
+#  define XValue  	0x0001
+#  define YValue	0x0002
+#  define WidthValue  	0x0004
+#  define HeightValue  	0x0008
+#  define AllValues 	0x000F
+#  define XNegative 	0x0010
+#  define YNegative 	0x0020
 #endif
 
-static int match(const char *a, const char *match, int atleast = 1) {
-  const char *b = match;
+static int fl_match(const char *a, const char *s, int atleast = 1) {
+  const char *b = s;
   while (*a && (*a == *b || tolower(*a) == *b)) {a++; b++;}
-  return !*a && b >= match+atleast;
+  return !*a && b >= s+atleast;
 }
 
 // flags set by previously parsed arguments:
-extern char fl_show_iconic; // in Fl_x.C
+extern char fl_show_iconic; // in Fl_x.cxx
 static char arg_called;
 static char return_i;
 static const char *name;
@@ -79,43 +80,78 @@ int Fl::arg(int argc, char **argv, int &i) {
   if (s[0] != '-' || s[1] == '-' || !s[1]) {return_i = 1; return 0;}
   s++; // point after the dash
 
-  if (match(s, "iconic")) {
+  if (fl_match(s, "iconic")) {
     fl_show_iconic = 1;
     i++;
     return 1;
+  } else if (fl_match(s, "kbd")) {
+    Fl::visible_focus(1);
+    i++;
+    return 1;
+  } else if (fl_match(s, "nokbd", 3)) {
+    Fl::visible_focus(0);
+    i++;
+    return 1;
+  } else if (fl_match(s, "dnd", 2)) {
+    Fl::dnd_text_ops(1);
+    i++;
+    return 1;
+  } else if (fl_match(s, "nodnd", 3)) {
+    Fl::dnd_text_ops(0);
+    i++;
+    return 1;
+  } else if (fl_match(s, "tooltips", 2)) {
+    Fl_Tooltip::enable();
+    i++;
+    return 1;
+  } else if (fl_match(s, "notooltips", 3)) {
+    Fl_Tooltip::disable();
+    i++;
+    return 1;
   }
+#ifdef __APPLE__
+  // The Finder application in MacOS X passes the "-psn_N_NNNNN" option
+  // to all apps...
+  else if (strncmp(s, "psn_", 4) == 0) {
+    i++;
+    return 1;
+  }
+#endif // __APPLE__
 
   const char *v = argv[i+1];
   if (i >= argc-1 || !v)
     return 0;	// all the rest need an argument, so if missing it is an error
 
-  if (match(s, "geometry")) {
+  if (fl_match(s, "geometry")) {
 
     int flags, gx, gy; unsigned int gw, gh;
     flags = XParseGeometry(v, &gx, &gy, &gw, &gh);
     if (!flags) return 0;
     geometry = v;
 
-#ifndef WIN32
-  } else if (match(s, "display")) {
+#if !defined(WIN32) && !defined(__APPLE__)
+  } else if (fl_match(s, "display", 2)) {
     Fl::display(v);
 #endif
 
-  } else if (match(s, "title")) {
+  } else if (fl_match(s, "title", 2)) {
     title = v;
 
-  } else if (match(s, "name")) {
+  } else if (fl_match(s, "name", 2)) {
     name = v;
 
-  } else if (match(s, "bg2", 3) || match(s, "background2", 11)) {
+  } else if (fl_match(s, "bg2", 3) || fl_match(s, "background2", 11)) {
     fl_bg2 = v;
 
-  } else if (match(s, "bg") || match(s, "background")) {
+  } else if (fl_match(s, "bg", 2) || fl_match(s, "background", 10)) {
     fl_bg = v;
 
-  } else if (match(s, "fg") || match(s, "foreground")) {
+  } else if (fl_match(s, "fg", 2) || fl_match(s, "foreground", 10)) {
     fl_fg = v;
 
+  } else if (fl_match(s, "scheme", 1)) {
+    Fl::scheme(v);
+
   } else return 0; // unrecognized
 
   i += 2;
@@ -138,27 +174,50 @@ int Fl::args(int argc, char** argv, int& i, int (*cb)(int,char**,int&)) {
   return i;
 }
 
-
 // show a main window, use any parsed arguments
 void Fl_Window::show(int argc, char **argv) {
-  if (!argc) {show(); return;}
-  if (!arg_called) Fl::args(argc,argv);
+  if (argc && !arg_called) Fl::args(argc,argv);
+
+  Fl::get_system_colors();
+
+#if !defined(WIN32) && !defined(__APPLE__)
+  // Get defaults for drag-n-drop and focus...
+  const char *key = 0, *val;
+
+  if (Fl::first_window()) key = Fl::first_window()->xclass();
+  if (!key) key = "fltk";
+
+  val = XGetDefault(fl_display, key, "dndTextOps");
+  if (val) Fl::dnd_text_ops(strcasecmp(val, "true") == 0 ||
+                            strcasecmp(val, "on") == 0 ||
+                            strcasecmp(val, "yes") == 0);
+
+  val = XGetDefault(fl_display, key, "tooltips");
+  if (val) Fl_Tooltip::enable(strcasecmp(val, "true") == 0 ||
+                              strcasecmp(val, "on") == 0 ||
+                              strcasecmp(val, "yes") == 0);
+
+  val = XGetDefault(fl_display, key, "visibleFocus");
+  if (val) Fl::visible_focus(strcasecmp(val, "true") == 0 ||
+                             strcasecmp(val, "on") == 0 ||
+                             strcasecmp(val, "yes") == 0);
+#endif // !WIN32 && !__APPLE__
 
   // set colors first, so background_pixel is correct:
   static char beenhere;
   if (!beenhere) {
     if (geometry) {
-      int flags = 0, gx = x(), gy = y(); unsigned int gw = w(), gh = h();
-      flags = XParseGeometry(geometry, &gx, &gy, &gw, &gh);
-      if (flags & XNegative) gx = Fl::w()-w()+gx;
-      if (flags & YNegative) gy = Fl::h()-h()+gy;
+      int fl = 0, gx = x(), gy = y(); unsigned int gw = w(), gh = h();
+      fl = XParseGeometry(geometry, &gx, &gy, &gw, &gh);
+      if (fl & XNegative) gx = Fl::w()-w()+gx;
+      if (fl & YNegative) gy = Fl::h()-h()+gy;
       //  int mw,mh; minsize(mw,mh);
       //  if (mw > gw) gw = mw;
       //  if (mh > gh) gh = mh;
       Fl_Widget *r = resizable();
       if (!r) resizable(this);
       // for WIN32 we assumme window is not mapped yet:
-      if (flags & (XValue | YValue))
+      if (fl & (XValue | YValue))
 	x(-1), resize(gx,gy,gw,gh);
       else
 	size(gw,gh);
@@ -168,7 +227,7 @@ void Fl_Window::show(int argc, char **argv) {
 
   // set the class, which is used by X version of get_system_colors:
   if (name) {xclass(name); name = 0;}
-  else if (!xclass()) xclass(filename_name(argv[0]));
+  else if (!xclass()) xclass(fl_filename_name(argv[0]));
 
   if (title) {label(title); title = 0;}
   else if (!label()) label(xclass());
@@ -176,41 +235,41 @@ void Fl_Window::show(int argc, char **argv) {
 
   if (!beenhere) {
     beenhere = 1;
-    Fl::get_system_colors(); // opens display!  May call Fl::fatal()
+    Fl::scheme(Fl::scheme()); // opens display!  May call Fl::fatal()
   }
 
-#ifndef WIN32
+#if !defined(WIN32) && !defined(__APPLE__)
   // set the command string, used by state-saving window managers:
   int j;
   int n=0; for (j=0; j<argc; j++) n += strlen(argv[j])+1;
-#ifdef __GNUC__
-  char buffer[n];
-#else
   char *buffer = new char[n];
-#endif
   char *p = buffer;
   for (j=0; j<argc; j++) for (const char *q = argv[j]; (*p++ = *q++););
   XChangeProperty(fl_display, fl_xid(this), XA_WM_COMMAND, XA_STRING, 8, 0,
 		  (unsigned char *)buffer, p-buffer-1);
-#ifndef __GNUC__
   delete[] buffer;
-#endif
-#endif
-
+#endif // !WIN32 && !__APPLE__
 }
 
 // Calls useful for simple demo programs, with automatic help message:
 
 static const char * const helpmsg =
 "options are:\n"
-" -d[isplay] host:n.n\n"
-" -g[eometry] WxH+X+Y\n"
-" -t[itle] windowtitle\n"
-" -n[ame] classname\n"
-" -i[conic]\n"
-" -fg color\n"
+" -bg2 color\n"
 " -bg color\n"
-" -bg2 color";
+" -di[splay] host:n.n\n"
+" -dn[d]\n"
+" -fg color\n"
+" -g[eometry] WxH+X+Y\n"
+" -i[conic]\n"
+" -k[bd]\n"
+" -na[me] classname\n"
+" -nod[nd]\n"
+" -nok[bd]\n"
+" -not[ooltips]\n"
+" -s[cheme] scheme\n"
+" -ti[tle] windowtitle\n"
+" -to[oltips]";
 
 const char * const Fl::help = helpmsg+13;
 
@@ -218,7 +277,7 @@ void Fl::args(int argc, char **argv) {
   int i; if (Fl::args(argc,argv,i) < argc) Fl::error(helpmsg);
 }
 
-#ifdef WIN32
+#if defined(WIN32) || defined(__APPLE__)
 
 /* the following function was stolen from the X sources as indicated. */
 
@@ -357,5 +416,5 @@ int XParseGeometry(const char* string, int* x, int* y,
 #endif // ifdef WIN32
 
 //
-// End of "$Id: Fl_arg.cxx,v 1.5.2.8 2001/03/20 18:02:52 spitzak Exp $".
+// End of "$Id: Fl_arg.cxx,v 1.5.2.8.2.17 2004/04/11 04:38:58 easysw Exp $".
 //
