@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_get_key.cxx,v 1.5.2.3 2001/01/22 15:13:40 easysw Exp $"
+// "$Id: Fl_get_key.cxx,v 1.5.2.3.2.6 2004/04/11 04:38:59 easysw Exp $"
 //
 // Keyboard state routines for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2004 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -24,30 +24,33 @@
 //
 
 #ifdef WIN32
-#include "Fl_get_key_win32.cxx"
+#  include "Fl_get_key_win32.cxx"
+#elif defined(__APPLE__)
+#  include "Fl_get_key_mac.cxx"
 #else
 
 // Return the current state of a key.  This is the X version.  I identify
 // keys (mostly) by the X keysym.  So this turns the keysym into a keycode
-// and looks it up in the X key bit vector, which Fl_x.C keeps track of.
+// and looks it up in the X key bit vector, which Fl_x.cxx keeps track of.
 
-#include <FL/Fl.H>
-#include <FL/x.H>
+#  include <FL/Fl.H>
+#  include <FL/x.H>
 
-extern char fl_key_vector[32]; // in Fl_x.C
+extern char fl_key_vector[32]; // in Fl_x.cxx
 
 int Fl::event_key(int k) {
   if (k > FL_Button && k <= FL_Button+8)
     return Fl::event_state(8<<(k-FL_Button));
   int i;
-#ifdef __sgi
+#  ifdef __sgi
   // get some missing PC keyboard keys:
   if (k == FL_Meta_L) i = 147;
   else if (k == FL_Meta_R) i = 148;
   else if (k == FL_Menu) i = 149;
   else
-#endif
+#  endif
     i = XKeysymToKeycode(fl_display, k);
+  if (i==0) return 0;
   return fl_key_vector[i/8] & (1 << (i%8));
 }
 
@@ -60,5 +63,5 @@ int Fl::get_key(int k) {
 #endif
 
 //
-// End of "$Id: Fl_get_key.cxx,v 1.5.2.3 2001/01/22 15:13:40 easysw Exp $".
+// End of "$Id: Fl_get_key.cxx,v 1.5.2.3.2.6 2004/04/11 04:38:59 easysw Exp $".
 //
