@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_grab.cxx,v 1.1.2.4 2001/01/22 15:13:40 easysw Exp $"
+// "$Id: Fl_grab.cxx,v 1.1.2.4.2.9 2004/04/11 04:38:59 easysw Exp $"
 //
 // Grab/release code for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2004 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -30,7 +30,7 @@
 ////////////////////////////////////////////////////////////////
 // "Grab" is done while menu systems are up.  This has several effects:
 // Events are all sent to the "grab window", which does not even
-// have to be displayed (and in the case of Fl_Menu.C it isn't).
+// have to be displayed (and in the case of Fl_Menu.cxx it isn't).
 // The system is also told to "grab" events and send them to this app.
 // This also modifies how Fl_Window::show() works, on X it turns on
 // override_redirect, it does similar things on WIN32.
@@ -44,12 +44,20 @@ extern void fl_fix_focus(); // in Fl.cxx
 extern HWND fl_capture;
 #endif
 
-void Fl::grab(Fl_Window* w) {
-  if (w) {
+#ifdef __APPLE__
+// MacOS Carbon does not seem to have a mechanism to grab the mouse pointer
+extern WindowRef fl_capture;
+#endif
+
+void Fl::grab(Fl_Window* win) {
+  if (win) {
     if (!grab_) {
 #ifdef WIN32
       SetActiveWindow(fl_capture = fl_xid(first_window()));
       SetCapture(fl_capture);
+#elif defined(__APPLE__)
+      fl_capture = fl_xid( first_window() );
+      SetUserFocusWindow( fl_capture );
 #else
       XGrabPointer(fl_display,
 		   fl_xid(first_window()),
@@ -69,12 +77,15 @@ void Fl::grab(Fl_Window* w) {
 		    fl_event_time);
 #endif
     }
-    grab_ = w;
+    grab_ = win;
   } else {
     if (grab_) {
 #ifdef WIN32
       fl_capture = 0;
       ReleaseCapture();
+#elif defined(__APPLE__)
+      fl_capture = 0;
+      SetUserFocusWindow( (WindowRef)kUserFocusAuto );
 #else
       XUngrabKeyboard(fl_display, fl_event_time);
       XUngrabPointer(fl_display, fl_event_time);
@@ -89,5 +100,5 @@ void Fl::grab(Fl_Window* w) {
 }
 
 //
-// End of "$Id: Fl_grab.cxx,v 1.1.2.4 2001/01/22 15:13:40 easysw Exp $".
+// End of "$Id: Fl_grab.cxx,v 1.1.2.4.2.9 2004/04/11 04:38:59 easysw Exp $".
 //
