@@ -1,9 +1,9 @@
 //
-// "$Id: filename_expand.cxx,v 1.4.2.4 2001/01/22 15:13:40 easysw Exp $"
+// "$Id: filename_expand.cxx,v 1.4.2.4.2.8 2004/04/11 04:38:59 easysw Exp $"
 //
 // Filename expansion routines for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2004 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -30,7 +30,7 @@
 
 #include <FL/filename.H>
 #include <stdlib.h>
-#include <string.h>
+#include "flstring.h"
 #if defined(WIN32) && !defined(__CYGWIN__)
 #else
 # include <unistd.h>
@@ -43,12 +43,12 @@ static inline int isdirsep(char c) {return c=='/' || c=='\\';}
 #define isdirsep(c) ((c)=='/')
 #endif
 
-int filename_expand(char *to,const char *from) {
+int fl_filename_expand(char *to,int tolen, const char *from) {
 
-  char temp[FL_PATH_MAX];
-  strcpy(temp,from);
-  const char *start = temp;
-  const char *end = temp+strlen(temp);
+  char *temp = new char[tolen];
+  strlcpy(temp,from, tolen);
+  char *start = temp;
+  char *end = temp+strlen(temp);
 
   int ret = 0;
 
@@ -81,8 +81,10 @@ int filename_expand(char *to,const char *from) {
       if (value[0] && value[1]==':') start = a;
 #endif
       int t = strlen(value); if (isdirsep(value[t-1])) t--;
+      if ((end+1-e+t) >= tolen) end += tolen - (end+1-e+t);
       memmove(a+t, e, end+1-e);
       end = a+t+(end-e);
+      *end = '\0';
       memcpy(a, value, t);
       ret++;
     } else {
@@ -92,10 +94,15 @@ int filename_expand(char *to,const char *from) {
 #endif
     }
   }
-  strcpy(to,start);
+
+  strlcpy(to, start, tolen);
+
+  delete[] temp;
+
   return ret;
 }
 
+
 //
-// End of "$Id: filename_expand.cxx,v 1.4.2.4 2001/01/22 15:13:40 easysw Exp $".
+// End of "$Id: filename_expand.cxx,v 1.4.2.4.2.8 2004/04/11 04:38:59 easysw Exp $".
 //
