@@ -1,9 +1,9 @@
 //
-// "$Id: filename_ext.cxx,v 1.4.2.4 2001/01/22 15:13:40 easysw Exp $"
+// "$Id: filename_ext.cxx,v 1.4.2.4.2.4 2004/04/11 04:38:59 easysw Exp $"
 //
 // Filename extension routines for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2004 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -27,7 +27,7 @@
 
 #include <FL/filename.H>
 
-const char *filename_ext(const char *buf) {
+const char *fl_filename_ext(const char *buf) {
   const char *q = 0;
   const char *p = buf;
   for (p=buf; *p; p++) {
@@ -41,5 +41,5 @@ const char *filename_ext(const char *buf) {
 }
 
 //
-// End of "$Id: filename_ext.cxx,v 1.4.2.4 2001/01/22 15:13:40 easysw Exp $".
+// End of "$Id: filename_ext.cxx,v 1.4.2.4.2.4 2004/04/11 04:38:59 easysw Exp $".
 //
