@@ -1,9 +1,9 @@
 //
-// "$Id: filename_match.cxx,v 1.5.2.4 2001/01/22 15:13:40 easysw Exp $"
+// "$Id: filename_match.cxx,v 1.5.2.4.2.6 2004/04/11 04:38:59 easysw Exp $"
 //
 // Pattern matching routines for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2004 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -27,7 +27,7 @@
 #include <FL/filename.H>
 #include <ctype.h>
 
-int filename_match(const char *s, const char *p) {
+int fl_filename_match(const char *s, const char *p) {
   int matched;
 
   for (;;) {
@@ -39,7 +39,7 @@ int filename_match(const char *s, const char *p) {
 
     case '*' :	// match 0-n of any characters
       if (!*p) return 1; // do trailing * quickly
-      while (!filename_match(s, p)) if (!*s++) return 0;
+      while (!fl_filename_match(s, p)) if (!*s++) return 0;
       return 1;
 
     case '[': {	// match one character in set of form [abc-d] or [^a-b]
@@ -63,7 +63,7 @@ int filename_match(const char *s, const char *p) {
 
     case '{' : // {pattern1|pattern2|pattern3}
     NEXTCASE:
-    if (filename_match(s,p)) return 1;
+    if (fl_filename_match(s,p)) return 1;
     for (matched = 0;;) {
       switch (*p++) {
       case '\\': if (*p) p++; break;
@@ -89,25 +89,16 @@ int filename_match(const char *s, const char *p) {
     case 0:	// end of pattern
       return !*s;
 
-#if defined(WIN32) && !defined(__CYGWIN__)
     case '\\':	// quote next character
       if (*p) p++;
-      if (*s++ != *(p-1)) return 0;
-      break;
     default:
       if (tolower(*s) != tolower(*(p-1))) return 0;
       s++;
-#else
-    case '\\':	// quote next character
-      if (*p) p++;
-    default  :
-      if (*s++ != *(p-1)) return 0;
       break;
-#endif
     }
   }
 }
 
 //
-// End of "$Id: filename_match.cxx,v 1.5.2.4 2001/01/22 15:13:40 easysw Exp $".
+// End of "$Id: filename_match.cxx,v 1.5.2.4.2.6 2004/04/11 04:38:59 easysw Exp $".
 //
