@@ -1,9 +1,9 @@
 //
-// "$Id: filename_setext.cxx,v 1.4 1999/01/07 19:17:35 mike Exp $"
+// "$Id: filename_setext.cxx,v 1.4.2.3.2.7 2004/04/11 04:38:59 easysw Exp $"
 //
 // Filename extension routines for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-1999 by Bill Spitzak and others.
+// Copyright 1998-2004 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,7 +20,7 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@easysw.com".
+// Please report all bugs and problems to "fltk-bugs@fltk.org".
 //
 
 // Replace .ext with new extension
@@ -28,14 +28,17 @@
 // If new extension is null, act like it is ""
 
 #include <FL/filename.H>
-#include <string.h>
+#include "flstring.h"
 
-char *filename_setext(char *buf, const char *ext) {
-  char *q = (char *)filename_ext(buf);
-  if (ext) strcpy(q,ext); else *q = 0;
+char *fl_filename_setext(char *buf, int buflen, const char *ext) {
+  char *q = (char *)fl_filename_ext(buf);
+  if (ext) {
+    strlcpy(q,ext,buflen - (q - buf));
+  } else *q = 0;
   return(buf);
 }
 
+
 //
-// End of "$Id: filename_setext.cxx,v 1.4 1999/01/07 19:17:35 mike Exp $".
+// End of "$Id: filename_setext.cxx,v 1.4.2.3.2.7 2004/04/11 04:38:59 easysw Exp $".
 //
