@@ -1,9 +1,9 @@
 //
-// "$Id: fl_ask.cxx,v 1.8.2.8 2001/01/22 15:13:40 easysw Exp $"
+// "$Id: fl_ask.cxx,v 1.8.2.8.2.14 2004/10/18 20:22:25 easysw Exp $"
 //
 // Standard dialog functions for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2004 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -30,8 +30,7 @@
 
 #include <stdio.h>
 #include <stdarg.h>
-#include <string.h>
-#include <config.h>
+#include "flstring.h"
 
 #include <FL/Fl.H>
 
@@ -44,6 +43,7 @@
 #include <FL/Fl_Input.H>
 #include <FL/Fl_Secret_Input.H>
 #include <FL/x.H>
+#include <FL/fl_draw.H>
 
 static Fl_Window *message_form;
 static Fl_Box *message;
@@ -51,7 +51,7 @@ static Fl_Box *icon;
 static Fl_Button *button[3];
 static Fl_Input *input;
 static const char *iconlabel = "?";
-uchar fl_message_font_ = 0;
+Fl_Font fl_message_font_ = FL_HELVETICA;
 uchar fl_message_size_ = 14;
 
 static Fl_Window *makeform() {
@@ -59,7 +59,7 @@ static Fl_Window *makeform() {
    message_form->size(410,103);
    return message_form;
  }
- Fl_Window *w = message_form = new Fl_Window(410,103);
+ Fl_Window *w = message_form = new Fl_Window(410,103,"");
  // w->clear_border();
  // w->box(FL_UP_BOX);
  (message = new Fl_Box(60, 25, 340, 20))
@@ -72,20 +72,95 @@ static Fl_Window *makeform() {
   o->color(FL_WHITE);
   o->labelcolor(FL_BLUE);
  }
- (button[0] = new Fl_Button(310, 70, 90, 23))->shortcut("^[");
+ button[0] = new Fl_Button(310, 70, 90, 23);
+ button[0]->shortcut("^[");
+ button[0]->align(FL_ALIGN_INSIDE|FL_ALIGN_WRAP);
  button[1] = new Fl_Return_Button(210, 70, 90, 23);
+ button[1]->align(FL_ALIGN_INSIDE|FL_ALIGN_WRAP);
  button[2] = new Fl_Button(110, 70, 90, 23);
+ button[2]->align(FL_ALIGN_INSIDE|FL_ALIGN_WRAP);
  w->resizable(new Fl_Box(60,10,110-60,27));
  w->end();
  w->set_modal();
  return w;
 }
 
-#if !HAVE_VSNPRINTF || defined(__hpux)
-extern "C" {
-int vsnprintf(char* str, size_t size, const char* fmt, va_list ap);
+/*
+ * 'resizeform()' - Resize the form and widgets so that they hold everything
+ *                  that is asked of them...
+ */
+
+void resizeform() {
+  int	i;
+  int	message_w, message_h;
+  int	icon_size;
+  int	button_w[3], button_h[3];
+  int	x, w, h, max_w, max_h;
+
+  fl_font(fl_message_font_, fl_message_size_);
+  message_w = message_h = 0;
+  fl_measure(message->label(), message_w, message_h);
+
+  message_w += 10;
+  message_h += 10;
+  if (message_w < 340)
+    message_w = 340;
+  if (message_h < 30)
+    message_h = 30;
+
+  fl_font(button[0]->labelfont(), button[0]->labelsize());
+
+  memset(button_w, 0, sizeof(button_w));
+  memset(button_h, 0, sizeof(button_h));
+
+  for (max_h = 25, i = 0; i < 3; i ++)
+    if (button[i]->visible())
+    {
+      fl_measure(button[i]->label(), button_w[i], button_h[i]);
+
+      if (i == 1)
+        button_w[1] += 20;
+
+      button_w[i] += 30;
+      button_h[i] += 10;
+
+      if (button_h[i] > max_h)
+        max_h = button_h[i];
+    }
+
+  if (input->visible()) icon_size = message_h + 25;
+  else icon_size = message_h;
+
+  max_w = message_w + 10 + icon_size;
+  w     = button_w[0] + button_w[1] + button_w[2] - 10;
+
+  if (w > max_w)
+    max_w = w;
+
+  message_w = max_w - 10 - icon_size;
+
+  w = max_w + 20;
+  h = max_h + 30 + icon_size;
+
+  message_form->size(w, h);
+  message_form->size_range(w, h, w, h);
+
+  message->resize(20 + icon_size, 10, message_w, message_h);
+  icon->resize(10, 10, icon_size, icon_size);
+  icon->labelsize((uchar)(icon_size - 10));
+  input->resize(20 + icon_size, 10 + message_h, message_w, 25);
+
+  for (x = w, i = 0; i < 3; i ++)
+    if (button_w[i])
+    {
+      x -= button_w[i];
+      button[i]->resize(x, h - 10 - max_h, button_w[i] - 10, max_h);
+
+//      printf("button %d (%s) is %dx%d+%d,%d\n", i, button[i]->label(),
+//             button[i]->w(), button[i]->h(),
+//	     button[i]->x(), button[i]->y());
+    }
 }
-#endif
 
 static int innards(const char* fmt, va_list ap,
   const char *b0,
@@ -97,11 +172,12 @@ static int innards(const char* fmt, va_list ap,
   if (!strcmp(fmt,"%s")) {
     message->label(va_arg(ap, const char*));
   } else {
-    vsnprintf(buffer, 1024, fmt, ap);
+    //: matt: MacOS provides two equally named vsnprintf's...
+    ::vsnprintf(buffer, 1024, fmt, ap);
     message->label(buffer);
   }
-  Fl_Font f = (Fl_Font)fl_message_font_;
-  message->labelfont(f);
+
+  message->labelfont(fl_message_font_);
   message->labelsize(fl_message_size_);
   if (b0) {button[0]->show(); button[0]->label(b0); button[1]->position(210,70);}
   else {button[0]->hide(); button[1]->position(310,70);}
@@ -111,6 +187,9 @@ static int innards(const char* fmt, va_list ap,
   else button[2]->hide();
   const char* prev_icon_label = icon->label();
   if (!prev_icon_label) icon->label(iconlabel);
+
+  resizeform();
+
   message_form->hotspot(button[0]);
   message_form->show();
   int r;
@@ -127,20 +206,64 @@ static int innards(const char* fmt, va_list ap,
   return r;
 }
 
-// pointers you can use to change fltk to a foreign language:
+// pointers you can use to change FLTK to a foreign language:
 const char* fl_no = "No";
 const char* fl_yes= "Yes";
 const char* fl_ok = "OK";
 const char* fl_cancel= "Cancel";
+const char* fl_close= "Close";
 
 // fltk functions:
+void fl_beep(int type) {
+#ifdef WIN32
+  switch (type) {
+    case FL_BEEP_QUESTION :
+    case FL_BEEP_PASSWORD :
+      MessageBeep(MB_ICONQUESTION);
+      break;
+    case FL_BEEP_MESSAGE :
+      MessageBeep(MB_ICONASTERISK);
+      break;
+    case FL_BEEP_NOTIFICATION :
+      MessageBeep(MB_ICONASTERISK);
+      break;
+    case FL_BEEP_ERROR :
+      MessageBeep(MB_ICONERROR);
+      break;
+    default :
+      MessageBeep(0xFFFFFFFF);
+      break;
+  }
+#elif defined(__APPLE__)
+  switch (type) {
+    case FL_BEEP_DEFAULT :
+    case FL_BEEP_ERROR :
+      SysBeep(30);
+      break;
+    default :
+      break;
+  }
+#else
+  switch (type) {
+    case FL_BEEP_DEFAULT :
+    case FL_BEEP_ERROR :
+      if (!fl_display) fl_open_display();
+
+      XBell(fl_display, 100);
+      break;
+    default :
+      if (!fl_display) fl_open_display();
+
+      XBell(fl_display, 50);
+      break;
+  }
+#endif // WIN32
+}
 
 void fl_message(const char *fmt, ...) {
   va_list ap;
 
-#ifdef WIN32
-  MessageBeep(MB_ICONASTERISK);
-#endif // WIN32
+  fl_beep(FL_BEEP_MESSAGE);
 
   va_start(ap, fmt);
   iconlabel = "i";
@@ -152,12 +275,7 @@ void fl_message(const char *fmt, ...) {
 void fl_alert(const char *fmt, ...) {
   va_list ap;
 
-#ifdef WIN32
-  MessageBeep(MB_ICONERROR);
-#else
-  if (!fl_display) fl_open_display();
-  XBell(fl_display, 100);
-#endif // WIN32
+  fl_beep(FL_BEEP_ERROR);
 
   va_start(ap, fmt);
   iconlabel = "!";
@@ -169,9 +287,7 @@ void fl_alert(const char *fmt, ...) {
 int fl_ask(const char *fmt, ...) {
   va_list ap;
 
-#ifdef WIN32
-  MessageBeep(MB_ICONQUESTION);
-#endif // WIN32
+  fl_beep(FL_BEEP_QUESTION);
 
   va_start(ap, fmt);
   int r = innards(fmt, ap, fl_no, fl_yes, 0);
@@ -183,9 +299,7 @@ int fl_ask(const char *fmt, ...) {
 int fl_choice(const char*fmt,const char *b0,const char *b1,const char *b2,...){
   va_list ap;
 
-#ifdef WIN32
-  MessageBeep(MB_ICONQUESTION);
-#endif // WIN32
+  fl_beep(FL_BEEP_QUESTION);
 
   va_start(ap, b2);
   int r = innards(fmt, ap, b0, b1, b2);
@@ -202,10 +316,7 @@ static const char* input_innards(const char* fmt, va_list ap,
   input->type(type);
   input->show();
   input->value(defstr);
-
-#ifdef WIN32
-  MessageBeep(MB_ICONQUESTION);
-#endif // WIN32
+  input->take_focus();
 
   int r = innards(fmt, ap, fl_cancel, fl_ok, 0);
   input->hide();
@@ -214,6 +325,8 @@ static const char* input_innards(const char* fmt, va_list ap,
 }
 
 const char* fl_input(const char *fmt, const char *defstr, ...) {
+  fl_beep(FL_BEEP_QUESTION);
+
   va_list ap;
   va_start(ap, defstr);
   const char* r = input_innards(fmt, ap, defstr, FL_NORMAL_INPUT);
@@ -222,6 +335,8 @@ const char* fl_input(const char *fmt, const char *defstr, ...) {
 }
 
 const char *fl_password(const char *fmt, const char *defstr, ...) {
+  fl_beep(FL_BEEP_PASSWORD);
+
   va_list ap;
   va_start(ap, defstr);
   const char* r = input_innards(fmt, ap, defstr, FL_SECRET_INPUT);
@@ -230,5 +345,5 @@ const char *fl_password(const char *fmt, const char *defstr, ...) {
 }
 
 //
-// End of "$Id: fl_ask.cxx,v 1.8.2.8 2001/01/22 15:13:40 easysw Exp $".
+// End of "$Id: fl_ask.cxx,v 1.8.2.8.2.14 2004/10/18 20:22:25 easysw Exp $".
 //
