@@ -1,9 +1,9 @@
 //
-// "$Id: fl_boxtype.cxx,v 1.8.2.4 2001/01/22 15:13:40 easysw Exp $"
+// "$Id: fl_boxtype.cxx,v 1.8.2.4.2.14 2004/04/11 04:38:59 easysw Exp $"
 //
 // Box drawing code for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2004 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -49,6 +49,9 @@ static uchar inactive_ramp[24] = {
   49, 49, 50, 50,
   51, 51, 52, 52};
 static int draw_it_active = 1;
+
+int Fl::draw_box_active() { return draw_it_active; }
+
 uchar *fl_gray_ramp() {return (draw_it_active?active_ramp:inactive_ramp)-'A';}
 
 void fl_frame(const char* s, int x, int y, int w, int h) {
@@ -176,11 +179,6 @@ void fl_rectbound(int x, int y, int w, int h, Fl_Color bgcolor) {
 }
 #define fl_border_box fl_rectbound
 
-void fl_rectf(int x, int y, int w, int h, Fl_Color c) {
-  fl_color(c);
-  fl_rectf(x, y, w, h);
-}
-
 void fl_border_frame(int x, int y, int w, int h, Fl_Color c) {
   fl_color(c);
   fl_rect(x, y, w, h);
@@ -191,46 +189,53 @@ void fl_border_frame(int x, int y, int w, int h, Fl_Color c) {
 static struct {
   Fl_Box_Draw_F *f;
   uchar dx, dy, dw, dh;
-} fl_box_table[] = {
+  int set;
+} fl_box_table[256] = {
 // must match list in Enumerations.H!!!
-  {fl_no_box,		0,0,0,0},		
-  {fl_rectf,		0,0,0,0}, // FL_FLAT_BOX
-  {fl_up_box,		D1,D1,D2,D2},
-  {fl_down_box,		D1,D1,D2,D2},
-  {fl_up_frame,		D1,D1,D2,D2},
-  {fl_down_frame,	D1,D1,D2,D2},
-  {fl_thin_up_box,	1,1,2,2},
-  {fl_thin_down_box,	1,1,2,2},
-  {fl_thin_up_frame,	1,1,2,2},
-  {fl_thin_down_frame,	1,1,2,2},
-  {fl_engraved_box,	2,2,4,4},
-  {fl_embossed_box,	2,2,4,4},
-  {fl_engraved_frame,	2,2,4,4},
-  {fl_embossed_frame,	2,2,4,4},
-  {fl_border_box,	1,1,2,2},
-  {fl_border_box,	1,1,2,2}, // _FL_SHADOW_BOX,
-  {fl_border_frame,	1,1,2,2},
-  {fl_border_frame,	1,1,2,2}, // _FL_SHADOW_FRAME,
-  {fl_border_box,	1,1,2,2}, // _FL_ROUNDED_BOX,
-  {fl_border_box,	1,1,2,2}, // _FL_RSHADOW_BOX,
-  {fl_border_frame,	1,1,2,2}, // _FL_ROUNDED_FRAME
-  {fl_rectf,		0,0,0,0}, // _FL_RFLAT_BOX,
-  {fl_up_box,		3,3,6,6}, // _FL_ROUND_UP_BOX
-  {fl_down_box,		3,3,6,6}, // _FL_ROUND_DOWN_BOX,
-  {fl_up_box,		0,0,0,0}, // _FL_DIAMOND_UP_BOX
-  {fl_down_box,		0,0,0,0}, // _FL_DIAMOND_DOWN_BOX
-  {fl_border_box,	1,1,2,2}, // _FL_OVAL_BOX,
-  {fl_border_box,	1,1,2,2}, // _FL_OVAL_SHADOW_BOX,
-  {fl_border_frame,	1,1,2,2}, // _FL_OVAL_FRAME
-  {fl_rectf,		0,0,0,0}, // _FL_OVAL_FLAT_BOX,
-  {fl_up_box,		3,3,6,6}, // FL_FREE_BOX+0
-  {fl_down_box,		3,3,6,6}, // FL_FREE_BOX+1
-  {fl_up_box,		3,3,6,6}, // FL_FREE_BOX+2
-  {fl_down_box,		3,3,6,6}, // FL_FREE_BOX+3
-  {fl_up_box,		3,3,6,6}, // FL_FREE_BOX+4
-  {fl_down_box,		3,3,6,6}, // FL_FREE_BOX+5
-  {fl_up_box,		3,3,6,6}, // FL_FREE_BOX+6
-  {fl_down_box,		3,3,6,6}, // FL_FREE_BOX+7
+  {fl_no_box,		0,0,0,0,1},		
+  {fl_rectf,		0,0,0,0,1}, // FL_FLAT_BOX
+  {fl_up_box,		D1,D1,D2,D2,1},
+  {fl_down_box,		D1,D1,D2,D2,1},
+  {fl_up_frame,		D1,D1,D2,D2,1},
+  {fl_down_frame,	D1,D1,D2,D2,1},
+  {fl_thin_up_box,	1,1,2,2,1},
+  {fl_thin_down_box,	1,1,2,2,1},
+  {fl_thin_up_frame,	1,1,2,2,1},
+  {fl_thin_down_frame,	1,1,2,2,1},
+  {fl_engraved_box,	2,2,4,4,1},
+  {fl_embossed_box,	2,2,4,4,1},
+  {fl_engraved_frame,	2,2,4,4,1},
+  {fl_embossed_frame,	2,2,4,4,1},
+  {fl_border_box,	1,1,2,2,1},
+  {fl_border_box,	1,1,2,2,0}, // _FL_SHADOW_BOX,
+  {fl_border_frame,	1,1,2,2,1},
+  {fl_border_frame,	1,1,2,2,0}, // _FL_SHADOW_FRAME,
+  {fl_border_box,	1,1,2,2,0}, // _FL_ROUNDED_BOX,
+  {fl_border_box,	1,1,2,2,0}, // _FL_RSHADOW_BOX,
+  {fl_border_frame,	1,1,2,2,0}, // _FL_ROUNDED_FRAME
+  {fl_rectf,		0,0,0,0,0}, // _FL_RFLAT_BOX,
+  {fl_up_box,		3,3,6,6,0}, // _FL_ROUND_UP_BOX
+  {fl_down_box,		3,3,6,6,0}, // _FL_ROUND_DOWN_BOX,
+  {fl_up_box,		0,0,0,0,0}, // _FL_DIAMOND_UP_BOX
+  {fl_down_box,		0,0,0,0,0}, // _FL_DIAMOND_DOWN_BOX
+  {fl_border_box,	1,1,2,2,0}, // _FL_OVAL_BOX,
+  {fl_border_box,	1,1,2,2,0}, // _FL_OVAL_SHADOW_BOX,
+  {fl_border_frame,	1,1,2,2,0}, // _FL_OVAL_FRAME
+  {fl_rectf,		0,0,0,0,0}, // _FL_OVAL_FLAT_BOX,
+  {fl_up_box,		4,4,8,8,0}, // _FL_PLASTIC_UP_BOX,
+  {fl_down_box,		2,2,4,4,0}, // _FL_PLASTIC_DOWN_BOX,
+  {fl_up_frame,		2,2,4,4,0}, // _FL_PLASTIC_UP_FRAME,
+  {fl_down_frame,	2,2,4,4,0}, // _FL_PLASTIC_DOWN_FRAME,
+  {fl_up_box,		2,2,4,4,0}, // _FL_PLASTIC_THIN_UP_BOX,
+  {fl_down_box,		2,2,4,4,0}, // _FL_PLASTIC_THIN_DOWN_BOX,
+  {fl_up_box,		3,3,6,6,0}, // FL_FREE_BOX+0
+  {fl_down_box,		3,3,6,6,0}, // FL_FREE_BOX+1
+  {fl_up_box,		3,3,6,6,0}, // FL_FREE_BOX+2
+  {fl_down_box,		3,3,6,6,0}, // FL_FREE_BOX+3
+  {fl_up_box,		3,3,6,6,0}, // FL_FREE_BOX+4
+  {fl_down_box,		3,3,6,6,0}, // FL_FREE_BOX+5
+  {fl_up_box,		3,3,6,6,0}, // FL_FREE_BOX+6
+  {fl_down_box,		3,3,6,6,0}, // FL_FREE_BOX+7
 };
 
 int Fl::box_dx(Fl_Boxtype t) {return fl_box_table[t].dx;}
@@ -238,15 +243,25 @@ int Fl::box_dy(Fl_Boxtype t) {return fl_box_table[t].dy;}
 int Fl::box_dw(Fl_Boxtype t) {return fl_box_table[t].dw;}
 int Fl::box_dh(Fl_Boxtype t) {return fl_box_table[t].dh;}
 
-void fl_internal_boxtype(Fl_Boxtype t, Fl_Box_Draw_F* f) {fl_box_table[t].f=f;}
+void fl_internal_boxtype(Fl_Boxtype t, Fl_Box_Draw_F* f) {
+  if (!fl_box_table[t].set) {
+    fl_box_table[t].f   = f;
+    fl_box_table[t].set = 1;
+  }
+}
+
+Fl_Box_Draw_F *Fl::get_boxtype(Fl_Boxtype t) {
+  return fl_box_table[t].f;
+}
 
 void Fl::set_boxtype(Fl_Boxtype t, Fl_Box_Draw_F* f,
 		      uchar a, uchar b, uchar c, uchar d) {
-  fl_box_table[t].f=f;
-  fl_box_table[t].dx = a;
-  fl_box_table[t].dy = b;
-  fl_box_table[t].dw = c;
-  fl_box_table[t].dh = d;
+  fl_box_table[t].f   = f;
+  fl_box_table[t].set = 1;
+  fl_box_table[t].dx  = a;
+  fl_box_table[t].dy  = b;
+  fl_box_table[t].dw  = c;
+  fl_box_table[t].dh  = d;
 }
 
 void Fl::set_boxtype(Fl_Boxtype t, Fl_Boxtype f) {
@@ -254,10 +269,10 @@ void Fl::set_boxtype(Fl_Boxtype t, Fl_Boxtype f) {
 }
 
 void fl_draw_box(Fl_Boxtype t, int x, int y, int w, int h, Fl_Color c) {
-  if (t) fl_box_table[t].f(x,y,w,h,c);
+  if (t && fl_box_table[t].f) fl_box_table[t].f(x,y,w,h,c);
 }
 
-//extern Fl_Widget *fl_boxcheat; // hack set by Fl_Window.C
+//extern Fl_Widget *fl_boxcheat; // hack set by Fl_Window.cxx
 
 void Fl_Widget::draw_box() const {
   int t = box_;
@@ -274,13 +289,13 @@ void Fl_Widget::draw_box(Fl_Boxtype b, Fl_Color c) const {
   draw_box(b, x_, y_, w_, h_, c);
 }
 
-void Fl_Widget::draw_box(Fl_Boxtype b, int x, int y, int w, int h, Fl_Color c)
+void Fl_Widget::draw_box(Fl_Boxtype b, int X, int Y, int W, int H, Fl_Color c)
 const {
   draw_it_active = active_r();
-  fl_box_table[b].f(x, y, w, h, c);
+  fl_box_table[b].f(X, Y, W, H, c);
   draw_it_active = 1;
 }
 
 //
-// End of "$Id: fl_boxtype.cxx,v 1.8.2.4 2001/01/22 15:13:40 easysw Exp $".
+// End of "$Id: fl_boxtype.cxx,v 1.8.2.4.2.14 2004/04/11 04:38:59 easysw Exp $".
 //
