@@ -1,9 +1,9 @@
 //
-// "$Id: fl_color.cxx,v 1.12.2.5 2001/01/22 15:13:40 easysw Exp $"
+// "$Id: fl_color.cxx,v 1.12.2.5.2.9 2004/04/11 04:38:59 easysw Exp $"
 //
 // Color functions for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2004 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -26,7 +26,9 @@
 // Implementation of fl_color(i), fl_color(r,g,b).
 
 #ifdef WIN32
-#include "fl_color_win32.cxx"
+#  include "fl_color_win32.cxx"
+#elif defined(__APPLE__)
+#  include "fl_color_mac.cxx"
 #else
 
 // Also code to look at the X visual and figure out the best way to turn
@@ -36,10 +38,10 @@
 // being used to index arrays.  So I always copy them to an integer
 // before use.
 
-#include "Fl_XColor.H"
-#include <FL/Fl.H>
-#include <FL/x.H>
-#include <FL/fl_draw.H>
+#  include "Fl_XColor.H"
+#  include <FL/Fl.H>
+#  include <FL/x.H>
+#  include <FL/fl_draw.H>
 
 ////////////////////////////////////////////////////////////////
 // figure_out_visual() calculates masks & shifts for generating
@@ -52,12 +54,12 @@ static uchar beenhere;
 static void figure_out_visual() {
   beenhere = 1;
   if (!fl_visual->red_mask || !fl_visual->green_mask || !fl_visual->blue_mask){
-#if USE_COLORMAP
+#  if USE_COLORMAP
     fl_redmask = 0;
     return;
-#else
+#  else
     Fl::fatal("Requires true color visual");
-#endif
+#  endif
   }
 
   // get the bit masks into a more useful form:
@@ -90,19 +92,19 @@ static void figure_out_visual() {
 }
 
 static unsigned fl_cmap[256] = {
-#include "fl_cmap.h" // this is a file produced by "cmap.C":
+#include "fl_cmap.h" // this is a file produced by "cmap.cxx":
 };
 
-#if HAVE_OVERLAY
+#  if HAVE_OVERLAY
 Fl_XColor fl_xmap[2][256];
 uchar fl_overlay;
 Colormap fl_overlay_colormap;
 XVisualInfo* fl_overlay_visual;
 ulong fl_transparent_pixel;
-#else
+#  else
 Fl_XColor fl_xmap[1][256];
-#define fl_overlay 0
-#endif
+#    define fl_overlay 0
+#  endif
 
 ////////////////////////////////////////////////////////////////
 // Get an rgb color.  This is easy for a truecolor visual.  For
@@ -113,7 +115,7 @@ Fl_XColor fl_xmap[1][256];
 
 ulong fl_xpixel(uchar r,uchar g,uchar b) {
   if (!beenhere) figure_out_visual();
-#if USE_COLORMAP
+#  if USE_COLORMAP
   if (!fl_redmask) {
     // find closest entry in the colormap:
     Fl_Color i =
@@ -125,7 +127,7 @@ ulong fl_xpixel(uchar r,uchar g,uchar b) {
       fl_cmap[i] = (r<<24)|(g<<16)|(b<<8);
     return fl_xpixel(i); // allocate an X color
   }
-#endif
+#  endif
   return
     (((r&fl_redmask) << fl_redshift)+
      ((g&fl_greenmask)<<fl_greenshift)+
@@ -134,6 +136,7 @@ ulong fl_xpixel(uchar r,uchar g,uchar b) {
 }
 
 void fl_color(uchar r,uchar g,uchar b) {
+  fl_color_ = fl_rgb_color(r, g, b);
   XSetForeground(fl_display, fl_gc, fl_xpixel(r,g,b));
 }
 
@@ -145,7 +148,7 @@ void fl_color(uchar r,uchar g,uchar b) {
 
 // calculate what color is actually on the screen for a mask:
 static inline uchar realcolor(uchar color, uchar mask) {
-#if 0
+#  if 0
   // accurate version if the display has linear gamma, but fl_draw_image
   // works better with the simpler version on most screens...
   uchar m = mask;
@@ -157,12 +160,16 @@ static inline uchar realcolor(uchar color, uchar mask) {
     result |= color&m;
   }
   return result;
-#else
+#  else
   return (color&mask) | (~mask)&(mask>>1);
-#endif
+#  endif
 }
 
 ulong fl_xpixel(Fl_Color i) {
+  if (i & 0xffffff00) {
+    return fl_xpixel((i >> 24) & 255, (i >> 16) & 255, (i >> 8) & 255);
+  }
+
   Fl_XColor &xmap = fl_xmap[fl_overlay][i];
   if (xmap.mapped) return xmap.pixel;
 
@@ -171,13 +178,13 @@ ulong fl_xpixel(Fl_Color i) {
   uchar r,g,b;
   {unsigned c = fl_cmap[i]; r=uchar(c>>24); g=uchar(c>>16); b=uchar(c>>8);}
 
-#if USE_COLORMAP
+#  if USE_COLORMAP
   Colormap colormap = fl_colormap;
-#if HAVE_OVERLAY
+#    if HAVE_OVERLAY
   if (fl_overlay) colormap = fl_overlay_colormap; else
-#endif
+#    endif
   if (fl_redmask) {
-#endif
+#  endif
     // return color for a truecolor visual:
     xmap.mapped = 2; // 2 prevents XFreeColor from being called
     xmap.r = realcolor(r, fl_redmask);
@@ -188,17 +195,17 @@ ulong fl_xpixel(Fl_Color i) {
        ((g&fl_greenmask)<<fl_greenshift)+
        ((b&fl_bluemask)<< fl_blueshift)
        ) >> fl_extrashift;
-#if USE_COLORMAP
+#  if USE_COLORMAP
   }
-#if HAVE_OVERLAY
+#    if HAVE_OVERLAY
   static XColor* ac[2];
   XColor*& allcolors = ac[fl_overlay];
   static int nc[2];
   int& numcolors = nc[fl_overlay];
-#else
+#    else
   static XColor *allcolors;
   static int numcolors;
-#endif
+#    endif
 
   // I don't try to allocate colors with XAllocColor once it fails
   // with any color.  It is possible that it will work, since a color
@@ -218,9 +225,9 @@ ulong fl_xpixel(Fl_Color i) {
     // I only read the colormap once.  Again this is due to the slowness
     // of round-trips to the X server, even though other programs may alter
     // the colormap after this and make decisions here wrong.
-#if HAVE_OVERLAY
+#    if HAVE_OVERLAY
     if (fl_overlay) numcolors = fl_overlay_visual->colormap_size; else
-#endif
+#    endif
       numcolors = fl_visual->colormap_size;
     if (!allcolors) allcolors = new XColor[numcolors];
     for (int p = numcolors; p--;) allcolors[p].pixel = p;
@@ -231,9 +238,9 @@ ulong fl_xpixel(Fl_Color i) {
   int mindist = 0x7FFFFFFF;
   unsigned int bestmatch = 0;
   for (unsigned int n = numcolors; n--;) {
-#if HAVE_OVERLAY
+#    if HAVE_OVERLAY
     if (fl_overlay && n == fl_transparent_pixel) continue;
-#endif
+#    endif
     XColor &a = allcolors[n];
     int d, t;
     t = int(r)-int(a.red>>8); d = t*t;
@@ -263,31 +270,36 @@ ulong fl_xpixel(Fl_Color i) {
   xmap.g = p.green>>8;
   xmap.b = p.blue>>8;
   return xmap.pixel;
-#endif
+#  endif
 }
 
 Fl_Color fl_color_;
 
 void fl_color(Fl_Color i) {
-  fl_color_ = i;
-  XSetForeground(fl_display, fl_gc, fl_xpixel(i));
+  if (i & 0xffffff00) {
+    unsigned rgb = (unsigned)i;
+    fl_color((uchar)(rgb >> 24), (uchar)(rgb >> 16), (uchar)(rgb >> 8));
+  } else {
+    fl_color_ = i;
+    XSetForeground(fl_display, fl_gc, fl_xpixel(i));
+  }
 }
 
 void Fl::free_color(Fl_Color i, int overlay) {
-#if HAVE_OVERLAY
-#else
+#  if HAVE_OVERLAY
+#  else
   if (overlay) return;
-#endif
+#  endif
   if (fl_xmap[overlay][i].mapped) {
-#if USE_COLORMAP
-#if HAVE_OVERLAY
+#  if USE_COLORMAP
+#    if HAVE_OVERLAY
     Colormap colormap = overlay ? fl_overlay_colormap : fl_colormap;
-#else
+#    else
     Colormap colormap = fl_colormap;
-#endif
+#    endif
     if (fl_xmap[overlay][i].mapped == 1)
       XFreeColors(fl_display, colormap, &(fl_xmap[overlay][i].pixel), 1, 0);
-#endif
+#  endif
     fl_xmap[overlay][i].mapped = 0;
   }
 }
@@ -295,9 +307,9 @@ void Fl::free_color(Fl_Color i, int overlay) {
 void Fl::set_color(Fl_Color i, unsigned c) {
   if (fl_cmap[i] != c) {
     free_color(i,0);
-#if HAVE_OVERLAY
+#  if HAVE_OVERLAY
     free_color(i,1);
-#endif
+#  endif
     fl_cmap[i] = c;
   }
 }
@@ -305,55 +317,65 @@ void Fl::set_color(Fl_Color i, unsigned c) {
 #endif // end of X-specific code
 
 unsigned Fl::get_color(Fl_Color i) {
-  return fl_cmap[i];
+  if (i & 0xffffff00) return (i);
+  else return fl_cmap[i];
 }
 
 void Fl::set_color(Fl_Color i, uchar red, uchar green, uchar blue) {
-  Fl::set_color(i,
+  Fl::set_color((Fl_Color)(i & 255),
 	((unsigned)red<<24)+((unsigned)green<<16)+((unsigned)blue<<8));
 }
 
 void Fl::get_color(Fl_Color i, uchar &red, uchar &green, uchar &blue) {
-  unsigned c = fl_cmap[i];
+  unsigned c;
+
+  if (i & 0xffffff00) c = (unsigned)i;
+  else c = fl_cmap[i];
+
   red   = uchar(c>>24);
   green = uchar(c>>16);
   blue  = uchar(c>>8);
 }
 
 Fl_Color fl_color_average(Fl_Color color1, Fl_Color color2, float weight) {
-  Fl_Color avg;
-  unsigned rgb1 = fl_cmap[color1];
-  unsigned rgb2 = fl_cmap[color2];
+  unsigned rgb1;
+  unsigned rgb2;
   uchar r, g, b;
 
+  if (color1 & 0xffffff00) rgb1 = color1;
+  else rgb1 = fl_cmap[color1 & 255];
+
+  if (color2 & 0xffffff00) rgb2 = color2;
+  else rgb2 = fl_cmap[color2 & 255];
+
   r = (uchar)(((uchar)(rgb1>>24))*weight + ((uchar)(rgb2>>24))*(1-weight));
   g = (uchar)(((uchar)(rgb1>>16))*weight + ((uchar)(rgb2>>16))*(1-weight));
   b = (uchar)(((uchar)(rgb1>>8))*weight + ((uchar)(rgb2>>8))*(1-weight));
 
-  if (r == g && r == b) { // get it out of gray ramp
-    avg = fl_gray_ramp(r*FL_NUM_GRAY/256);
-  } else {		// get it out of color cube:
-    avg = fl_color_cube(r*FL_NUM_RED/256,g*FL_NUM_GREEN/256,b*FL_NUM_BLUE/256);
-  }
-
-  return avg;
+  return fl_rgb_color(r, g, b);
 }
 
-Fl_Color inactive(Fl_Color c) {
+Fl_Color fl_inactive(Fl_Color c) {
   return fl_color_average(c, FL_GRAY, .33f);
 }
 
-Fl_Color contrast(Fl_Color fg, Fl_Color bg) {
-  int c1 = int(fl_cmap[fg]);
-  int c2 = int(fl_cmap[bg]);
+Fl_Color fl_contrast(Fl_Color fg, Fl_Color bg) {
+  unsigned c1, c2;
+
+  if (fg & 0xffffff00) c1 = (unsigned)fg;
+  else c1 = fl_cmap[fg];
+
+  if (bg & 0xffffff00) c2 = (unsigned)bg;
+  else c2 = fl_cmap[bg];
+
   if ((c1^c2)&0x80800000)
     return fg;
   else if (c2&0x80800000)
-    return FL_GRAY_RAMP; // black from gray ramp
+    return FL_BLACK;
   else
-    return (Fl_Color)(FL_COLOR_CUBE-1); // white from gray ramp
+    return FL_WHITE;
 }
 
 //
-// End of "$Id: fl_color.cxx,v 1.12.2.5 2001/01/22 15:13:40 easysw Exp $".
+// End of "$Id: fl_color.cxx,v 1.12.2.5.2.9 2004/04/11 04:38:59 easysw Exp $".
 //
