@@ -1,9 +1,9 @@
 //
-// "$Id: fl_diamond_box.cxx,v 1.5.2.3 2001/01/22 15:13:40 easysw Exp $"
+// "$Id: fl_diamond_box.cxx,v 1.5.2.3.2.4 2004/04/11 04:38:59 easysw Exp $"
 //
 // Diamond box code for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2004 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -67,12 +67,12 @@ static void fl_diamond_down_box(int x,int y,int w,int h,Fl_Color bgcolor) {
 }
 
 extern void fl_internal_boxtype(Fl_Boxtype, Fl_Box_Draw_F*);
-Fl_Boxtype define_FL_DIAMOND_BOX() {
+Fl_Boxtype fl_define_FL_DIAMOND_BOX() {
   fl_internal_boxtype(_FL_DIAMOND_DOWN_BOX, fl_diamond_down_box);
   fl_internal_boxtype(_FL_DIAMOND_UP_BOX,fl_diamond_up_box);
   return _FL_DIAMOND_UP_BOX;
 }
 
 //
-// End of "$Id: fl_diamond_box.cxx,v 1.5.2.3 2001/01/22 15:13:40 easysw Exp $".
+// End of "$Id: fl_diamond_box.cxx,v 1.5.2.3.2.4 2004/04/11 04:38:59 easysw Exp $".
 //
