@@ -1,9 +1,9 @@
 //
-// "$Id: fl_draw_image_win32.cxx,v 1.4.2.3 2001/01/22 15:13:40 easysw Exp $"
+// "$Id: fl_draw_image_win32.cxx,v 1.4.2.3.2.6 2004/04/11 04:38:59 easysw Exp $"
 //
 // WIN32 image drawing code for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2004 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -51,7 +51,7 @@
 
 #if USE_COLORMAP
 
-// error-diffusion dither into the fltk colormap
+// error-diffusion dither into the FLTK colormap
 static void dither(uchar* to, const uchar* from, int w, int delta) {
   static int ri, gi, bi, dir;
   int r=ri, g=gi, b=bi;
@@ -82,7 +82,7 @@ static void dither(uchar* to, const uchar* from, int w, int delta) {
   ri = r; gi = g; bi = b;
 }
 
-// error-diffusion dither into the fltk colormap
+// error-diffusion dither into the FLTK colormap
 static void monodither(uchar* to, const uchar* from, int w, int delta) {
   static int ri,dir;
   int r=ri;
@@ -137,17 +137,17 @@ static void innards(const uchar *buf, int X, int Y, int W, int H,
   }
 #if USE_COLORMAP
   if (indexed) {
-    for (int i=0; i<256; i++) {
+    for (short i=0; i<256; i++) {
       *((short*)(bmi.bmiColors)+i) = i;
     }
   } else
 #endif
   if (mono) {
     for (int i=0; i<256; i++) {
-      bmi.bmiColors[i].rgbBlue = i;
-      bmi.bmiColors[i].rgbGreen = i;
-      bmi.bmiColors[i].rgbRed = i;
-      bmi.bmiColors[i].rgbReserved = i;
+      bmi.bmiColors[i].rgbBlue = (uchar)i;
+      bmi.bmiColors[i].rgbGreen = (uchar)i;
+      bmi.bmiColors[i].rgbRed = (uchar)i;
+      bmi.bmiColors[i].rgbReserved = (uchar)i;
     }
   }
   bmi.bmiHeader.biWidth = w;
@@ -258,5 +258,5 @@ void fl_rectf(int x, int y, int w, int h, uchar r, uchar g, uchar b) {
 }
 
 //
-// End of "$Id: fl_draw_image_win32.cxx,v 1.4.2.3 2001/01/22 15:13:40 easysw Exp $".
+// End of "$Id: fl_draw_image_win32.cxx,v 1.4.2.3.2.6 2004/04/11 04:38:59 easysw Exp $".
 //
