@@ -1,9 +1,9 @@
 //
-// "$Id: fl_draw_pixmap.cxx,v 1.4.2.8 2001/04/13 19:07:40 easysw Exp $"
+// "$Id: fl_draw_pixmap.cxx,v 1.4.2.8.2.16 2004/09/11 19:32:43 easysw Exp $"
 //
 // Pixmap drawing code for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2004 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -33,13 +33,11 @@
 // as I want to discourage programs that require support files to work.
 // All data needed by a program ui should be compiled in!!!
 
-#include <config.h>
 #include <FL/Fl.H>
 #include <FL/fl_draw.H>
 #include <FL/x.H>
-#include <ctype.h>
 #include <stdio.h>
-#include <string.h>
+#include "flstring.h"
 
 static int ncolors, chars_per_pixel;
 
@@ -48,7 +46,7 @@ int fl_measure_pixmap(/*const*/ char* const* data, int &w, int &h) {
 }
 
 int fl_measure_pixmap(const char * const *data, int &w, int &h) {
-  int i = sscanf(data[0],"%d %d %d %d",&w,&h,&ncolors,&chars_per_pixel);
+  int i = sscanf(data[0],"%d%d%d%d",&w,&h,&ncolors,&chars_per_pixel);
   if (i<4 || w<=0 || h<=0 ||
       chars_per_pixel!=1 && chars_per_pixel!=2) return w=0;
   return 1;
@@ -71,12 +69,20 @@ static void cb1(void*v, int x, int y, int w, uchar* buf) {
   pixmap_data& d = *(pixmap_data*)v;
   const uchar* p = d.data[y]+x;
   U64* q = (U64*)buf;
-  for (int X=(w+1)/2; X--; p += 2) {
-#if WORDS_BIGENDIAN
-    *q++ = (d.colors[p[0]]<<32) | d.colors[p[1]];
-#else
-    *q++ = (d.colors[p[1]]<<32) | d.colors[p[0]];
-#endif
+  for (int X=w; X>0; X-=2, p += 2) {
+    if (X>1) {
+#  if WORDS_BIGENDIAN
+      *q++ = (d.colors[p[0]]<<32) | d.colors[p[1]];
+#  else
+      *q++ = (d.colors[p[1]]<<32) | d.colors[p[0]];
+#  endif
+    } else {
+#  if WORDS_BIGENDIAN
+      *q++ = d.colors[p[0]]<<32;
+#  else
+      *q++ = d.colors[p[0]];
+#  endif
+    }
   }
 }
 
@@ -85,20 +91,28 @@ static void cb2(void*v, int x, int y, int w, uchar* buf) {
   pixmap_data& d = *(pixmap_data*)v;
   const uchar* p = d.data[y]+2*x;
   U64* q = (U64*)buf;
-  for (int X=(w+1)/2; X--;) {
+  for (int X=w; X>0; X-=2) {
     U64* colors = d.byte1[*p++];
     int index = *p++;
-    U64* colors1 = d.byte1[*p++];
-    int index1 = *p++;
-#if WORDS_BIGENDIAN
-    *q++ = (colors[index]<<32) | colors1[index1];
-#else
-    *q++ = (colors1[index1]<<32) | colors[index];
-#endif
+    if (X>1) {
+      U64* colors1 = d.byte1[*p++];
+      int index1 = *p++;
+#  if WORDS_BIGENDIAN
+      *q++ = (colors[index]<<32) | colors1[index1];
+#  else
+      *q++ = (colors1[index1]<<32) | colors[index];
+#  endif
+    } else {
+#  if WORDS_BIGENDIAN
+      *q++ = colors[index]<<32;
+#  else
+      *q++ = colors[index];
+#  endif
+    }
   }
 }
 
-#else
+#else // U32
 
 // The callback from fl_draw_image to get a row of data passes this:
 struct pixmap_data {
@@ -110,6 +124,8 @@ struct pixmap_data {
   };
 };
 
+#  ifndef __APPLE_QUARTZ__
+
 // callback for 1 byte per pixel:
 static void cb1(void*v, int x, int y, int w, uchar* buf) {
   pixmap_data& d = *(pixmap_data*)v;
@@ -129,12 +145,9 @@ static void cb2(void*v, int x, int y, int w, uchar* buf) {
   }
 }
 
-#endif
+#  endif  // !__APPLE_QUARTZ__
 
-#ifdef WIN32
-// this is in Fl_arg.C:
-extern int fl_parse_color(const char*, uchar&, uchar&, uchar&);
-#endif
+#endif // U64 else U32
 
 uchar **fl_mask_bitmap; // if non-zero, create bitmap and store pointer here
 
@@ -148,18 +161,18 @@ int fl_draw_pixmap(const char*const* di, int x, int y, Fl_Color bg) {
   const uchar*const* data = (const uchar*const*)(di+1);
   int transparent_index = -1;
 
-  if (ncolors < 0) {	// fltk (non standard) compressed colormap
+  if (ncolors < 0) {	// FLTK (non standard) compressed colormap
     ncolors = -ncolors;
     const uchar *p = *data++;
     // if first color is ' ' it is transparent (put it later to make
     // it not be transparent):
     if (*p == ' ') {
-      uchar* c = (uchar*)&d.colors[' '];
+      uchar* c = (uchar*)&d.colors[(int)' '];
 #ifdef U64
       *(U64*)c = 0;
-#if WORDS_BIGENDIAN
+#  if WORDS_BIGENDIAN
       c += 4;
-#endif
+#  endif
 #endif
       transparent_index = ' ';
       Fl::get_color(bg, c[0], c[1], c[2]); c[3] = 0;
@@ -171,39 +184,44 @@ int fl_draw_pixmap(const char*const* di, int x, int y, Fl_Color bg) {
       uchar* c = (uchar*)&d.colors[*p++];
 #ifdef U64
       *(U64*)c = 0;
-#if WORDS_BIGENDIAN
+#  if WORDS_BIGENDIAN
       c += 4;
-#endif
+#  endif
 #endif
       *c++ = *p++;
       *c++ = *p++;
       *c++ = *p++;
+#ifdef __APPLE_QUARTZ__
+      *c = 255;
+#else
       *c = 0;
+#endif
     }
   } else {	// normal XPM colormap with names
     if (chars_per_pixel>1) memset(d.byte1, 0, sizeof(d.byte1));
     for (int i=0; i<ncolors; i++) {
       const uchar *p = *data++;
       // the first 1 or 2 characters are the color index:
-      int index = *p++;
+      int ind = *p++;
       uchar* c;
       if (chars_per_pixel>1) {
 #ifdef U64
-	U64* colors = d.byte1[index];
-	if (!colors) colors = d.byte1[index] = new U64[256];
+	U64* colors = d.byte1[ind];
+	if (!colors) colors = d.byte1[ind] = new U64[256];
 #else
-	U32* colors = d.byte1[index];
-	if (!colors) colors = d.byte1[index] = new U32[256];
+	U32* colors = d.byte1[ind];
+	if (!colors) colors = d.byte1[ind] = new U32[256];
 #endif
 	c = (uchar*)&colors[*p];
-	index = (index<<8)+*p++;
+	ind = (ind<<8)|*p++;
       } else {
-	c = (uchar *)&d.colors[index];
+	c = (uchar *)&d.colors[ind];
       }
       // look for "c word", or last word if none:
       const uchar *previous_word = p;
       for (;;) {
-	while (*p && isspace(*p)) p++; uchar what = *p++;
+	while (*p && isspace(*p)) p++;
+	uchar what = *p++;
 	while (*p && !isspace(*p)) p++;
 	while (*p && isspace(*p)) p++;
 	if (!*p) {p = previous_word; break;}
@@ -213,36 +231,38 @@ int fl_draw_pixmap(const char*const* di, int x, int y, Fl_Color bg) {
       }
 #ifdef U64
       *(U64*)c = 0;
-#if WORDS_BIGENDIAN
+#  if WORDS_BIGENDIAN
       c += 4;
+#  endif
 #endif
+#ifdef __APPLE_QUARTZ__
+      c[3] = 255;
 #endif
-#ifdef WIN32
-      if (fl_parse_color((const char*)p, c[0], c[1], c[2])) {;
-#else
-      XColor x;
-      if (XParseColor(fl_display, fl_colormap, (const char*)p, &x)) {
-	c[0] = x.red>>8; c[1] = x.green>>8; c[2] = x.blue>>8;
-#endif
-      } else { // assumme "None" or "#transparent" for any errors
-	// this should be transparent...
+      if (!fl_parse_color((const char*)p, c[0], c[1], c[2])) {
+        // assume "None" or "#transparent" for any errors
+	// "bg" should be transparent...
 	Fl::get_color(bg, c[0], c[1], c[2]);
-	transparent_index = index;
+#ifdef __APPLE_QUARTZ__
+        c[3] = 0;
+#endif
+	transparent_index = ind;
       }
     }
   }
   d.data = data;
 
+#ifndef __APPLE_QUARTZ__
+
   // build the mask bitmap used by Fl_Pixmap:
   if (fl_mask_bitmap && transparent_index >= 0) {
     int W = (d.w+7)/8;
     uchar* bitmap = new uchar[W * d.h];
     *fl_mask_bitmap = bitmap;
-    for (int y = 0; y < d.h; y++) {
-      const uchar* p = data[y];
+    for (int Y = 0; Y < d.h; Y++) {
+      const uchar* p = data[Y];
       if (chars_per_pixel <= 1) {
-	for (int x = 0; x < W; x++) {
-	  int b = (*p++ != transparent_index);
+	for (int X = 0; X < W; X++) {
+	  uchar b = (*p++ != transparent_index);
 	  if (*p++ != transparent_index) b |= 2;
 	  if (*p++ != transparent_index) b |= 4;
 	  if (*p++ != transparent_index) b |= 8;
@@ -253,24 +273,66 @@ int fl_draw_pixmap(const char*const* di, int x, int y, Fl_Color bg) {
 	  *bitmap++ = b;
 	}
       } else {
-	for (int x = 0; x < W; x++) {
-	  int b = 0;
-	  for (int i = 0; i < 8; i++) {
-	    int index = *p++;
-	    index = (index<<8) | (*p++);
-	    if (index != transparent_index) b |= (1<<i);
+        uchar b = 0, bit = 1;
+	for (int X = 0; X < d.w; X++) {
+	  int ind = *p++;
+	  ind = (ind<<8) | (*p++);
+	  if (ind != transparent_index) b |= bit;
+
+          if (bit < 128) bit <<= 1;
+	  else {
+	    *bitmap++ = b;
+	    b = 0;
+	    bit = 1;
 	  }
-	  *bitmap++ = b;
 	}
+
+        if (bit > 1) *bitmap++ = b;
       }
     }
   }
 
   fl_draw_image(chars_per_pixel==1 ? cb1 : cb2, &d, x, y, d.w, d.h, 4);
-  if (chars_per_pixel > 1) for (int i = 0; i < 256; i++) delete d.byte1[i];
+
+#else // __APPLE_QUARTZ__
+
+  bool transparent = (transparent_index>=0);
+  transparent = true;
+  U32 *array = new U32[d.w * d.h], *q = array;
+  for (int Y = 0; Y < d.h; Y++) {
+    const uchar* p = data[Y];
+    if (chars_per_pixel <= 1) {
+      for (int X = 0; X < d.w; X++) {
+        *q++ = d.colors[*p++];
+      }
+    } else {
+      for (int X = 0; X < d.w; X++) {
+        U32* colors = d.byte1[*p++];
+        *q++ = colors[*p++];
+      }
+    }
+  }
+  CGColorSpaceRef lut = CGColorSpaceCreateDeviceRGB();
+  CGDataProviderRef src = CGDataProviderCreateWithData( 0L, array, d.w * d.h * 4, 0L);
+  CGImageRef img = CGImageCreate(d.w, d.h, 8, 4*8, 4*d.w,
+        lut, transparent?kCGImageAlphaLast:kCGImageAlphaNoneSkipLast,
+        src, 0L, false, kCGRenderingIntentDefault);
+  CGColorSpaceRelease(lut);
+  CGDataProviderRelease(src);
+  CGRect rect = { x, y, d.w, d.h };
+  Fl_X::q_begin_image(rect, x, y, d.w, d.h);
+  CGContextDrawImage(fl_gc, rect, img);
+  Fl_X::q_end_image();
+  CGContextFlush(fl_gc);
+  CGImageRelease(img);
+  delete array;
+
+#endif // !__APPLE_QUARTZ__
+
+  if (chars_per_pixel > 1) for (int i = 0; i < 256; i++) delete[] d.byte1[i];
   return 1;
 }
 
 //
-// End of "$Id: fl_draw_pixmap.cxx,v 1.4.2.8 2001/04/13 19:07:40 easysw Exp $".
+// End of "$Id: fl_draw_pixmap.cxx,v 1.4.2.8.2.16 2004/09/11 19:32:43 easysw Exp $".
 //
