@@ -1,9 +1,9 @@
 //
-// "$Id: fl_font_win32.cxx,v 1.9.2.3 2001/01/22 15:13:41 easysw Exp $"
+// "$Id: fl_font_win32.cxx,v 1.9.2.3.2.9 2004/09/11 18:06:29 easysw Exp $"
 //
 // WIN32 font selection routines for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2004 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -23,16 +23,6 @@
 // Please report all bugs and problems to "fltk-bugs@fltk.org".
 //
 
-#include <config.h>
-#include <FL/Fl.H>
-#include <FL/fl_draw.H>
-#include <FL/win32.H>
-#include "Fl_Font.H"
-
-#include <ctype.h>
-#include <stdlib.h>
-#include <string.h>
-
 Fl_FontSize::Fl_FontSize(const char* name, int size) {
   int weight = FW_NORMAL;
   int italic = 0;
@@ -129,8 +119,8 @@ static Fl_FontSize* find(int fnum, int size) {
 ////////////////////////////////////////////////////////////////
 // Public interface:
 
-int fl_font_;
-int fl_size_;
+int fl_font_ = 0;
+int fl_size_ = 0;
 //static HDC font_gc;
 
 void fl_font(int fnum, int size) {
@@ -140,39 +130,38 @@ void fl_font(int fnum, int size) {
 }
 
 int fl_height() {
-  return (fl_fontsize->metr.tmAscent + fl_fontsize->metr.tmDescent);
+  if (fl_fontsize) return (fl_fontsize->metr.tmAscent + fl_fontsize->metr.tmDescent);
+  else return -1;
 }
 
 int fl_descent() {
-  return fl_fontsize->metr.tmDescent;
-}
-
-double fl_width(const char* c) {
-  double w = 0.0;
-  while (*c) w += fl_fontsize->width[uchar(*c++)];
-  return w;
+  if (fl_fontsize) return fl_fontsize->metr.tmDescent;
+  else return -1;
 }
 
 double fl_width(const char* c, int n) {
+  if (!fl_fontsize) return -1.0;
   double w = 0.0;
   while (n--) w += fl_fontsize->width[uchar(*c++)];
   return w;
 }
 
 double fl_width(uchar c) {
-  return fl_fontsize->width[c];
+  if (fl_fontsize) return fl_fontsize->width[c];
+  else return -1.0;
 }
 
 void fl_draw(const char* str, int n, int x, int y) {
-  SetTextColor(fl_gc, fl_RGB());
+  COLORREF oldColor = SetTextColor(fl_gc, fl_RGB());
   SelectObject(fl_gc, fl_fontsize->fid);
   TextOut(fl_gc, x, y, str, n);
+  SetTextColor(fl_gc, oldColor);
 }
 
-void fl_draw(const char* str, int x, int y) {
-  fl_draw(str, strlen(str), x, y);
+void fl_draw(const char* str, int n, float x, float y) {
+  fl_draw(str, n, (int)x, (int)y);
 }
 
 //
-// End of "$Id: fl_font_win32.cxx,v 1.9.2.3 2001/01/22 15:13:41 easysw Exp $".
+// End of "$Id: fl_font_win32.cxx,v 1.9.2.3.2.9 2004/09/11 18:06:29 easysw Exp $".
 //
