@@ -1,9 +1,9 @@
 //
-// "$Id: fl_overlay_visual.cxx,v 1.4.2.4 2001/05/05 23:39:01 spitzak Exp $"
+// "$Id: fl_overlay_visual.cxx,v 1.4.2.5.2.4 2004/04/11 04:39:00 easysw Exp $"
 //
 // X overlay support for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2004 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -26,7 +26,7 @@
 // Return an overlay visual, if any.  Also allocate a colormap and
 // record the depth for fl_color() to use.
 // Another disgusting X interface, based on code extracted and
-// purified with great difficulty from XLayerUtil.C:
+// purified with great difficulty from XLayerUtil.cxx:
 
 #include <config.h>
 #if HAVE_OVERLAY
@@ -76,7 +76,7 @@ XVisualInfo *fl_find_overlay_visual() {
       int num;
       XVisualInfo *v1=XGetVisualInfo(fl_display, VisualIDMask, &templt, &num);
       if (v1->screen == fl_screen && v1->c_class == PseudoColor
-	  && (!v || v1->depth >= v->depth && v1->depth <= 8)) {
+	  && (!v || v1->depth > v->depth && v1->depth <= 8)) {
 	if (v) XFree((char*)v);
 	v = v1;
 	fl_transparent_pixel = overlayInfo[i].value;
@@ -99,5 +99,5 @@ XVisualInfo *fl_find_overlay_visual() {
 #endif
 
 //
-// End of "$Id: fl_overlay_visual.cxx,v 1.4.2.4 2001/05/05 23:39:01 spitzak Exp $".
+// End of "$Id: fl_overlay_visual.cxx,v 1.4.2.5.2.4 2004/04/11 04:39:00 easysw Exp $".
 //
