@@ -1,9 +1,9 @@
 //
-// "$Id: fl_shadow_box.cxx,v 1.4.2.3 2001/01/22 15:13:41 easysw Exp $"
+// "$Id: fl_shadow_box.cxx,v 1.4.2.3.2.4 2004/04/11 04:39:00 easysw Exp $"
 //
 // Shadow box drawing routines for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2004 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -43,12 +43,12 @@ static void fl_shadow_box(int x, int y, int w, int h, Fl_Color c) {
 }
 
 extern void fl_internal_boxtype(Fl_Boxtype, Fl_Box_Draw_F*);
-Fl_Boxtype define_FL_SHADOW_BOX() {
+Fl_Boxtype fl_define_FL_SHADOW_BOX() {
   fl_internal_boxtype(_FL_SHADOW_FRAME, fl_shadow_frame);
   fl_internal_boxtype(_FL_SHADOW_BOX, fl_shadow_box);
   return _FL_SHADOW_BOX;
 }
 
 //
-// End of "$Id: fl_shadow_box.cxx,v 1.4.2.3 2001/01/22 15:13:41 easysw Exp $".
+// End of "$Id: fl_shadow_box.cxx,v 1.4.2.3.2.4 2004/04/11 04:39:00 easysw Exp $".
 //
