@@ -1,9 +1,9 @@
 //
-// "$Id: fl_shortcut.cxx,v 1.4.2.9 2001/01/22 15:13:41 easysw Exp $"
+// "$Id: fl_shortcut.cxx,v 1.4.2.9.2.14 2004/09/21 13:35:40 easysw Exp $"
 //
 // Shortcut support routines for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2004 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -44,8 +44,8 @@
 #include <FL/Fl_Button.H>
 #include <FL/fl_draw.H>
 #include <ctype.h>
-#include <string.h>
-#ifndef WIN32
+#include "flstring.h"
+#if !defined(WIN32) && !defined(__APPLE__)
 #include <FL/x.H>
 #endif
 
@@ -74,10 +74,11 @@ int Fl::test_shortcut(int shortcut) {
   return 0;
 }
 
-#ifdef WIN32 // if not X
+#if defined(WIN32) || defined(__APPLE__) // if not X
 // This table must be in numeric order by fltk (X) keysym number:
 struct Keyname {int key; const char* name;};
 static Keyname table[] = {
+  {' ', "Space"},
   {FL_BackSpace, "Backspace"},
   {FL_Tab,	"Tab"},
   {0xff0b/*XK_Clear*/, "Clear"},
@@ -115,12 +116,20 @@ const char * fl_shortcut_label(int shortcut) {
   static char buf[20];
   char *p = buf;
   if (!shortcut) {*p = 0; return buf;}
+#ifdef __APPLE__
+  // \todo Mac :  we might want to change the symbols for Mac users - consider drawing Apple Symbols... .
+  if (shortcut & FL_SHIFT) {strcpy(p,"Shift+"); p += 6;} //: Mac hollow up arrow
+  if (shortcut & FL_META)  {strcpy(p,"Cmd+"); p += 4;}  //: Mac 'Apple' key
+  if (shortcut & FL_ALT)   {strcpy(p,"Option+"); p += 7;}   //: Mac 'Alt/Option' or fancy switch symbol
+  if (shortcut & FL_CTRL)  {strcpy(p,"Ctrl+"); p += 5;}  //: Mac ctrl key
+#else
   if (shortcut & FL_META) {strcpy(p,"Meta+"); p += 5;}
   if (shortcut & FL_ALT) {strcpy(p,"Alt+"); p += 4;}
   if (shortcut & FL_SHIFT) {strcpy(p,"Shift+"); p += 6;}
   if (shortcut & FL_CTRL) {strcpy(p,"Ctrl+"); p += 5;}
+#endif // __APPLE__
   int key = shortcut & 0xFFFF;
-#ifdef WIN32 // if not X
+#if defined(WIN32) || defined(__APPLE__) // if not X
   if (key >= FL_F && key <= FL_F_Last) {
     *p++ = 'F';
     if (key > FL_F+9) *p++ = (key-FL_F)/10+'0';
@@ -167,20 +176,20 @@ int fl_old_shortcut(const char* s) {
   if (*s == '#') {n |= FL_ALT; s++;}
   if (*s == '+') {n |= FL_SHIFT; s++;}
   if (*s == '^') {n |= FL_CTRL; s++;}
-  if (s[1]) return n | (int)strtol(s,0,0); // allow 0xf00 to get any key
+  if (*s && s[1]) return n | (int)strtol(s,0,0); // allow 0xf00 to get any key
   return n | *s;
 }
 
 // Tests for &x shortcuts in button labels:
 
-int Fl_Widget::test_shortcut(const char *label) {
+int Fl_Widget::test_shortcut(const char *l) {
   char c = Fl::event_text()[0];
-  if (!c || !label) return 0;
+  if (!c || !l) return 0;
   for (;;) {
-    if (!*label) return 0;
-    if (*label++ == '&' && *label) {
-      if (*label == '&') label++;
-      else if (*label == c) return 1;
+    if (!*l) return 0;
+    if (*l++ == '&' && *l) {
+      if (*l == '&') l++;
+      else if (*l == c) return 1;
       else return 0;
     }
   }
@@ -192,5 +201,5 @@ int Fl_Widget::test_shortcut() {
 }
 
 //
-// End of "$Id: fl_shortcut.cxx,v 1.4.2.9 2001/01/22 15:13:41 easysw Exp $".
+// End of "$Id: fl_shortcut.cxx,v 1.4.2.9.2.14 2004/09/21 13:35:40 easysw Exp $".
 //
