@@ -1,9 +1,9 @@
 //
-// "$Id: fl_show_colormap.cxx,v 1.5.2.4 2001/01/22 15:13:41 easysw Exp $"
+// "$Id: fl_show_colormap.cxx,v 1.5.2.4.2.6 2004/04/11 04:39:00 easysw Exp $"
 //
 // Colormap color selection dialog for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2004 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -56,14 +56,14 @@ ColorMenu::ColorMenu(Fl_Color oldcol) :
 
 void ColorMenu::drawbox(Fl_Color c) {
   if (c < 0 || c > 255) return;
-  int x = (c%8)*BOXSIZE+BORDER;
-  int y = (c/8)*BOXSIZE+BORDER;
+  int X = (c%8)*BOXSIZE+BORDER;
+  int Y = (c/8)*BOXSIZE+BORDER;
 #if BORDER_WIDTH < 3
-  if (c == which) fl_draw_box(FL_DOWN_BOX, x+1, y+1, BOXSIZE-1, BOXSIZE-1, c);
-  else fl_draw_box(FL_BORDER_BOX, x, y, BOXSIZE+1, BOXSIZE+1, c);
+  if (c == which) fl_draw_box(FL_DOWN_BOX, X+1, Y+1, BOXSIZE-1, BOXSIZE-1, c);
+  else fl_draw_box(FL_BORDER_BOX, X, Y, BOXSIZE+1, BOXSIZE+1, c);
 #else
   fl_draw_box(c == which ? FL_DOWN_BOX : FL_BORDER_BOX,
-	      x, y, BOXSIZE+1, BOXSIZE+1, c);
+	      X, Y, BOXSIZE+1, BOXSIZE+1, c);
 #endif
 }
 
@@ -102,6 +102,7 @@ int ColorMenu::handle(int e) {
     case FL_Left: if (c > 0) c--; break;
     case FL_Right: if (c < 255) c++; break;
     case FL_Escape: which = initial; done = 1; return 1;
+    case FL_KP_Enter:
     case FL_Enter: done = 1; return 1;
     default: return 0;
     }
@@ -136,8 +137,8 @@ Fl_Color ColorMenu::run() {
     position(Fl::event_x_root()-(initial%8)*BOXSIZE-BOXSIZE/2-BORDER,
 	     Fl::event_y_root()-(initial/8)*BOXSIZE-BOXSIZE/2-BORDER);
   }
-  Fl::grab(*this);
   show();
+  Fl::grab(*this);
   done = 0;
   while (!done) Fl::wait();
   Fl::release();
@@ -150,5 +151,5 @@ Fl_Color fl_show_colormap(Fl_Color oldcol) {
 }
 
 //
-// End of "$Id: fl_show_colormap.cxx,v 1.5.2.4 2001/01/22 15:13:41 easysw Exp $".
+// End of "$Id: fl_show_colormap.cxx,v 1.5.2.4.2.6 2004/04/11 04:39:00 easysw Exp $".
 //
