@@ -1,9 +1,9 @@
 //
-// "$Id: fl_symbols.cxx,v 1.8 1999/02/22 22:09:17 mike Exp $"
+// "$Id: fl_symbols.cxx,v 1.8.2.3.2.11 2004/09/24 16:00:11 easysw Exp $"
 //
 // Symbol drawing code for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-1999 by Bill Spitzak and others.
+// Copyright 1998-2004 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,7 +20,7 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@easysw.com".
+// Please report all bugs and problems to "fltk-bugs@fltk.org".
 //
 
 // These are small graphics drawn by the normal label-drawing
@@ -34,7 +34,7 @@
 
 #include <FL/Fl.H>
 #include <FL/fl_draw.H>
-#include <string.h>
+#include "flstring.h"
 
 typedef struct {
   const char *name;
@@ -92,7 +92,7 @@ int fl_add_symbol(const char *name, void (*drawit)(Fl_Color), int scalable)
 int fl_return_arrow(int x,int y,int w,int h);
 
 // provided for back compatability:
-int fl_draw_symbol(const char *label,int x,int y,int w,int h,Fl_Color col) {
+int fl_draw_symbol(const char *label,int x,int y,int w,int h,Fl_Color col) {  
   const char *p = label;
   if (*p++ != '@') return 0;
   fl_init_symbols();
@@ -151,6 +151,8 @@ int fl_draw_symbol(const char *label,int x,int y,int w,int h,Fl_Color col) {
 
 #define BP fl_begin_polygon()
 #define EP fl_end_polygon()
+#define BCP fl_begin_complex_polygon()
+#define ECP fl_end_complex_polygon()
 #define BL fl_begin_line()
 #define EL fl_end_line()
 #define BC fl_begin_loop()
@@ -159,7 +161,7 @@ int fl_draw_symbol(const char *label,int x,int y,int w,int h,Fl_Color col) {
 
 //for the outline color
 static void set_outline_color(Fl_Color c) {
-  fl_color(fl_color_average(c, FL_BLACK, .5));
+  fl_color(fl_darker(c));
 }
 
 static void rectangle(double x,double y,double x2,double y2,Fl_Color col) {
@@ -171,6 +173,51 @@ static void rectangle(double x,double y,double x2,double y2,Fl_Color col) {
 
 /* The drawing routines */
 
+static void draw_fltk(Fl_Color col) 
+{
+  fl_color(col);
+  // F fill
+  BCP; vv(-2.0, -0.5); vv(-1.0, -0.5); vv(-1.0, -0.3); vv(-1.8, -0.3);
+  vv(-1.8, -0.1); vv(-1.2, -0.1); vv(-1.2, 0.1); vv(-1.8, 0.1);
+  vv(-1.8, 0.5); vv(-2.0, 0.5); ECP;
+  // L fill
+  BCP; vv(-1.0, -0.5); vv(-0.8, -0.5); vv(-0.8, 0.3); vv(0.0, 0.3);
+  vv(0.0, 0.5); vv(-1.0, 0.5); ECP;
+  // T outline
+  BCP; vv(-0.1, -0.5); vv(1.1, -0.5); vv(1.1, -0.3); vv(0.6, -0.3);
+  vv(0.6, 0.5); vv(0.4, 0.5); vv(0.4, -0.3); vv(-0.1, -0.3); ECP;
+  // K outline
+  BCP; vv(1.1, -0.5); vv(1.3, -0.5); vv(1.3, -0.15); vv(1.70, -0.5);
+  vv(2.0, -0.5); vv(1.43, 0.0); vv(2.0, 0.5); vv(1.70, 0.5);
+  vv(1.3, 0.15); vv(1.3, 0.5); vv(1.1, 0.5); ECP;
+  set_outline_color(col);
+  // F outline
+  BC; vv(-2.0, -0.5); vv(-1.0, -0.5); vv(-1.0, -0.3); vv(-1.8, -0.3);
+  vv(-1.8, -0.1); vv(-1.2, -0.1); vv(-1.2, 0.1); vv(-1.8, 0.1);
+  vv(-1.8, 0.5); vv(-2.0, 0.5); EC;
+  // L outline
+  BC; vv(-1.0, -0.5); vv(-0.8, -0.5); vv(-0.8, 0.3); vv(0.0, 0.3);
+  vv(0.0, 0.5); vv(-1.0, 0.5); EC;
+  // T outline
+  BC; vv(-0.1, -0.5); vv(1.1, -0.5); vv(1.1, -0.3); vv(0.6, -0.3);
+  vv(0.6, 0.5); vv(0.4, 0.5); vv(0.4, -0.3); vv(-0.1, -0.3); EC;
+  // K outline
+  BC; vv(1.1, -0.5); vv(1.3, -0.5); vv(1.3, -0.15); vv(1.70, -0.5);
+  vv(2.0, -0.5); vv(1.43, 0.0); vv(2.0, 0.5); vv(1.70, 0.5);
+  vv(1.3, 0.15); vv(1.3, 0.5); vv(1.1, 0.5); EC;
+}
+
+static void draw_search(Fl_Color col) 
+{
+  fl_color(col);
+  BP; vv(.4, .13); vv(1.0, .73); vv(.73, 1.0); vv(.13, .4); EP;
+  set_outline_color(col);
+  fl_line_style(FL_SOLID, 3, 0);
+  BC; fl_circle(-.2, -.2, .7); EC;
+  fl_line_style(FL_SOLID, 1, 0);
+  BC; vv(.4, .13); vv(1.0, .73); vv(.73, 1.0); vv(.13, .4); EC;
+}
+
 static void draw_arrow1(Fl_Color col)
 {
   fl_color(col);
@@ -358,30 +405,10 @@ static void fl_init_symbols(void) {
   fl_add_symbol("UpArrow",	draw_uparrow,		1);
   fl_add_symbol("DnArrow",	draw_downarrow,		1);
   fl_add_symbol("||",		draw_doublebar,		1);
-}
-
-////////////////////////////////////////////////////////////////
-
-#include <FL/Fl_Widget.H>
-
-// this is the labeltype function:
-extern void fl_normal_label(const Fl_Label*, int, int, int, int, Fl_Align);
-static void fl_symbol_label(
-    const Fl_Label* o, int x, int y, int w, int h, Fl_Align align)
-{
-  if (!fl_draw_symbol(o->value, x, y, w, h, (Fl_Color)o->color))
-    fl_normal_label(o, x, y, w, h, align);
-}
-
-Fl_Labeltype define_FL_SYMBOL_LABEL() {
-  Fl::set_labeltype(_FL_SYMBOL_LABEL, fl_symbol_label, 0);
-  return _FL_SYMBOL_LABEL;
-}
-
-void Fl::enable_symbols() {
-  Fl::set_labeltype(FL_NORMAL_LABEL, fl_symbol_label, 0);
+  fl_add_symbol("search",       draw_search,            1);
+  fl_add_symbol("FLTK",         draw_fltk,              1);
 }
 
 //
-// End of "$Id: fl_symbols.cxx,v 1.8 1999/02/22 22:09:17 mike Exp $".
+// End of "$Id: fl_symbols.cxx,v 1.8.2.3.2.11 2004/09/24 16:00:11 easysw Exp $".
 //
