@@ -1,9 +1,9 @@
 //
-// "$Id: fl_vertex.cxx,v 1.5.2.3 2001/01/22 15:13:41 easysw Exp $"
+// "$Id: fl_vertex.cxx,v 1.5.2.3.2.12 2004/08/31 22:00:49 matthiaswm Exp $"
 //
 // Portable drawing routines for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2004 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -24,8 +24,12 @@
 //
 
 // Portable drawing code for drawing arbitrary shapes with
-// simple 2D transformations.  See also fl_arc.C
+// simple 2D transformations.  See also fl_arc.cxx
 
+// matt: the Quartz implementation purposly doesn't use the Quartz matrix
+//       operations for reasons of compatibility and maintainability
+
+#include <config.h>
 #include <FL/fl_draw.H>
 #include <FL/x.H>
 #include <FL/math.h>
@@ -71,14 +75,22 @@ void fl_rotate(double d) {
   }
 }
 
-static XPoint *p = (XPoint *)0;
 // typedef what the x,y fields in a point are:
 #ifdef WIN32
 typedef int COORD_T;
+#  define XPOINT XPoint
+#elif defined(__APPLE_QUARTZ__)
+typedef float COORD_T;
+typedef struct { float x; float y; } QPoint;
+#  define XPOINT QPoint
+extern float fl_quartz_line_width_;
 #else
 typedef short COORD_T;
+#  define XPOINT XPoint
 #endif
 
+static XPOINT *p = (XPOINT *)0;
+
 static int p_size;
 static int n;
 static int what;
@@ -104,7 +116,7 @@ static void fl_transformed_vertex(COORD_T x, COORD_T y) {
   if (!n || x != p[n-1].x || y != p[n-1].y) {
     if (n >= p_size) {
       p_size = p ? 2*p_size : 16;
-      p = (XPoint *)realloc((void*)p, p_size*sizeof(*p));
+      p = (XPOINT*)realloc((void*)p, p_size*sizeof(*p));
     }
     p[n].x = x;
     p[n].y = y;
@@ -113,7 +125,11 @@ static void fl_transformed_vertex(COORD_T x, COORD_T y) {
 }
 
 void fl_transformed_vertex(double xf, double yf) {
-  fl_transformed_vertex(COORD_T(xf+.5), COORD_T(yf+.5));
+#ifdef __APPLE_QUARTZ__
+  fl_transformed_vertex(COORD_T(xf), COORD_T(yf));
+#else
+  fl_transformed_vertex(COORD_T(rint(xf)), COORD_T(rint(yf)));
+#endif
 }
 
 void fl_vertex(double x,double y) {
@@ -123,14 +139,38 @@ void fl_vertex(double x,double y) {
 void fl_end_points() {
 #ifdef WIN32
   for (int i=0; i<n; i++) SetPixel(fl_gc, p[i].x, p[i].y, fl_RGB());
+#elif defined(__APPLE_QD__)
+  for (int i=0; i<n; i++) { MoveTo(p[i].x, p[i].y); Line(0, 0); } 
+#elif defined(__APPLE_QUARTZ__)
+  if (fl_quartz_line_width_==1.0f) CGContextSetShouldAntialias(fl_gc, false);
+  for (int i=0; i<n; i++) { 
+    CGContextMoveToPoint(fl_gc, p[i].x, p[i].y);
+    CGContextAddLineToPoint(fl_gc, p[i].x, p[i].y);
+    CGContextStrokePath(fl_gc);
+  }
+  if (fl_quartz_line_width_==1.0f) CGContextSetShouldAntialias(fl_gc, false);
 #else
   if (n>1) XDrawPoints(fl_display, fl_window, fl_gc, p, n, 0);
 #endif
 }
 
 void fl_end_line() {
+  if (n < 2) {
+    fl_end_points();
+    return;
+  }
 #ifdef WIN32
   if (n>1) Polyline(fl_gc, p, n);
+#elif defined(__APPLE_QD__)
+  if (n<=1) return;
+  MoveTo(p[0].x, p[0].y);
+  for (int i=1; i<n; i++) LineTo(p[i].x, p[i].y);
+#elif defined(__APPLE_QUARTZ__)
+  if (n<=1) return;
+  CGContextMoveToPoint(fl_gc, p[0].x, p[0].y);
+  for (int i=1; i<n; i++)
+    CGContextAddLineToPoint(fl_gc, p[i].x, p[i].y);
+  CGContextStrokePath(fl_gc);
 #else
   if (n>1) XDrawLines(fl_display, fl_window, fl_gc, p, n, 0);
 #endif
@@ -148,11 +188,30 @@ void fl_end_loop() {
 
 void fl_end_polygon() {
   fixloop();
+  if (n < 3) {
+    fl_end_line();
+    return;
+  }
 #ifdef WIN32
   if (n>2) {
     SelectObject(fl_gc, fl_brush());
     Polygon(fl_gc, p, n);
   }
+#elif defined(__APPLE_QD__)
+  if (n<=1) return;
+  PolyHandle ph = OpenPoly();
+  MoveTo(p[0].x, p[0].y);
+  for (int i=1; i<n; i++) LineTo(p[i].x, p[i].y);
+  ClosePoly();
+  PaintPoly(ph);
+  KillPoly(ph);
+#elif defined(__APPLE_QUARTZ__)
+  if (n<=1) return;
+  CGContextMoveToPoint(fl_gc, p[0].x, p[0].y);
+  for (int i=1; i<n; i++) 
+    CGContextAddLineToPoint(fl_gc, p[i].x, p[i].y);
+  CGContextClosePath(fl_gc);
+  CGContextFillPath(fl_gc);
 #else
   if (n>2) XFillPolygon(fl_display, fl_window, fl_gc, p, n, Convex, 0);
 #endif
@@ -187,11 +246,30 @@ void fl_gap() {
 
 void fl_end_complex_polygon() {
   fl_gap();
+  if (n < 3) {
+    fl_end_line();
+    return;
+  }
 #ifdef WIN32
   if (n>2) {
     SelectObject(fl_gc, fl_brush());
     PolyPolygon(fl_gc, p, counts, numcount);
   }
+#elif defined(__APPLE_QD__)
+  if (n<=1) return;
+  PolyHandle ph = OpenPoly();
+  MoveTo(p[0].x, p[0].y);
+  for (int i=1; i<n; i++) LineTo(p[i].x, p[i].y);
+  ClosePoly();
+  PaintPoly(ph);
+  KillPoly(ph);
+#elif defined(__APPLE_QUARTZ__)
+  if (n<=1) return;
+  CGContextMoveToPoint(fl_gc, p[0].x, p[0].y);
+  for (int i=1; i<n; i++)
+    CGContextAddLineToPoint(fl_gc, p[i].x, p[i].y);
+  CGContextClosePath(fl_gc);
+  CGContextFillPath(fl_gc);
 #else
   if (n>2) XFillPolygon(fl_display, fl_window, fl_gc, p, n, 0, 0);
 #endif
@@ -206,16 +284,23 @@ void fl_circle(double x, double y,double r) {
   double yt = fl_transform_y(x,y);
   double rx = r * (m.c ? sqrt(m.a*m.a+m.c*m.c) : fabs(m.a));
   double ry = r * (m.b ? sqrt(m.b*m.b+m.d*m.d) : fabs(m.d));
-  int llx = int(xt-rx+.5);
-  int w = int(xt+rx+.5)-llx;
-  int lly = int(yt-ry+.5);
-  int h = int(yt+ry+.5)-lly;
+  int llx = (int)rint(xt-rx);
+  int w = (int)rint(xt+rx)-llx;
+  int lly = (int)rint(yt-ry);
+  int h = (int)rint(yt+ry)-lly;
 #ifdef WIN32
   if (what==POLYGON) {
     SelectObject(fl_gc, fl_brush());
     Pie(fl_gc, llx, lly, llx+w, lly+h, 0,0, 0,0); 
   } else
     Arc(fl_gc, llx, lly, llx+w, lly+h, 0,0, 0,0); 
+#elif defined(__APPLE_QD__)
+  Rect rt; rt.left=llx; rt.right=llx+w; rt.top=lly; rt.bottom=lly+h;
+  (what == POLYGON ? PaintOval : FrameOval)(&rt);
+#elif defined(__APPLE_QUARTZ__)
+  // Quartz warning : circle won't scale to current matrix!
+  CGContextAddArc(fl_gc, xt, yt, (w+h)*0.25f, 0, 2.0f*M_PI, 1);
+  (what == POLYGON ? CGContextFillPath : CGContextStrokePath)(fl_gc);
 #else
   (what == POLYGON ? XFillArc : XDrawArc)
     (fl_display, fl_window, fl_gc, llx, lly, w, h, 0, 360*64);
@@ -223,5 +308,5 @@ void fl_circle(double x, double y,double r) {
 }
 
 //
-// End of "$Id: fl_vertex.cxx,v 1.5.2.3 2001/01/22 15:13:41 easysw Exp $".
+// End of "$Id: fl_vertex.cxx,v 1.5.2.3.2.12 2004/08/31 22:00:49 matthiaswm Exp $".
 //
