@@ -1,9 +1,9 @@
 //
-// "$Id: forms_compatability.cxx,v 1.5.2.3 2001/01/22 15:13:41 easysw Exp $"
+// "$Id: forms_compatability.cxx,v 1.5.2.3.2.5 2004/04/11 04:39:00 easysw Exp $"
 //
 // Forms compatibility functions for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2004 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -60,9 +60,9 @@ void Fl_Group::forms_end() {
     int Y = o->h();
     Fl_Widget*const* a = array();
     for (int i=children(); i--;) {
-      Fl_Widget* o = *a++;
-      int newy = Y-o->y()-o->h();
-      o->y(newy);
+      Fl_Widget* ow = *a++;
+      int newy = Y-ow->y()-ow->h();
+      ow->y(newy);
     }
   }
   end();
@@ -89,8 +89,6 @@ char fl_modal_next; // set by fl_freeze_forms()
 
 void fl_show_form(Fl_Window *f,int place,int b,const char *n) {
 
-  Fl::enable_symbols();
-
   f->label(n);
   if (!b) f->clear_border();
   if (fl_modal_next || b==FL_TRANSIENT) {f->set_modal(); fl_modal_next = 0;}
@@ -204,5 +202,5 @@ char *fl_show_simple_input(const char *str1, const char *defstr) {
 }
 
 //
-// End of "$Id: forms_compatability.cxx,v 1.5.2.3 2001/01/22 15:13:41 easysw Exp $".
+// End of "$Id: forms_compatability.cxx,v 1.5.2.3.2.5 2004/04/11 04:39:00 easysw Exp $".
 //
