@@ -1,9 +1,9 @@
 //
-// "$Id: forms_timer.cxx,v 1.4.2.3 2001/01/22 15:13:41 easysw Exp $"
+// "$Id: forms_timer.cxx,v 1.4.2.3.2.7 2004/07/27 16:02:21 easysw Exp $"
 //
 // Forms timer object for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2004 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -31,10 +31,15 @@
 #include <FL/Fl_Timer.H>
 #include <FL/fl_draw.H>
 #ifdef WIN32
-# include <sys/types.h> 
-# include <sys/timeb.h>
+#  ifdef __MWERKS__
+#    include <time.h>
+#  else
+#    include <sys/types.h> 
+#    include <sys/timeb.h>
+#  endif
 #else
-# include <sys/time.h>
+#  include <time.h>
+#  include <sys/time.h>
 #endif
 #include <stdio.h>
 
@@ -42,10 +47,17 @@
 
 void fl_gettime(long* sec, long* usec) {
 #ifdef WIN32
+# ifdef __MWERKS__
+  time_t localTime = time(NULL);
+  struct tm *now = localtime(&localTime);
+  *sec = now->tm_sec + 60*now->tm_min + 3600*now->tm_hour + 24*3600*now->tm_yday;
+  *usec = 0;
+# else
   struct timeb tp;
   ftime(&tp);
   *sec = tp.time;
   *usec = tp.millitm * 1000;
+# endif
 #else
   struct timeval tp;
   struct timezone tzp;
@@ -99,6 +111,7 @@ void Fl_Timer::step() {
       redraw();
       Fl::add_timeout(FL_TIMER_BLINKRATE, stepcb, this);
     }
+    set_changed();
     do_callback();
   } else {
     if (type() == FL_VALUE_TIMER) redraw();
@@ -115,8 +128,8 @@ Fl_Timer::~Fl_Timer() {
   Fl::remove_timeout(stepcb, this);
 }
 
-Fl_Timer::Fl_Timer(uchar t, int x, int y, int w, int h, const char* l)
-: Fl_Widget(x, y, w, h, l) {
+Fl_Timer::Fl_Timer(uchar t, int X, int Y, int W, int H, const char* l)
+: Fl_Widget(X, Y, W, H, l) {
   box(FL_DOWN_BOX);
   selection_color(FL_RED);
   delay = 0;
@@ -150,5 +163,5 @@ void Fl_Timer::suspended(char d) {
 }
 
 //
-// End of "$Id: forms_timer.cxx,v 1.4.2.3 2001/01/22 15:13:41 easysw Exp $".
+// End of "$Id: forms_timer.cxx,v 1.4.2.3.2.7 2004/07/27 16:02:21 easysw Exp $".
 //
