@@ -1,9 +1,9 @@
 //
-// "$Id: gl_start.cxx,v 1.6.2.5 2001/03/14 17:20:02 spitzak Exp $"
+// "$Id: gl_start.cxx,v 1.6.2.5.2.11 2004/09/09 21:34:48 matthiaswm Exp $"
 //
 // OpenGL context routines for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2004 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,7 +20,7 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@easysw.com".
+// Please report all bugs and problems to "fltk-bugs@fltk.org".
 //
 
 // You MUST use gl_visual() to select the default visual before doing
@@ -44,7 +44,7 @@
 #include <FL/fl_draw.H>
 #include "Fl_Gl_Choice.H"
 
-extern int fl_clip_state_number; // in fl_rect.C
+extern int fl_clip_state_number; // in fl_rect.cxx
 
 static GLContext context;
 static int clip_state_number=-1;
@@ -54,19 +54,29 @@ static int pw, ph;
 static Fl_Gl_Choice* gl_choice;
 #endif
 
-Region XRectangleRegion(int x, int y, int w, int h); // in fl_rect.C
+#ifdef __APPLE__
+static Fl_Gl_Choice* gl_choice;
+#endif
+
+Fl_Region XRectangleRegion(int x, int y, int w, int h); // in fl_rect.cxx
 
 void gl_start() {
   if (!context) {
 #ifdef WIN32
     if (!gl_choice) Fl::gl_visual(0);
     context = fl_create_gl_context(Fl_Window::current(), gl_choice);
+#elif defined(__APPLE_QD__)
+    // \todo Mac : We need to check the code and verify it with Apple Sample code. The 'shiny'-test should at least work with the software OpenGL emulator
+    context = fl_create_gl_context(Fl_Window::current(), gl_choice);
+#elif defined(__APPLE_QUARTZ__)
+    // warning: the Quartz version should probably use Core GL (CGL) instead of AGL
+    context = fl_create_gl_context(Fl_Window::current(), gl_choice);
 #else
     context = fl_create_gl_context(fl_visual);
 #endif
   }
   fl_set_gl_context(Fl_Window::current(), context);
-#ifndef WIN32
+#if !defined(WIN32) && !defined(__APPLE__)
   glXWaitX();
 #endif
   if (pw != Fl_Window::current()->w() || ph != Fl_Window::current()->h()) {
@@ -93,7 +103,7 @@ void gl_start() {
 
 void gl_finish() {
   glFlush();
-#ifndef WIN32
+#if !defined(WIN32) && !defined(__APPLE__)
   glXWaitGL();
 #endif
 }
@@ -103,6 +113,11 @@ int Fl::gl_visual(int mode, int *alist) {
   if (!c) return 0;
 #ifdef WIN32
   gl_choice = c;
+#elif defined(__APPLE_QD__)
+  gl_choice = c;
+#elif defined(__APPLE_QUARTZ__)
+  // warning: the Quartz version should probably use Core GL (CGL) instead of AGL
+  gl_choice = c;
 #else
   fl_visual = c->vis;
   fl_colormap = c->colormap;
@@ -113,5 +128,5 @@ int Fl::gl_visual(int mode, int *alist) {
 #endif
 
 //
-// End of "$Id: gl_start.cxx,v 1.6.2.5 2001/03/14 17:20:02 spitzak Exp $".
+// End of "$Id: gl_start.cxx,v 1.6.2.5.2.11 2004/09/09 21:34:48 matthiaswm Exp $".
 //
