@@ -1,9 +1,9 @@
 //
-// "$Id: glut_compatability.cxx,v 1.4.2.5 2001/01/22 15:13:41 easysw Exp $"
+// "$Id: glut_compatability.cxx,v 1.4.2.5.2.7 2004/04/11 04:39:00 easysw Exp $"
 //
 // GLUT emulation routines for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2004 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -47,6 +47,7 @@ void (*glut_menustate_function)(int);
 void (*glut_menustatus_function)(int,int,int);
 
 static void default_reshape(int w, int h) {glViewport(0,0,w,h);}
+static void default_display() {}
 
 void Fl_Glut_Window::make_current() {
   glut_window = this;
@@ -82,6 +83,7 @@ int Fl_Glut_Window::handle(int event) {
   switch (event) {
 
   case FL_PUSH:
+    if (keyboard || special) Fl::focus(this);
     button = Fl::event_button()-1;
     if (button<0) button = 0;
     if (button>2) button = 2;
@@ -91,6 +93,13 @@ int Fl_Glut_Window::handle(int event) {
     if (motion) return 1;
     break;
 
+  case FL_MOUSEWHEEL:
+    button = Fl::event_dy();
+    while (button < 0) {mouse(3,GLUT_DOWN,ex,ey); ++button;}
+    while (button > 0) {mouse(4,GLUT_DOWN,ex,ey); --button;}
+    return 1;
+    break;
+
   case FL_RELEASE:
     for (button = 0; button < 3; button++) if (mouse_down & (1<<button)) {
       if (mouse) mouse(button,GLUT_UP,ex,ey);
@@ -120,15 +129,18 @@ int Fl_Glut_Window::handle(int event) {
     if (keyboard || special) return 1;
     break;
 
+  case FL_SHORTCUT:
+    if (!keyboard && !special) break;
+
   case FL_KEYBOARD:
     if (Fl::event_text()[0]) {
       if (keyboard) {keyboard(Fl::event_text()[0],ex,ey); return 1;}
       break;
     } else {
       if (special) {
-	int i = Fl::event_key();
-	if (i > FL_F && i <= FL_F_Last) i = i-FL_F;
-	special(i,ex,ey);
+	int k = Fl::event_key();
+	if (k > FL_F && k <= FL_F_Last) k -= FL_F;
+	special(k,ex,ey);
 	return 1;
       }
       break;
@@ -153,6 +165,8 @@ void Fl_Glut_Window::_init() {
   windows[number] = this;
   menu[0] = menu[1] = menu[2] = 0;
   reshape = default_reshape;
+  display = default_display;
+  overlaydisplay = default_display;
   keyboard = 0;
   mouse = 0;
   motion = 0;
@@ -164,11 +178,11 @@ void Fl_Glut_Window::_init() {
   mode(glut_mode);
 }
 
-Fl_Glut_Window::Fl_Glut_Window(int w, int h, const char *t) :
-  Fl_Gl_Window(w,h,t) {_init();}
+Fl_Glut_Window::Fl_Glut_Window(int W, int H, const char *t) :
+  Fl_Gl_Window(W,H,t) {_init();}
 
-Fl_Glut_Window::Fl_Glut_Window(int x,int y,int w,int h, const char *t) :
-  Fl_Gl_Window(x,y,w,h,t) {_init();}
+Fl_Glut_Window::Fl_Glut_Window(int X,int Y,int W,int H, const char *t) :
+  Fl_Gl_Window(X,Y,W,H,t) {_init();}
 
 static int initargc;
 static char **initargv;
@@ -399,5 +413,5 @@ int glutLayerGet(GLenum type) {
 #endif
 
 //
-// End of "$Id: glut_compatability.cxx,v 1.4.2.5 2001/01/22 15:13:41 easysw Exp $".
+// End of "$Id: glut_compatability.cxx,v 1.4.2.5.2.7 2004/04/11 04:39:00 easysw Exp $".
 //
