@@ -1,9 +1,9 @@
 //
-// "$Id: CubeMain.cxx,v 1.2.2.5 2001/03/15 22:39:56 easysw Exp $"
+// "$Id: CubeMain.cxx,v 1.2.2.5.2.4 2004/04/11 04:39:00 easysw Exp $"
 //
 // CubeView class definitions for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2004 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -28,7 +28,7 @@
 #include "CubeViewUI.h"
 
 int
-main(int /*argc*/, char ** /*argv*/) {
+main(int argc, char **argv) {
 
     CubeViewUI *cvui=new CubeViewUI;
     
@@ -36,12 +36,12 @@ main(int /*argc*/, char ** /*argv*/) {
     
     Fl::visual(FL_DOUBLE|FL_INDEX);
 
-    cvui->show();
+    cvui->show(argc, argv);
     
     return Fl::run();
 }
 
 
 //
-// End of "$Id: CubeMain.cxx,v 1.2.2.5 2001/03/15 22:39:56 easysw Exp $".
+// End of "$Id: CubeMain.cxx,v 1.2.2.5.2.4 2004/04/11 04:39:00 easysw Exp $".
 //
