@@ -1,9 +1,9 @@
 //
-// "$Id: boxtype.cxx,v 1.4.2.3 2001/01/22 15:13:41 easysw Exp $"
+// "$Id: boxtype.cxx,v 1.4.2.3.2.5 2004/04/11 04:39:00 easysw Exp $"
 //
 // Boxtype test program for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2004 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -30,9 +30,9 @@
 #include <FL/Fl_Box.H>
 
 int N = 0;
-#define W 150
+#define W 200
 #define H 50
-#define ROWS 8
+#define ROWS 10
 
 Fl_Window *window;
 
@@ -82,12 +82,19 @@ int main(int argc, char ** argv) {
   bt("FL_ROUND_DOWN_BOX",FL_ROUND_DOWN_BOX);
   bt("FL_DIAMOND_UP_BOX",FL_DIAMOND_UP_BOX);
   bt("FL_DIAMOND_DOWN_BOX",FL_DIAMOND_DOWN_BOX);
+  bt("FL_PLASTIC_UP_BOX",FL_PLASTIC_UP_BOX);
+  bt("FL_PLASTIC_DOWN_BOX",FL_PLASTIC_DOWN_BOX);
+  bt("FL_PLASTIC_UP_FRAME",FL_PLASTIC_UP_FRAME);
+  bt("FL_PLASTIC_DOWN_FRAME",FL_PLASTIC_DOWN_FRAME);
+  bt("FL_PLASTIC_THIN_UP_BOX",FL_PLASTIC_THIN_UP_BOX);
+  bt("FL_PLASTIC_THIN_DOWN_BOX",FL_PLASTIC_THIN_DOWN_BOX);
   window->resizable(window);
   window->end();
-  window->show(argc,argv);
+//  window->show(argc,argv);
+  window->show();
   return Fl::run();
 }
 
 //
-// End of "$Id: boxtype.cxx,v 1.4.2.3 2001/01/22 15:13:41 easysw Exp $".
+// End of "$Id: boxtype.cxx,v 1.4.2.3.2.5 2004/04/11 04:39:00 easysw Exp $".
 //
