@@ -1,9 +1,9 @@
 //
-// "$Id: browser.cxx,v 1.5.2.6 2001/01/22 15:13:41 easysw Exp $"
+// "$Id: browser.cxx,v 1.5.2.6.2.7 2004/07/26 20:52:52 easysw Exp $"
 //
 // Browser test program for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2004 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -63,7 +63,8 @@ That was a blank line above this.
 #include <FL/Fl_Select_Browser.H>
 #include <FL/Fl_Double_Window.H>
 #include <FL/Fl_Button.H>
-#include <FL/Fl_Input.H>
+#include <FL/Fl_Int_Input.H>
+#include <FL/fl_ask.H>
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
@@ -73,8 +74,9 @@ Fl_Select_Browser *browser;
 Fl_Button	*top,
 		*bottom,
 		*middle,
-		*visible;
-Fl_Input	*field;
+		*visible,
+		*swap;
+Fl_Int_Input	*field;
 
 void b_cb(Fl_Widget* o, void*) {
   printf("callback, selection = %d, event_clicks = %d\n",
@@ -84,6 +86,12 @@ void b_cb(Fl_Widget* o, void*) {
 void show_cb(Fl_Widget *o, void *) {
   int line = atoi(field->value());
 
+  if (!line) {
+    fl_alert("Please enter a number in the text field\n"
+             "before clicking on the buttons.");
+    return;
+  }
+
   if (o == top)
     browser->topline(line);
   else if (o == bottom)
@@ -94,6 +102,19 @@ void show_cb(Fl_Widget *o, void *) {
     browser->make_visible(line);
 }
 
+void swap_cb(Fl_Widget *o, void *) {
+  int a = -1, b = -1;
+  for ( int t=0; t<browser->size(); t++ ) {	// find two selected items
+    if ( browser->selected(t) ) {
+      if ( a < 0 )
+	{ a = t; }
+      else 
+	{ b = t; break; }
+    }
+  }
+  browser->swap(a, b);				// swap them
+}
+
 int main(int argc, char **argv) {
   int i;
   if (!Fl::args(argc,argv,i)) Fl::fatal(Fl::help);
@@ -107,32 +128,53 @@ int main(int argc, char **argv) {
   // browser->scrollbar_right();
   //browser->has_scrollbar(Fl_Browser::BOTH_ALWAYS);
   if (!browser->load(fname)) {
+#ifdef _MSC_VER
+    // if 'browser' was started from the VisualC environment in Win32, 
+    // the current directory is set to the environment itself, 
+    // so we need to correct the browser file path
+    int done = 1;
+    if ( i == argc ) 
+    {
+      fname = "../test/browser.cxx";
+      done = browser->load(fname);
+    }
+    if ( !done )
+    {
+      printf("Can't load %s, %s\n", fname, strerror(errno));
+      exit(1);
+    }
+#else
     printf("Can't load %s, %s\n", fname, strerror(errno));
     exit(1);
+#endif
   }
   browser->position(0);
 
-  field = new Fl_Input(50, 350, 350, 25, "Line #:");
+  field = new Fl_Int_Input(50, 350, 350, 25, "Line #:");
   field->callback(show_cb);
 
-  top = new Fl_Button(0, 375, 100, 25, "Top");
+  top = new Fl_Button(0, 375, 80, 25, "Top");
   top->callback(show_cb);
 
-  bottom = new Fl_Button(100, 375, 100, 25, "Bottom");
+  bottom = new Fl_Button(80, 375, 80, 25, "Bottom");
   bottom->callback(show_cb);
 
-  middle = new Fl_Button(200, 375, 100, 25, "Middle");
+  middle = new Fl_Button(160, 375, 80, 25, "Middle");
   middle->callback(show_cb);
 
-  visible = new Fl_Button(300, 375, 100, 25, "Make Vis.");
+  visible = new Fl_Button(240, 375, 80, 25, "Make Vis.");
   visible->callback(show_cb);
 
+  swap = new Fl_Button(320, 375, 80, 25, "Swap");
+  swap->callback(swap_cb);
+  swap->tooltip("Swaps two selected lines\n(Use CTRL-click to select two lines)");
+
   window.resizable(browser);
   window.show(argc,argv);
   return Fl::run();
 }
 
 //
-// End of "$Id: browser.cxx,v 1.5.2.6 2001/01/22 15:13:41 easysw Exp $".
+// End of "$Id: browser.cxx,v 1.5.2.6.2.7 2004/07/26 20:52:52 easysw Exp $".
 //
 
