@@ -1,12 +1,12 @@
 //
-// "$Id: checkers.cxx,v 1.9.2.7 2001/04/13 19:07:40 easysw Exp $"
+// "$Id: checkers.cxx,v 1.9.2.7.2.5 2004/04/11 04:39:00 easysw Exp $"
 //
 // Checkers game for the Fast Light Tool Kit (FLTK).
 //
 // Hours of fun: the FLTK checkers game!
 // Based on a very old algorithim, but it still works!
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2004 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -56,11 +56,10 @@ const char* copyright =
 #define FLTK
 //#define VT100
 
-#include <string.h>
+#include "../src/flstring.h"
 #include <stdlib.h>
 #include <stdio.h>
 #include <stdarg.h>
-#include <ctype.h>
 #include <time.h>
 
 ////////////////////////////////////////////////////////////////
@@ -185,7 +184,7 @@ char check(int target,int direction) {
   int src = target+direction;
   if (tb[src] == FRIENDKING);
   else if (direction < 0 || tb[src] != FRIEND) return(0);
-  piece a = tb[target]; piece b = tb[src];
+  piece aa = tb[target]; piece bb = tb[src];
   tb[target] = EMPTY; tb[src] = EMPTY;
   int safe =
     (tb[src-4]&FRIEND && tb[src-8]&ENEMY
@@ -196,7 +195,7 @@ char check(int target,int direction) {
      ||tb[src+5]&FRIEND && tb[src+10]==ENEMYKING
      ||tb[dst+4]==ENEMYKING && !tb[dst-4]
      ||tb[dst+5]==ENEMYKING && !tb[dst-5]);
-  tb[target] = a; tb[src] = b;
+  tb[target] = aa; tb[src] = bb;
   return(safe);
 }
 
@@ -1057,15 +1056,6 @@ void Board::animate(node* move, int backwards) {
 
 int busy; // causes pop-up abort menu
 
-void message(const char* m, ...) {
-  char buffer[2048];
-  va_list a;
-  va_start(a,m);
-  vsprintf(buffer, m, a);
-  va_end(a);
-  fl_message(buffer);
-}
-
 void Board::computer_move(int help) {
   if (!playing) return;
   cursor(FL_CURSOR_WAIT);
@@ -1075,7 +1065,7 @@ void Board::computer_move(int help) {
   busy = 0;
   if (move) {
     if (!help && move->value <= -30000) {
-      message("%s resigns", move->who ? "White" : "Black");
+      fl_message("%s resigns", move->who ? "White" : "Black");
       playing = autoplay = 0;
       cursor(FL_CURSOR_DEFAULT);
       return;
@@ -1085,7 +1075,7 @@ void Board::computer_move(int help) {
   }
   expandnode(root);
   if (!root->son) {
-    message("%s has no move", root->who ? "Black" : "White");
+    fl_message("%s has no move", root->who ? "Black" : "White");
     playing = autoplay = 0;
   }
   if (!autoplay) cursor(FL_CURSOR_DEFAULT);
@@ -1365,5 +1355,5 @@ int main(int argc, char **argv) {
 }
 
 //
-// End of "$Id: checkers.cxx,v 1.9.2.7 2001/04/13 19:07:40 easysw Exp $".
+// End of "$Id: checkers.cxx,v 1.9.2.7.2.5 2004/04/11 04:39:00 easysw Exp $".
 //
