@@ -1,9 +1,9 @@
 //
-// "$Id: fonts.cxx,v 1.5.2.3 2001/01/22 15:13:41 easysw Exp $"
+// "$Id: fonts.cxx,v 1.5.2.3.2.5 2004/09/09 00:55:41 matthiaswm Exp $"
 //
 // Font demo program for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2004 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -131,8 +131,13 @@ void create_the_forms() {
 #include <FL/fl_ask.H>
 
 int main(int argc, char **argv) {
+  Fl::scheme(NULL);
   create_the_forms();
+#ifdef __APPLE__
+  int i = 0;
+#else
   int i = fl_choice("Which fonts:","-*","iso8859","All");
+#endif
   int k = Fl::set_fonts(i ? (i>1 ? "*" : 0) : "-*");
   for (i = 0; i < k; i++) {
     int t; const char *name = Fl::get_font_name((Fl_Font)i,&t);
@@ -164,5 +169,5 @@ int main(int argc, char **argv) {
 }
 
 //
-// End of "$Id: fonts.cxx,v 1.5.2.3 2001/01/22 15:13:41 easysw Exp $".
+// End of "$Id: fonts.cxx,v 1.5.2.3.2.5 2004/09/09 00:55:41 matthiaswm Exp $".
 //
