@@ -1,5 +1,5 @@
 //
-// "$Id: forms.cxx,v 1.4.2.4 2001/04/13 19:07:40 easysw Exp $"
+// "$Id: forms.cxx,v 1.4.2.4.2.5 2004/04/11 04:39:01 easysw Exp $"
 //
 // Another forms demo for the Fast Light Tool Kit (FLTK).
 //
@@ -9,7 +9,7 @@
 // This demo show the different boxtypes. Note that some
 // boxtypes are not appropriate for some objects
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2004 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -61,6 +61,8 @@ static VN_struct btypes[]=
    {FL_ROUNDED3D_DOWNBOX,"rounded3d downbox"},
    {FL_OVAL3D_UPBOX,"oval3d upbox"},
    {FL_OVAL3D_DOWNBOX,"oval3d downbox"},
+   {FL_PLASTIC_UP_BOX,"plastic upbox"},
+   {FL_PLASTIC_DOWN_BOX,"plastic downbox"},
    /* sentinel */
    {-1}
 };
@@ -229,5 +231,5 @@ main (int argc, char *argv[])
 }
 
 //
-// End of "$Id: forms.cxx,v 1.4.2.4 2001/04/13 19:07:40 easysw Exp $".
+// End of "$Id: forms.cxx,v 1.4.2.4.2.5 2004/04/11 04:39:01 easysw Exp $".
 //
