@@ -1,9 +1,9 @@
 //
-// "$Id: mandelbrot.cxx,v 1.8.2.4 2001/04/13 19:07:40 easysw Exp $"
+// "$Id: mandelbrot.cxx,v 1.8.2.4.2.4 2004/04/11 04:39:01 easysw Exp $"
 //
 // Mandelbrot set demo for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2004 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -23,7 +23,7 @@
 // Please report all bugs and problems to "fltk-bugs@fltk.org".
 //
 
-#include "mandelbrot_ui.cxx"
+#include "mandelbrot_ui.h"
 #include <FL/fl_draw.H>
 #include <stdio.h>
 #include <stdlib.h>
@@ -197,5 +197,5 @@ void Drawing_Area::resize(int XX,int YY,int WW,int HH) {
 }
 
 //
-// End of "$Id: mandelbrot.cxx,v 1.8.2.4 2001/04/13 19:07:40 easysw Exp $".
+// End of "$Id: mandelbrot.cxx,v 1.8.2.4.2.4 2004/04/11 04:39:01 easysw Exp $".
 //
