@@ -1,12 +1,12 @@
 //
-// "$Id: navigation.cxx,v 1.5.2.3 2001/01/22 15:13:41 easysw Exp $"
+// "$Id: navigation.cxx,v 1.5.2.3.2.4 2004/04/11 04:39:01 easysw Exp $"
 //
 // Navigation test program for the Fast Light Tool Kit (FLTK).
 //
 // Silly test of navigation keys. This is not a recommended method of
 // laying out your panels!
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2004 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -61,10 +61,10 @@ int main(int argc, char **argv) {
     if (n < window.children()) continue;
     window.insert(*(new Fl_Input(x,y,w,h)),j);
   }
-  window.show();
+  window.show(argc, argv);
   return Fl::run();
 }
 
 //
-// End of "$Id: navigation.cxx,v 1.5.2.3 2001/01/22 15:13:41 easysw Exp $".
+// End of "$Id: navigation.cxx,v 1.5.2.3.2.4 2004/04/11 04:39:01 easysw Exp $".
 //
