@@ -1,9 +1,9 @@
 //
-// "$Id: output.cxx,v 1.4.2.3 2001/01/22 15:13:41 easysw Exp $"
+// "$Id: output.cxx,v 1.4.2.3.2.4 2004/04/11 04:39:01 easysw Exp $"
 //
 // Output test program for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2004 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -85,11 +85,13 @@ int main(int argc, char **argv) {
   text2 = new Fl_Multiline_Output(100,150,200,100,"Fl_Multiline_Output");
   text2->value(input->value());
   text2->align(FL_ALIGN_BOTTOM);
+  text2->tooltip("This is an Fl_Multiline_Output widget.");
   window->resizable(text2);
 
   text = new Fl_Output(100,280,200,30,"Fl_Output");
   text->value(input->value());
   text->align(FL_ALIGN_BOTTOM);
+  text->tooltip("This is an Fl_Output widget.");
 
   window->forms_end();
   window->show(argc,argv);
@@ -97,5 +99,5 @@ int main(int argc, char **argv) {
 }
 
 //
-// End of "$Id: output.cxx,v 1.4.2.3 2001/01/22 15:13:41 easysw Exp $".
+// End of "$Id: output.cxx,v 1.4.2.3.2.4 2004/04/11 04:39:01 easysw Exp $".
 //
