@@ -1,9 +1,9 @@
 //
-// "$Id: pixmap.cxx,v 1.4.2.3 2001/01/22 15:13:41 easysw Exp $"
+// "$Id: pixmap.cxx,v 1.4.2.3.2.5 2004/04/11 04:39:01 easysw Exp $"
 //
 // Pixmap label test program for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2004 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -33,7 +33,7 @@
 
 #include <FL/Fl_Toggle_Button.H>
 
-Fl_Toggle_Button *leftb,*rightb,*topb,*bottomb,*insideb;
+Fl_Toggle_Button *leftb,*rightb,*topb,*bottomb,*insideb,*overb,*inactb;
 Fl_Button *b;
 Fl_Window *w;
 
@@ -44,7 +44,10 @@ void button_cb(Fl_Widget *,void *) {
   if (topb->value()) i |= FL_ALIGN_TOP;
   if (bottomb->value()) i |= FL_ALIGN_BOTTOM;
   if (insideb->value()) i |= FL_ALIGN_INSIDE;
+  if (overb->value()) i |= FL_ALIGN_TEXT_OVER_IMAGE;
   b->align(i);
+  if (inactb->value()) b->deactivate();
+  else b->activate();
   w->redraw();
 }
 
@@ -56,29 +59,35 @@ int arg(int, char **argv, int &i) {
 
 #include <FL/Fl_Multi_Label.H>
 
-Fl_Multi_Label multi = {
-  0, "This is the text", 0, FL_NORMAL_LABEL
-};
-
 int main(int argc, char **argv) {
   int i = 1;
   if (Fl::args(argc,argv,i,arg) < argc)
     Fl::fatal(" -8 # : use default visual\n%s\n",Fl::help);
 
   Fl_Window window(400,400); ::w = &window;
-  Fl_Button b(140,160,120,120,0); ::b = &b;
-  (new Fl_Pixmap(porsche_xpm))->label(&b);
-  multi.labela = b.label(); multi.typea = b.labeltype(); multi.label(&b);
-  leftb = new Fl_Toggle_Button(50,75,50,25,"left");
+  Fl_Button b(140,160,120,120,"Pixmap"); ::b = &b;
+  Fl_Pixmap *pixmap = new Fl_Pixmap(porsche_xpm);
+  Fl_Pixmap *depixmap;
+  depixmap = (Fl_Pixmap *)pixmap->copy();
+  depixmap->inactive();
+
+  b.image(pixmap);
+  b.deimage(depixmap);
+
+  leftb = new Fl_Toggle_Button(25,50,50,25,"left");
   leftb->callback(button_cb);
-  rightb = new Fl_Toggle_Button(100,75,50,25,"right");
+  rightb = new Fl_Toggle_Button(75,50,50,25,"right");
   rightb->callback(button_cb);
-  topb = new Fl_Toggle_Button(150,75,50,25,"top");
+  topb = new Fl_Toggle_Button(125,50,50,25,"top");
   topb->callback(button_cb);
-  bottomb = new Fl_Toggle_Button(200,75,50,25,"bottom");
+  bottomb = new Fl_Toggle_Button(175,50,50,25,"bottom");
   bottomb->callback(button_cb);
-  insideb = new Fl_Toggle_Button(250,75,50,25,"inside");
+  insideb = new Fl_Toggle_Button(225,50,50,25,"inside");
   insideb->callback(button_cb);
+  overb = new Fl_Toggle_Button(25,75,100,25,"text over");
+  overb->callback(button_cb);
+  inactb = new Fl_Toggle_Button(125,75,100,25,"inactive");
+  inactb->callback(button_cb);
   if (!dvisual) Fl::visual(FL_RGB);
   window.resizable(window);
   window.end();
@@ -87,5 +96,5 @@ int main(int argc, char **argv) {
 }
 
 //
-// End of "$Id: pixmap.cxx,v 1.4.2.3 2001/01/22 15:13:41 easysw Exp $".
+// End of "$Id: pixmap.cxx,v 1.4.2.3.2.5 2004/04/11 04:39:01 easysw Exp $".
 //
