@@ -1,9 +1,9 @@
 //
-// "$Id: tile.cxx,v 1.4.2.3 2001/01/22 15:13:41 easysw Exp $"
+// "$Id: tile.cxx,v 1.4.2.3.2.4 2004/04/11 04:39:01 easysw Exp $"
 //
 // Tile test program for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2004 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -40,7 +40,7 @@ int main(int argc, char** argv) {
   box0.align(FL_ALIGN_CLIP);
   Fl_Window w1(150,0,150,150,"1");
   w1.box(FL_NO_BOX);
-  Fl_Box box1(0,0,150,150,"1\nThis is a\nchild\nX window");
+  Fl_Box box1(0,0,150,150,"1\nThis is a\nchild\nwindow");
   box1.box(FL_DOWN_BOX);
   box1.color(19);
   box1.labelsize(18);
@@ -86,5 +86,5 @@ int main(int argc, char** argv) {
 }
 
 //
-// End of "$Id: tile.cxx,v 1.4.2.3 2001/01/22 15:13:41 easysw Exp $".
+// End of "$Id: tile.cxx,v 1.4.2.3.2.4 2004/04/11 04:39:01 easysw Exp $".
 //
